--[[
MapsterEnhanced - GuildPositions (Sindragosa)

Based on Cartographer
Credits: ckknight
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "GuildPositions"
local GuildPositions = Mapster:NewModule(MODNAME, "AceEvent-3.0")
local Roster = LibStub("Roster-3.0")
local LibGuild = LibStub("LibGuild-1.0")
local LibGuildPositions = LibStub("LibGuildPositions-1.0")
local Tablet = AceLibrary("Tablet-2.0")

-- local --
local _G = getfenv(0)
local UnitLevel = UnitLevel
local GetGuildInfo = GetGuildInfo
local GetTime = GetTime
local IsInGuild = IsInGuild
local table_concat = table.concat
local format = format
local pairs = pairs
local next = next
local rawget = rawget
local setmetatable = setmetatable
local tostring = tostring

local ICONPATH = "Interface\\AddOns\\MapsterEnhanced\\Textures\\"
local updateInterval = 5
local yards = L["%.0f yd"]
local setupUnitPOIs

local RANK = RANK or "RANK"
local GUILD = GUILD or "GUILD"
local CLASS = CLASS or "CLASS"
local RACE = RACE or "RACE"
local LEVEL = LEVEL or "LEVEL"
local NOTE_COLON = NOTE_COLON or "NOTE_COLON"
local OFFICER_NOTE_COLON = OFFICER_NOTE_COLON or "OFFICER_NOTE_COLON"
if not OFFICER_NOTE_COLON:find(":") then
	OFFICER_NOTE_COLON = OFFICER_NOTE_COLON .. ":"
end

local Notes, Coords

-- defaults --
local db
local defaults = {
	profile = {
		sharePosition = true,
	}
}

local optGetter, optSetter
do
	local mod = GuildPositions
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["GuildPositions"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The GuildPositions module shows you your fellow guild members' positions, as well as allowing them to see you on the World Map."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					name = L["Enable GuildPositions"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
				},
				sharePosition = {
					order = 3,
					type = "toggle",
					name = L["Share own position"],
					get = function(info) return db.sharePosition end,
					set = function(info, value)
						db.sharePosition = value
						LibGuildPositions:ShareSelf(value)
					end,
				},
			},
		}
	end

	return options
end

function GuildPositions:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["GuildPositions"])
end

function GuildPositions:OnEnable()
	if not self:IsEnabled() then return end
	self.initializing = true
	Notes = Mapster:GetModule("Notes")
	Coords = Mapster:GetModule("Coords")
	if not Notes then
		Mapster:SetModuleEnabled(MODNAME, false)
		return
	end
	self:SetupUnitPOIs()
	LibGuild.RegisterCallback(self, "Joined", "LibGuild_Joined")
	LibGuild.RegisterCallback(self, "Left", "LibGuild_Left")

	LibGuildPositions:ShareSelf(db.sharePosition)
	if IsInGuild() and not self.hooked then
		self.hooked = true
		self:RegisterMessage("MAPSTER_MAP_OPENED", "Mapster_MapOpened")
		self:RegisterMessage("MAPSTER_MAP_CLOSED", "Mapster_MapClosed")
		self:RegisterMessage("MAPSTER_CHANGE_ZONE", "Mapster_ChangeZone")
	end

	if not self.updateMap then
		self.updateMap = CreateFrame("Frame", "Mapster_GuildPositions_UpdateMap")
		self.lastUpdate = 0
	end
	self.updateMap:SetScript("OnUpdate", function(self, elapsed) GuildPositions:UpdateMap(elapsed) end)
	self.updateMap:Show()
	self.initializing = nil
	self:Refresh()
end

local function clear(name)
	if GuildPositions.zones[name] then
		GuildPositions.zones[name] = nil
		GuildPositions.x[name] = nil
		GuildPositions.y[name] = nil
		GuildPositions.pois:del(name)
	end
end

function GuildPositions:OnDisable()
	if self.updateMap then
		self.updateMap:SetScript("OnUpdate", nil)
		self.updateMap:Hide()
	end
	if self.initializing then return end
	LibGuild.UnregisterAll(self)
	if self.hooked then
		self.hooked = nil
		self:UnregisterMessage("MAPSTER_MAP_OPENED")
		self:UnregisterMessage("MAPSTER_MAP_CLOSED")
		self:UnregisterMessage("MAPSTER_CHANGE_ZONE")
	end
	for k in pairs(self.pois) do
		if k ~= "del" then
			clear(k)
		end
	end
end

function GuildPositions:Refresh()
	db = self.db.profile
	if not self:IsEnabled() then return end
	if not Notes then
		Mapster:SetModuleEnabled(MODNAME, false)
		return
	end

	if Notes then
		yards = (Notes.db.profile.metric and L["%.0f m"] or L["%.0f yd"])
	end
end

function GuildPositions:SetupUnitPOIs()
	if setupUnitPOIs then return end
	local poiCache = {}

	local function OnEnter(this)
		WorldMapUnit_OnEnter(this)
	end
	local function OnLeave(this)
		WorldMapUnit_OnLeave(this)
	end
	local tmp = {}
	local function OnLineRequest(self)
		tmp[#tmp+1] = GUILD .. ": "
		local name = self.name
		tmp[#tmp+1] = name
		tmp[#tmp+1] = " - |cff"
		local level = LibGuild:GetLevel(name) or 0
		local playerlevel = UnitLevel("player")
		tmp[#tmp+1] = Notes:GetDifficultyHexColor(level)
		tmp[#tmp+1] = tostring(level)
		tmp[#tmp+1] = "|r |cff"
		tmp[#tmp+1] = LibGuild:GetClassHexColor(name)
		local class = LibGuild:GetClass(name)
		tmp[#tmp+1] = class
		tmp[#tmp+1] = "|r"
		local gname = LibGuild:GetGuildName()
		tmp[#tmp+1] = " - |cff00ff00<"
		tmp[#tmp+1] = gname
		tmp[#tmp+1] = ">|r"
		local text = table_concat(tmp, "")
		for i = 1, #tmp do
			tmp[i] = nil
		end
		local tex = self.tex
		local left, top, _, _, _, _, right, bottom = tex:GetTexCoord()
		local r, g, b = tex:GetVertexColor()
		return 'text', text,
			'hasCheck', true,
			'checked', true,
			'checkIcon', tex:GetTexture(),
			'checkCoordLeft', left,
			'checkCoordRight', right,
			'checkCoordTop', top,
			'checkCoordBottom', bottom,
			'checkColorR', r,
			'checkColorG', g,
			'checkColorB', b
	end
	local function OnTabletRequest(self)
		local name = self.name
		Tablet:SetTitle(GUILD .. ": " .. name)
		local cat = Tablet:AddCategory('columns', 2)
		local class = LibGuild:GetClass(name)
		local r, g, b = LibGuild:GetClassColor(name)
		cat:AddLine(
			'text', CLASS .. ":",
			'text2', class,
			'text2R', r,
			'text2G', g,
			'text2B', b
		)
		local level = LibGuild:GetLevel(name) or 0
		local playerlevel = UnitLevel("player")
		local r, g, b = Notes:GetDifficultyColor(level)
		cat:AddLine(
			'text', LEVEL .. ":",
			'text2', level,
			'text2R', r,
			'text2G', g,
			'text2B', b
		)
		local grank, granknum = LibGuild:GetRank(name), LibGuild:GetRankIndex(name)
		local mygname, mygrank, mygranknum = GetGuildInfo("player")

		cat:AddLine(
			'text', GUILD .. ":",
			'text2', "<" .. mygname .. ">",
			'text2R', 0,
			'text2G', 1,
			'text2B', 0
		)

		local grankdiff = granknum - mygranknum
		if grankdiff >= 2 then
			r, g, b = 0, 1, 0
		elseif grankdiff == 1 then
			r, g, b = 0.5, 1, 0
		elseif grankdiff == 0 then
			r, g, b = 1, 1, 0
		elseif grankdiff == -1 then
			r, g, b = 1, 0.5, 0
		else
			r, g, b = 1, 0, 0
		end

		cat:AddLine(
			'text', RANK .. ":",
			'text2', grank,
			'text2R', r,
			'text2G', g,
			'text2B', b
		)

		local note, onote = LibGuild:GetNote(name), LibGuild:GetOfficerNote(name)

		if note then
			cat:AddLine(
				'text', NOTE_COLON,
				'text2', note
			)
		end

		if onote then
			cat:AddLine(
				'text', OFFICER_NOTE_COLON,
				'text2', onote
			)
		end

		local posString
		if Coords then
			local highPrec = Coords.db.profile.accuracy
			posString = "%." .. highPrec .. "f, %." .. highPrec .. "f"
		else
			posString = "%.1f, %.1f"
		end
		local x, y, zone = GuildPositions.x[name], GuildPositions.y[name], GuildPositions.zones[name]
		local dist = Notes:GetDistanceToPoint(x, y, zone)

		if Notes.db.profile.location and x and x > 0 and y and y > 0 and zone then
			cat:AddLine(
				'text', L["Location:"],
				'text2', Notes:GetZoneFromMapTexture(zone) .. " (" .. (format(posString, x * 100, y * 100)) .. ")"
			)
		end

		if Notes.db.profile.showDistance and dist and dist >= 1 then
			yards = (Notes.db.profile.metric and L["%.0f m"] or L["%.0f yd"])
			cat:AddLine(
				'text', L["Distance:"],
				'text2', format(yards, dist)
			)
		end
	end

	local num = 0
	self.zones = {}
	self.x = {}
	self.y = {}
	self.times = {}
	self.pois = setmetatable({del = function(self, key)
		if key == "del" then
			return
		end
		local poi = rawget(self, key)
		if not poi then
			return
		end
		self[key] = nil
		poiCache[poi] = true
		poi:Hide()
	end}, {__index = function(self, key)
		if not key then
			return nil
		end
		local poi = next(poiCache)
		if poi then
			poiCache[poi] = nil
			self[key] = poi
			poi:Show()
			poi.name = key
			return poi
		end
		num = num + 1
		poi = CreateFrame("Frame", "Mapster_GuildPositions" .. num, WorldMapButton)
		poi:SetWidth(16)
		poi:SetHeight(16)
		poi:SetPoint("CENTER")
		local tex = poi:CreateTexture(nil, "OVERLAY")
		poi.tex = tex
		tex:SetTexture(ICONPATH .. "Guild")
		tex:SetAllPoints(poi)
		poi:SetFrameLevel(poi:GetFrameLevel() + 4)
		poi:SetScript("OnEnter", OnEnter)
		poi:SetScript("OnLeave", OnLeave)
		poi:EnableMouse(true)
		Notes:AddPOI(poi)
		poi.name = key
		poi.OnLineRequest = OnLineRequest
		poi.OnTabletRequest = OnTabletRequest
		self[key] = poi
		return poi
	end})
	setupUnitPOIs = true
end

function GuildPositions:Mapster_MapOpened()
	--if self.hooked then
	--end
end

function GuildPositions:Mapster_MapClosed()
	if self.hooked then
		for k in pairs(self.pois) do
			self.pois:del(k)
		end
	end
end

local function showPOI(name)
	if Roster:GetUnitIDFromName(name) then
		GuildPositions.pois:del(name)
		return
	end
	local x, y, zone = GuildPositions.x[name], GuildPositions.y[name], GuildPositions.zones[name]
	x, y = Notes:TranslateWorldMapPosition(x, y, zone, Notes:GetCurrentMapTexture())
	if not x or x < 0 or x > 1 or not y or y < 0 or y > 1 then
		GuildPositions.pois:del(name)
		return
	end
	local w, h = WorldMapButton:GetWidth(), WorldMapButton:GetHeight()
	local poi = GuildPositions.pois[name]
	poi:SetPoint("CENTER", WorldMapButton, "TOPLEFT", w*x, -h*y)
	poi.tex:SetVertexColor(LibGuild:GetClassColor(name))
end

function GuildPositions:Mapster_ChangeZone()
	if self.hooked then
		for k in pairs(self.pois) do
			self.pois:del(k)
		end
		if WorldMapFrame:IsShown() then
			for name in pairs(self.zones) do
				showPOI(name)
			end
		end
	end
end

function GuildPositions:LibGuild_Joined()
	if not self.hooked then
		self.hooked = true
		self:RegisterMessage("MAPSTER_MAP_OPENED", "Mapster_MapOpened")
		self:RegisterMessage("MAPSTER_MAP_CLOSED", "Mapster_MapClosed")
		self:RegisterMessage("MAPSTER_CHANGE_ZONE", "Mapster_ChangeZone")
	end

	if WorldMapFrame:IsShown() then
		for name in pairs(self.zones) do
			showPOI(name)
		end
	end
end

function GuildPositions:LibGuild_Left()
	if self.hooked then
		self.hooked = nil
		self:UnregisterMessage("MAPSTER_MAP_OPENED")
		self:UnregisterMessage("MAPSTER_MAP_CLOSED")
		self:UnregisterMessage("MAPSTER_CHANGE_ZONE")
	end

	for k in pairs(self.zones) do
		clear(k)
	end
end

function GuildPositions:UpdateMap(elapsed)
	self.lastUpdate = self.lastUpdate + elapsed
	if (self.lastUpdate >= updateInterval) then
		self.lastUpdate = 0

		local t = GetTime()
		for k,v in pairs(self.times) do
			if t >= v then
				clear(k)
			end
		end

		for name, x, y, zone in LibGuildPositions:IterateGuildMembers() do
			self.x[name] = x
			self.y[name] = y
			self.zones[name] = zone
			self.times[name] = GetTime() + 60
			if WorldMapFrame:IsShown() then
				showPOI(name)
			end
		end
	end
end
