--[[
Name: Astrolabe
Revision: $Rev: 128 $
$Date: 2009-08-05 07:34:29 +0000 (Wed, 05 Aug 2009) $
Author(s): Esamynn (esamynn at wowinterface.com)
Inspired By: Gatherer by Norganna
             MapLibrary by Kristofer Karlsson (krka at kth.se)
Documentation: http://wiki.esamynn.org/Astrolabe
SVN: http://svn.esamynn.org/astrolabe/
Description:
	This is a library for the World of Warcraft UI system to place
	icons accurately on both the Minimap and on Worldmaps.
	This library also manages and updates the position of Minimap icons
	automatically.

Copyright (C) 2006-2008 James Carrothers

License:
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Lesser General Public
	License as published by the Free Software Foundation; either
	version 2.1 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public
	License along with this library; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Note:
	This library's source code is specifically designed to work with
	World of Warcraft's interpreted AddOn system.  You have an implicit
	licence to use this library with these facilities since that is its
	designated purpose as per:
	http://www.fsf.org/licensing/licenses/gpl-faq.html#InterpreterIncompat
]]

-- WARNING!!!
-- DO NOT MAKE CHANGES TO THIS LIBRARY WITHOUT FIRST CHANGING THE LIBRARY_VERSION_MAJOR
-- STRING (to something unique) OR ELSE YOU MAY BREAK OTHER ADDONS THAT USE THIS LIBRARY!!!
local LIBRARY_VERSION_MAJOR = "Astrolabe-0.7-Global"
local LIBRARY_VERSION_MINOR = tonumber(string.match("$Revision: 128 $", "(%d+)") or 1)

if not DongleStub then error(LIBRARY_VERSION_MAJOR .. " requires DongleStub.") end
if not DongleStub:IsNewerVersion(LIBRARY_VERSION_MAJOR, LIBRARY_VERSION_MINOR) then return end

local Astrolabe = {};

-- define local variables for Data Tables (defined at the end of this file)
local WorldMapSize, MinimapSize, ValidMinimapShapes, VirtualContinentIndexes, VirtualMapTextures, MapTexturesC, MapTexturesZ, MapZones, MapZoneTextures, ContinentMapTextures;

function Astrolabe:GetVersion()
	return LIBRARY_VERSION_MAJOR, LIBRARY_VERSION_MINOR;
end


--------------------------------------------------------------------------------------------------------------
-- Config Constants
--------------------------------------------------------------------------------------------------------------

local configConstants = {
	MinimapUpdateMultiplier = true,
}

-- this constant is multiplied by the current framerate to determine
-- how many icons are updated each frame
Astrolabe.MinimapUpdateMultiplier = 1;


--------------------------------------------------------------------------------------------------------------
-- Tables
--------------------------------------------------------------------------------------------------------------

Astrolabe.LastPlayerPosition = { 0, 0, 0, 0 };
Astrolabe.MinimapIcons = {};
Astrolabe.IconsOnEdge = {};
Astrolabe.IconsOnEdge_GroupChangeCallbacks = {};

Astrolabe.MinimapIconCount = 0
Astrolabe.ForceNextUpdate = false;
Astrolabe.IconsOnEdgeChanged = false;

-- This variable indicates whether we know of a visible World Map or not.
-- The state of this variable is controlled by the AstrolabeMapMonitor library.
Astrolabe.WorldMapVisible = false;

local AddedOrUpdatedIcons = {}
local MinimapIconsMetatable = { __index = AddedOrUpdatedIcons }


--------------------------------------------------------------------------------------------------------------
-- Local Pointers for often used API functions
--------------------------------------------------------------------------------------------------------------

local twoPi = math.pi * 2;
local atan2 = math.atan2;
local sin = math.sin;
local cos = math.cos;
local abs = math.abs;
local sqrt = math.sqrt;
local min = math.min
local pi = math.pi;
local deg = math.deg;
local max = math.max;
local yield = coroutine.yield;
local match = string.match;
local next = next;
local ipairs = ipairs;
local pairs = pairs;
local tonumber = tonumber;
local GetFramerate = GetFramerate;
local GetCurrentMapContinent, GetCurrentMapZone = GetCurrentMapContinent, GetCurrentMapZone;
local GetMapContinents, GetMapZones = GetMapContinents, GetMapZones;
local GetCurrentMapAreaID = GetCurrentMapAreaID;
local GetCurrentMapDungeonLevel = GetCurrentMapDungeonLevel;
local GetMapInfo = GetMapInfo;
local SetMapToCurrentZone = SetMapToCurrentZone;
local SetMapZoom = SetMapZoom;
local SetMapByID = SetMapByID;
local SetDungeonMapLevel = SetDungeonMapLevel;
local GetPlayerMapPosition = GetPlayerMapPosition;
local GetPlayerFacing = GetPlayerFacing;
local GetMinimapShape = GetMinimapShape;
local GetCVar = GetCVar;
local IsInInstance = IsInInstance;

--------------------------------------------------------------------------------------------------------------
-- Internal Utility Functions
--------------------------------------------------------------------------------------------------------------

local function assert(level,condition,message)
	if not condition then
		error(message,level)
	end
end

local function argcheck(value, num, ...)
	assert(1, type(num) == "number", "Bad argument #2 to 'argcheck' (number expected, got " .. type(level) .. ")")

	for i=1,select("#", ...) do
		if type(value) == select(i, ...) then return end
	end

	local types = strjoin(", ", ...)
	local name = string.match(debugstack(2,2,0), ": in function [`<](.-)['>]")
	error(string.format("Bad argument #%d to 'Astrolabe.%s' (%s expected, got %s)", num, name, types, type(value)), 3)
end

function getContPosition( zoneData, z, x, y )
	if ( z ~= 0 ) then
		zoneData = zoneData[z];
		x = x * zoneData.width + zoneData.xOffset;
		y = y * zoneData.height + zoneData.yOffset;
	else
		x = x * zoneData.width;
		y = y * zoneData.height;
	end
	return x, y;
end

local function table_getn( t )
	local i = 0;
	for k in pairs(t) do
		i = i + 1;
	end
	return i;
end

local temp = {};
function table_unpack( t )
	local i, k = 0, 0;
	local c = table_getn(t);
	wipe(temp);
	while ( i < c ) do
		k = k + 1;
		if t[k] then
			temp[k] = t[k];
			i = i + 1;
		else
			temp[k] = "";
		end
	end
	return unpack(temp);
end

--------------------------------------------------------------------------------------------------------------
-- Virtual Continent Functions
--------------------------------------------------------------------------------------------------------------

function Astrolabe:GetCurrentVirtualMapCZ()
	local C, Z = GetCurrentMapContinent(), GetCurrentMapZone();
	if C == -1 and Z == 0 then
		-- Cosmic
		local mapName = GetMapInfo();
		if VirtualMapTextures[mapName] and mapName == "CoTStratholme" and GetCurrentMapDungeonLevel() == 0 then
			C = VirtualMapTextures[mapName];
			Z = 2;
		elseif mapName == "TheArgentColiseum" and GetCurrentMapAreaID() == 542 then
			C = VirtualMapTextures["TheArgentColiseumRaid"];
			Z = GetCurrentMapDungeonLevel();
		elseif mapName and VirtualMapTextures[mapName] then
			C = VirtualMapTextures[mapName];
			Z = GetCurrentMapDungeonLevel();
		end
	elseif (C == 1 and Z == 17) or (C == 4 and Z == 3) then
		local mapName = GetMapInfo();
		if mapName and VirtualMapTextures[mapName] and GetCurrentMapDungeonLevel() > 1 then
			C = VirtualMapTextures[mapName];
			Z = GetCurrentMapDungeonLevel();
		end
	end
	return C, Z;
end

function Astrolabe:GetCurrentVirtualMapContinent() local C, Z = self:GetCurrentVirtualMapCZ(); return C; end
function Astrolabe:GetCurrentVirtualMapZone() local C, Z = self:GetCurrentVirtualMapCZ(); return Z; end

-- Does much the same as GetMapContinents, but returns as an array and includes the virtual continents in the mix
function Astrolabe:GetMapVirtualContinents()
	local rv = { GetMapContinents() };
	for k, v in pairs(VirtualContinentIndexes) do
		rv[k] = v.continent;
	end
	--return unpack(rv);
	return rv;
end

-- Does much the same as GetMapZones, but returns as an array and includes the virtual continents in the mix
function Astrolabe:GetMapVirtualZones(c)
	if not c or c < -1 then
		return;
	end

	if VirtualContinentIndexes[c] then
		if VirtualContinentIndexes[c].mapFloors and VirtualContinentIndexes[c].mapFloors > 0 then
			local zoneList = {};
			for i = 1, VirtualContinentIndexes[c].mapFloors do
				zoneList[#zoneList+1] = VirtualContinentIndexes[c].mapTexture..":"..i;
			end
			--return unpack(zoneList);
			return zoneList;
		else
			return { VirtualContinentIndexes[c].mapTexture };
		end
	end

	return { GetMapZones(c) };
end

-- Converts C, Z to MapTexture as returned by GetMapInfo()
function Astrolabe:GetMapTexture(c, z)
	if not c or c < -1 or (z and z < 0) then
		return nil;
	end
	z = z or 0;

	if ( c > 5 ) then
		if VirtualContinentIndexes[c] then
			if z == 0 then
				return VirtualContinentIndexes[c].mapTexture;
			elseif z > 0 then
				if VirtualContinentIndexes[c].mapFloors >= z then
					return VirtualContinentIndexes[c].mapTexture..":"..z;
				else
					return nil;
				end
			end
		else
			return nil;
		end
	else
		if z == 0 then
			return ContinentMapTextures[c];
		end

		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		if ( c == lastCont and z == lastZone ) then
			local mapTexture = GetMapInfo();
			mapTexture = match(mapTexture, "^(.+)_terrain(%d+)$") or mapTexture;
			return mapTexture;
		else
			local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel();
			SetMapZoom(c, z);
			local mapTexture = GetMapInfo();
			mapTexture = match(mapTexture, "^(.+)_terrain(%d+)$") or mapTexture;
			if ( lastMapID > 0 ) then
				SetMapByID(lastMapID); -- set map zoom back to what it was before
				SetDungeonMapLevel(lastFloor);
			else
				SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
			end
			return mapTexture;
		end
	end
end

-- Converts MapTexture to C, Z
function Astrolabe:GetMapCZ(mapTexture, mapFloor)
	if not mapTexture then
		return nil, nil;
	end

	if VirtualMapTextures[mapTexture] then
		if mapFloor and VirtualContinentIndexes[VirtualMapTextures[mapTexture]].mapFloors >= mapFloor then
			return VirtualMapTextures[mapTexture], mapFloor;
		end
		return VirtualMapTextures[mapTexture], 0;
	elseif MapTexturesC[mapTexture] then
		return MapTexturesC[mapTexture], MapTexturesZ[mapTexture];
	elseif match(mapTexture, ":%d+$") then
		local zone, level = match(mapTexture, "^(.+):(%d+)$");
		level = tonumber(level);
		if level < 0 then
			level = 0;
		end
		if VirtualMapTextures[zone] and VirtualContinentIndexes[VirtualMapTextures[zone]].mapFloors >= level then
			return VirtualMapTextures[zone], level;
		end
	end

	return nil, nil;
end

function Astrolabe:GetMapID(c, z)
	c = c or 0;
	z = z or 0;
	if ( c > 5 ) then
		if ( VirtualContinentIndexes[c] ) then
			return VirtualContinentIndexes[c].mapID;
		else
			return;
		end
	else
		if ( c == 0 ) then
			return 0;
		elseif ( c == -1 ) then
			return -1;
		end
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		if ( c == lastCont and z == lastZone ) then
			return GetCurrentMapAreaID();
		else
			local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel();
			SetMapZoom(c, z);
			local mapID = GetCurrentMapAreaID();
			if ( lastMapID > 0 ) then
				SetMapByID(lastMapID); -- set map zoom back to what it was before
				SetDungeonMapLevel(lastFloor);
			else
				SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
			end
			return mapID;
		end
	end
end

-- Converts MapTexture to MapZone
function Astrolabe:GetMapZone(mapTexture)
	if not mapTexture then
		return nil;
	end

	return MapZones[mapTexture];
end

-- Converts MapZone to MapTexture
function Astrolabe:GetMapZoneTexture(mapZone)
	if not mapZone then
		return nil;
	end

	return MapZoneTextures[mapZone];
end

-- Converts MapTexture to MapZone
function Astrolabe:GetMapFloors(c, z)
	if not c then
		return nil;
	end

	if c > 5 and VirtualContinentIndexes[c] and VirtualContinentIndexes[c].mapFloors then
		return VirtualContinentIndexes[c].mapFloors;
	elseif (c == 1 and z == 17) or (c == 4 and z == 3) then
		return 2;
	end
	return 0;
end

-- Converts MapZone to MapTexture
function Astrolabe:GetMapFloorsFromTexture(mapTexture)
	if not mapTexture then
		return nil;
	end

	if VirtualContinentIndexes[VirtualMapTextures[mapTexture]] and VirtualContinentIndexes[VirtualMapTextures[mapTexture]].mapFloors then
		return VirtualContinentIndexes[VirtualMapTextures[mapTexture]].mapFloors;
	end
	return 0;
end

--------------------------------------------------------------------------------------------------------------
-- General Utility Functions
--------------------------------------------------------------------------------------------------------------

function Astrolabe:ComputeDistance( c1, z1, x1, y1, c2, z2, x2, y2 )
	--[[
	argcheck(c1, 2, "number");
	assert(3, c1 >= 0, "ComputeDistance: Illegal continent index to c1: "..c1);
	argcheck(z1, 3, "number", "nil");
	argcheck(x1, 4, "number");
	argcheck(y1, 5, "number");
	argcheck(c2, 6, "number");
	assert(3, c2 >= 0, "ComputeDistance: Illegal continent index to c2: "..c2);
	argcheck(z2, 7, "number", "nil");
	argcheck(x2, 8, "number");
	argcheck(y2, 9, "number");
	--]]

	if not c1 or not c2 or not x1 or not x2 or not y1 or not y2 then return end
	c1 = c1 or 0;
	c2 = c2 or 0;
	z1 = z1 or 0;
	z2 = z2 or 0;

	if (c1 < 0) or (c2 < 0) or (c1 > 5 and not VirtualContinentIndexes[c1]) or (c2 > 5 and not VirtualContinentIndexes[c2]) then
		return nil, nil, nil;
	end
	if (z1 < 0) or (z2 < 0) or (c1 > 5 and z1 > 0 and z1 > VirtualContinentIndexes[c1].mapFloors) or (c2 > 5 and z2 > 0 and z2 > VirtualContinentIndexes[c2].mapFloors) then
		return nil, nil, nil;
	end

	local dist, xDelta, yDelta;
	if ( c1 == c2 and z1 == z2 ) then
		-- points in the same zone
		local zoneData = WorldMapSize[c1];
		if ( z1 ~= 0 ) then
			zoneData = zoneData[z1];
		end
		xDelta = (x2 - x1) * zoneData.width;
		yDelta = (y2 - y1) * zoneData.height;
	elseif ( c1 == c2 ) then
		-- points on the same continent
		local zoneData = WorldMapSize[c1];
		x1, y1 = getContPosition(zoneData, z1, x1, y1);
		x2, y2 = getContPosition(zoneData, z2, x2, y2);
		xDelta = (x2 - x1);
		yDelta = (y2 - y1);
	elseif ( c1 and c2 ) and (( WorldMapSize[c1].parentContinent ~= 3 and WorldMapSize[c2].parentContinent ~= 3 ) or ( WorldMapSize[c1].parentContinent == 3 and WorldMapSize[c2].parentContinent == 3 )) then
		local cont1 = WorldMapSize[c1];
		local cont2 = WorldMapSize[c2];
		x1, y1 = getContPosition(cont1, z1, x1, y1);
		x2, y2 = getContPosition(cont2, z2, x2, y2);
		if ( c1 ~= cont1.parentContinent ) then
			x1 = x1 + cont1.xOffset + (cont1.xOffsetEntrance or 0);
			y1 = y1 + cont1.yOffset + (cont1.yOffsetEntrance or 0);
		end
		if ( c2 ~= cont2.parentContinent ) then
			x2 = x2 + cont2.xOffset + (cont2.xOffsetEntrance or 0);
			y2 = y2 + cont2.yOffset + (cont2.yOffsetEntrance or 0);
		end

		xDelta = x2 - x1;
		yDelta = y2 - y1;
	elseif ( c1 and c2 ) and ( WorldMapSize[c1].parentContinent == 3 or WorldMapSize[c2].parentContinent == 3 ) then
		if ( WorldMapSize[c2].parentContinent == 3 ) then
			local xo1, yo1, xo2, yo2;

			local cont1 = WorldMapSize[c1];
			local cont2 = WorldMapSize[2];
			xo1, yo1 = getContPosition(cont1, z1, x1, y1);
			xo2, yo2 = getContPosition(cont2, 4, 0.55, 0.542);
			if ( c1 ~= cont1.parentContinent ) then
				xo1 = xo1 + cont1.xOffset + (cont1.xOffsetEntrance or 0);
				yo1 = yo1 + cont1.yOffset + (cont1.yOffsetEntrance or 0);
			end
			if ( 2 ~= cont2.parentContinent ) then
				xo2 = xo2 + cont2.xOffset + (cont2.xOffsetEntrance or 0);
				yo2 = yo2 + cont2.yOffset + (cont2.yOffsetEntrance or 0);
			end

			xDelta = xo2 - xo1;
			yDelta = yo2 - yo1;

			local cont1 = WorldMapSize[3];
			local cont2 = WorldMapSize[c2];
			xo1, yo1 = getContPosition(cont1, 2, 0.899, 0.502);
			xo2, yo2 = getContPosition(cont2, z2, x2, y2);
			if ( 3 ~= cont1.parentContinent ) then
				xo1 = xo1 + cont1.xOffset + (cont1.xOffsetEntrance or 0);
				yo1 = yo1 + cont1.yOffset + (cont1.yOffsetEntrance or 0);
			end
			if ( c2 ~= cont2.parentContinent ) then
				xo2 = xo2 + cont2.xOffset + (cont2.xOffsetEntrance or 0);
				yo2 = yo2 + cont2.yOffset + (cont2.yOffsetEntrance or 0);
			end

			xDelta = xDelta + xo2 - xo1;
			yDelta = yDelta + yo2 - yo1;
		elseif ( WorldMapSize[c1].parentContinent == 3 ) then
			local xo1, yo1, xo2, yo2;

			local cont1 = WorldMapSize[c1];
			local cont2 = WorldMapSize[3];
			xo1, yo1 = getContPosition(cont1, z1, x1, y1);
			xo2, yo2 = getContPosition(cont2, 2, 0.899, 0.502);
			if ( c1 ~= cont1.parentContinent ) then
				xo1 = xo1 + cont1.xOffset + (cont1.xOffsetEntrance or 0);
				yo1 = yo1 + cont1.yOffset + (cont1.yOffsetEntrance or 0);
			end
			if ( 3 ~= cont2.parentContinent ) then
				xo2 = xo2 + cont2.xOffset + (cont2.xOffsetEntrance or 0);
				yo2 = yo2 + cont2.yOffset + (cont2.yOffsetEntrance or 0);
			end

			xDelta = xo2 - xo1;
			yDelta = yo2 - yo1;

			local cont1 = WorldMapSize[2];
			local cont2 = WorldMapSize[c2];
			xo1, yo1 = getContPosition(cont1, 4, 0.55, 0.542);
			xo2, yo2 = getContPosition(cont2, z2, x2, y2);
			if ( 2 ~= cont1.parentContinent ) then
				xo1 = xo1 + cont1.xOffset + (cont1.xOffsetEntrance or 0);
				yo1 = yo1 + cont1.yOffset + (cont1.yOffsetEntrance or 0);
			end
			if ( c2 ~= cont2.parentContinent ) then
				xo2 = xo2 + cont2.xOffset + (cont2.xOffsetEntrance or 0);
				yo2 = yo2 + cont2.yOffset + (cont2.yOffsetEntrance or 0);
			end

			xDelta = xDelta + xo2 - xo1;
			yDelta = yDelta + yo2 - yo1;
		end
	end
	if ( xDelta and yDelta ) then
		dist = sqrt(xDelta*xDelta + yDelta*yDelta);
	end
	return dist, xDelta, yDelta;
end
Astrolabe.GetDistanceToPoint = Astrolabe.ComputeDistance;

function Astrolabe:TranslateWorldMapPosition( C, Z, xPos, yPos, nC, nZ )
	--[[
	argcheck(C, 2, "number");
	argcheck(Z, 3, "number", "nil");
	argcheck(xPos, 4, "number");
	argcheck(yPos, 5, "number");
	argcheck(nC, 6, "number");
	argcheck(nZ, 7, "number", "nil");
	--]]

	if (C < 0) or (C > 5 and not VirtualContinentIndexes[C]) or (nC > 5 and not VirtualContinentIndexes[nC]) then
		return nil, nil;
	end
	if (Z < 0) or (nZ < 0) or (C > 5 and Z > 0 and Z > VirtualContinentIndexes[C].mapFloors) or (nC > 5 and nZ > 0 and nZ > VirtualContinentIndexes[nC].mapFloors) then
		return nil, nil;
	end

	Z = Z or 0;
	nZ = nZ or 0;
	if ( nC < 0 ) or (C == 0 and Z > 0) or (nC == 0 and nZ > 0) then
		return;
	end

	local zoneData;
	if ( C == nC and Z == nZ ) then
		return xPos, yPos;

	elseif ( C == nC ) then
		-- points on the same continent
		zoneData = WorldMapSize[C];
		xPos, yPos = getContPosition(zoneData, Z, xPos, yPos);
		if ( nZ ~= 0 ) then
			zoneData = WorldMapSize[C][nZ];
			xPos = xPos - zoneData.xOffset;
			yPos = yPos - zoneData.yOffset;
		end

	elseif ( C and nC ) and ( WorldMapSize[C].parentContinent == WorldMapSize[nC].parentContinent ) then
		-- different continents, same world
		zoneData = WorldMapSize[C];
		local parentContinent = zoneData.parentContinent;
		xPos, yPos = getContPosition(zoneData, Z, xPos, yPos);
		if ( C ~= parentContinent ) then
			-- translate up to world map if we aren't there already
			xPos = xPos + zoneData.xOffset;
			yPos = yPos + zoneData.yOffset;
			zoneData = WorldMapSize[parentContinent];
		end
		if ( nC ~= parentContinent ) then
			-- translate down to the new continent
			zoneData = WorldMapSize[nC];
			xPos = xPos - zoneData.xOffset;
			yPos = yPos - zoneData.yOffset;
			if ( nZ ~= 0 ) then
				zoneData = zoneData[nZ];
				xPos = xPos - zoneData.xOffset;
				yPos = yPos - zoneData.yOffset;
			end
		end

	elseif ( C and nC ) and (( WorldMapSize[C].parentContinent ~= 3 and WorldMapSize[nC].parentContinent ~= 3 ) or ( WorldMapSize[C].parentContinent == 3 and WorldMapSize[nC].parentContinent == 3 )) then
		-- different continents, same world
		zoneData = WorldMapSize[C];
		local parentContinent = zoneData.parentContinent;
		xPos, yPos = getContPosition(zoneData, Z, xPos, yPos);
		if ( C ~= parentContinent ) then
			-- translate up to world map if we aren't there already
			xPos = xPos + zoneData.xOffset + (zoneData.xOffsetEntrance or 0);
			yPos = yPos + zoneData.yOffset + (zoneData.yOffsetEntrance or 0);
			zoneData = WorldMapSize[parentContinent];
		end
		if ( nC ~= parentContinent ) then
			-- translate down to the new continent
			zoneData = WorldMapSize[nC];
		end
		xPos = xPos - (zoneData.xOffset or 0) - (zoneData.xOffsetEntrance or 0);
		yPos = yPos - (zoneData.yOffset or 0) - (zoneData.yOffsetEntrance or 0);
		if ( nZ ~= 0 ) then
			zoneData = zoneData[nZ];
			xPos = xPos - zoneData.xOffset;
			yPos = yPos - zoneData.yOffset;
		end

	else
		return;
	end

	return (xPos / zoneData.width), (yPos / zoneData.height);
end

--*****************************************************************************
-- This function will return the absolute position on the parent continent.
-- 0 24185.581756984 4980.9683211935
-- parentContinent	- parent continent
-- hYardDistance	- horizontal yard distance to upper left corner of parent continent
-- vYardDistance	- vertical yard distance to upper left corner of parent continent
--*****************************************************************************
function Astrolabe:GetAbsoluteWorldPosition( C, Z, xPos, yPos )
	if not C or not Z then
		return;
	end
	if C == -1 or (C > 5 and not VirtualContinentIndexes[C]) then -- We're in a battleground that doesn't have a virtual continent, we're just kind of fucked.
		return;
	end

	local x, y = Astrolabe:TranslateWorldMapPosition(C, Z, xPos, yPos, WorldMapSize[C].parentContinent, 0);
	if not x or not y then return end
	local zoneData = WorldMapSize[WorldMapSize[C].parentContinent];
	return WorldMapSize[C].parentContinent, (x * zoneData.width), (y * zoneData.height);
end

function Astrolabe:GetAbsoluteContinentPosition( C, Z, xPos, yPos )
	if not C or not Z then
		return;
	end
	if C == -1 or (C > 5 and not VirtualContinentIndexes[C]) then -- We're in a battleground that doesn't have a virtual continent, we're just kind of fucked.
		return;
	end

	local x, y = Astrolabe:TranslateWorldMapPosition(C, Z, xPos, yPos, C, 0);
	if not x or not y then return end
	local zoneData = WorldMapSize[C];
	return C, (x * zoneData.width), (y * zoneData.height);
end

function Astrolabe:FromAbsoluteContinentPosition(C, xPos, yPos)
	if (C < -1) or (C > 5 and not VirtualContinentIndexes[C]) then return end
	return C, 0, xPos / WorldMapSize[C].width, yPos / WorldMapSize[C].height;
end

function Astrolabe:GetZoneDimensions( C, Z )
	if not C then
		return;
	end
	Z = Z or 0;
	if C == -1 or (C > 5 and not VirtualContinentIndexes[C]) then
		return nil, nil;
	end

	if Z > 0 then
		if WorldMapSize[C][Z] then
			return WorldMapSize[C][Z].width, WorldMapSize[C][Z].height;
		else
			return nil, nil;
		end
	else
		return WorldMapSize[C].width, WorldMapSize[C].height;
	end
end

--*****************************************************************************
-- This function will return the global coordinates for the specified position.
-- -41.2 70 for the upper left corner.
--*****************************************************************************
function Astrolabe:GetAbsoluteGlobalPosition( C, Z, xPos, yPos )
	if C == -1 or C == 3 or (C > 5 and not VirtualContinentIndexes[C]) then return end
--	51.5 87.5
	local x, y = self:TranslateWorldMapPosition(C, Z, xPos, yPos, 0, 0);
	if not x or not y then return end
	local xC, yC = (x - 0.515), (y - 0.875);
	xC, yC = xC * 100, (yC - yC * 2) * 100;
	return (xC*70/87.5), (yC*70/87.5);
end

--*****************************************************************************
-- This function will return the global coordinates for the specified position.
-- "N", 50, 40, 14, 0.7, "E", 50, 40, 14, 0.7
--*****************************************************************************
function Astrolabe:GetAbsoluteGlobalCoordinates( C, Z, xPos, yPos )
	if C == -1 or C == 3 or (C > 5 and not VirtualContinentIndexes[C]) then return end
--	51.5 87.5
	local x, y = self:TranslateWorldMapPosition(C, Z, xPos, yPos, 0, 0);
	if not x or not y then return end
	local xC, yC = (x - 0.515), (y - 0.875);
	xC, yC = xC * 100, (yC - yC * 2) * 100;
	xC, yC = (xC*70/87.5), (yC*70/87.5);

	local cySign = (yC < 0) and "S" or "N";
	local cy = abs(yC);				-- 50.67075000
	local yDec = cy - floor(cy);	-- 0.67075000
	local yDeg = floor(cy);			-- 50
	local yMin = yDec * 60;			-- 40.24500000
	local ySec = (yMin - floor(yMin)) * 60;	-- 14.70000000
	yDec = ySec - floor(ySec);	-- 0.70000000
	yMin = floor(yMin);	-- 40
	ySec = floor(ySec);	-- 14

	local cxSign = (xC < 0) and "W" or "E";
	local cx = abs(xC);
	local xDec = cx - floor(cx);
	local xDeg = floor(cx);
	local xMin = xDec * 60;
	local xSec = (xMin - floor(xMin)) * 60;
	xDec = xSec - floor(xSec);
	xMin = floor(xMin);
	xSec = floor(xSec);

	return cySign, yDeg, yMin, ySec, yDec, cxSign, xDeg, xMin, xSec, xDec;
end

--*****************************************************************************
-- This function will return the global coordinates for the specified position.
-- Center point is based at Maelstrom.
-- -70 70 for the upper left corner.
--*****************************************************************************
function Astrolabe:GetAbsoluteMaelstromPosition( C, Z, xPos, yPos )
	if C == -1 or C == 3 or (C > 5 and not VirtualContinentIndexes[C]) then return end
--	51.5 51.5
	local x, y = self:TranslateWorldMapPosition(C, Z, xPos, yPos, 0, 0);
	if not x or not y then return end
	local xC, yC = (x - 0.515), (y - 0.515);
	xC, yC = xC * 100, (yC - yC * 2) * 100;
	return (xC*70/51.5), (yC*70/51.5);
end

--*****************************************************************************
-- This function will return the global coordinates for the specified position.
-- Center point is based at Maelstrom.
-- "N", 50, 40, 14, 0.7, "E", 50, 40, 14, 0.7
--*****************************************************************************
function Astrolabe:GetAbsoluteMaelstromCoordinates( C, Z, xPos, yPos )
	if C == -1 or C == 3 or (C > 5 and not VirtualContinentIndexes[C]) then return end
--	51.5 51.5
	local x, y = self:TranslateWorldMapPosition(C, Z, xPos, yPos, 0, 0);
	if not x or not y then return end
	local xC, yC = (x - 0.515), (y - 0.515);
	xC, yC = xC * 100, (yC - yC * 2) * 100;
	xC, yC = (xC*70/51.5), (yC*70/51.5);

	local cySign = (yC < 0) and "S" or "N";
	local cy = abs(yC);
	local yDec = cy - floor(cy);
	local yDeg = floor(cy);
	local yMin = yDec * 60;
	local ySec = (yMin - floor(yMin)) * 60;
	yDec = ySec - floor(ySec);
	yMin = floor(yMin);
	ySec = floor(ySec);

	local cxSign = (xC < 0) and "W" or "E";
	local cx = abs(xC);
	local xDec = cx - floor(cx);
	local xDeg = floor(cx);
	local xMin = xDec * 60;
	local xSec = (xMin - floor(xMin)) * 60;
	xDec = xSec - floor(xSec);
	xMin = floor(xMin);
	xSec = floor(xSec);

	return cySign, yDeg, yMin, ySec, yDec, cxSign, xDeg, xMin, xSec, xDec;
end

--*****************************************************************************
-- This function will return the direction the specified unit is facing.
--*****************************************************************************
-- GetPlayerFacing seems to return values between -pi and pi instead of 0 - 2pi sometimes since 3.3.3
--[[local GetPlayerFacing = function(...)
	local result = GetPlayerFacing(...)
	if result < 0 then
		result = result + twoPi
	end
	return result
end]]--

function Astrolabe:GetPlayerFacing(...)
	local result = GetPlayerFacing(...);
	if result < 0 then
		result = result + twoPi;
	end
	return result;
end

--*****************************************************************************
-- This function will return the direction the specified unit is facing.
--*****************************************************************************
function Astrolabe:GetPlayerSpeed(unit)
	local result = floor(GetUnitSpeed(unit) / 7 * 100 + 0.1);
	return result;
end

function Astrolabe:GetPlayerHeading()
	local facing = deg(-GetPlayerFacing()) % 360;
	return facing;
end

--*****************************************************************************
-- This function will return the direction to the specified unit.
--*****************************************************************************
function Astrolabe:GetHeadingToUnit( unit )
	if ( not UnitExists(unit) or not UnitIsPlayer(unit) or not (UnitInRaid(unit) or UnitInParty(unit)) ) then
		return;
	end
	local c1, z1, x1, y1 = self:GetCurrentPlayerPosition();
	local c2, z2, x2, y2 = self:GetUnitPosition(unit);

	if ( not c1 or not c2 or not z1 or not z2 or not x1 or not x2 or not y1 or not y2 ) or ( x1 <= 0 and y1 <= 0 ) or ( x2 <= 0 and y2 <= 0 ) then
		return nil;
	end
	if c1 == -1 or (c1 > 5 and not VirtualContinentIndexes[c1]) then return end
	if c2 == -1 or (c2 > 5 and not VirtualContinentIndexes[c2]) then return end

	local dist, xDist, yDist;
	if ( c1 == 3 and c2 ~= 3 ) or ( c1 ~= 3 and c2 == 3 ) then
		if ( c2 == 3 ) then
			dist, xDist, yDist = self:ComputeDistance(c1, z1, x1, y1, 2, 4, 0.55, 0.542);
		elseif ( c1 == 3 ) then
			dist, xDist, yDist = self:ComputeDistance(c1, z1, x1, y1, 3, 2, 0.899, 0.502);
		end
	else
		dist, xDist, yDist = self:ComputeDistance(c1, z1, x1, y1, c2, z2, x2, y2);
	end
	local angle = atan2(xDist, -yDist);
	if ( angle > 0 ) then
		angle = twoPi - angle;
	else
		angle = -angle;
	end
	local direction = deg(GetPlayerFacing() - angle) % 360;
	return direction, deg(-angle) % 360;
end

--*****************************************************************************
-- This function will return the direction to the specified position.
--*****************************************************************************
function Astrolabe:GetHeadingToPosition( c2, z2, x2, y2 )
	local c1, z1, x1, y1 = self:GetCurrentPlayerPosition();

	if ( not c1 or not c2 or not z1 or not z2 or not x1 or not x2 or not y1 or not y2 ) or ( x1 <= 0 and y1 <= 0 ) or ( x2 <= 0 and y2 <= 0 ) then
		return nil;
	end
	if c1 == -1 or (c1 > 5 and not VirtualContinentIndexes[c1]) then return end
	if c2 == -1 or (c2 > 5 and not VirtualContinentIndexes[c2]) then return end

	local dist, xDist, yDist;
	if ( c1 == 3 and c2 ~= 3 ) or ( c1 ~= 3 and c2 == 3 ) then
		if ( c2 == 3 ) then
			dist, xDist, yDist = self:ComputeDistance(c1, z1, x1, y1, 2, 4, 0.55, 0.542);
		elseif ( c1 == 3 ) then
			dist, xDist, yDist = self:ComputeDistance(c1, z1, x1, y1, 3, 2, 0.899, 0.502);
		end
	else
		dist, xDist, yDist = self:ComputeDistance(c1, z1, x1, y1, c2, z2, x2, y2);
	end
	local angle = atan2(xDist, -yDist);
	if ( angle > 0 ) then
		angle = twoPi - angle;
	else
		angle = -angle;
	end
	local direction = deg(GetPlayerFacing() - angle) % 360;
	return direction, deg(-angle) % 360;
end

--*****************************************************************************
-- This function will do its utmost to retrieve some sort of valid position
-- for the specified unit, including changing the current map zoom (if needed).
-- Map Zoom is returned to its previous setting before this function returns.
--*****************************************************************************
function Astrolabe:GetUnitPosition( unit, noMapChange )
	if ( not UnitExists(unit) or not UnitIsPlayer(unit) or not (UnitInRaid(unit) or UnitInParty(unit)) ) then
		return;
	end
	local x, y = GetPlayerMapPosition(unit);
	local vC, vZ = self:GetCurrentVirtualMapContinent(), self:GetCurrentVirtualMapZone();

	if (vC == 1 and vZ == 17 and x > 0 and y > 0 and GetCurrentMapDungeonLevel() == 1 and not noMapChange) then -- Orgrimmar
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition("player");
		vC, vZ = self:GetCurrentVirtualMapContinent(), self:GetCurrentVirtualMapZone();
		if ( self.WorldMapVisible ) then
			if ( lastMapID > 0 ) then
				SetMapByID(lastMapID); -- set map zoom back to what it was before
				SetDungeonMapLevel(lastFloor);
			else
				SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
				if ( lastFloor > 0 ) then
					SetDungeonMapLevel(lastFloor);
				end
			end
		end
	end

	if ( x <= 0 and y <= 0 ) then
		if ( noMapChange ) then
			-- no valid position on the current map, and we aren't allowed
			-- to change map zoom, so return
			return;
		end
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition(unit);
		if ( x <= 0 and y <= 0 ) then
			SetMapZoom(GetCurrentMapContinent());
			x, y = GetPlayerMapPosition(unit);
			if ( x <= 0 and y <= 0 ) then
				-- we are in an instance or otherwise off the continent map
				if ( lastMapID > 0 ) then
					SetMapByID(lastMapID); -- set map zoom back to what it was before
					SetDungeonMapLevel(lastFloor);
				elseif ( IsInInstance() ) then
					SetMapToCurrentZone();
				else
					SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
					if ( lastFloor > 0 ) then
						SetDungeonMapLevel(lastFloor);
					end
				end
				return;
			end
		end
		local vC, vZ = self:GetCurrentVirtualMapContinent(), self:GetCurrentVirtualMapZone();
		if ( lastMapID > 0 ) then
			SetMapByID(lastMapID); -- set map zoom back to what it was before
			SetDungeonMapLevel(lastFloor);
		elseif ( IsInInstance() ) then
			SetMapToCurrentZone();
		else
			SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
			if ( lastFloor > 0 ) then
				SetDungeonMapLevel(lastFloor);
			end
		end
		return vC, vZ, x, y;
	end
	--return GetCurrentMapContinent(), GetCurrentMapZone(), x, y;
	return vC, vZ, x, y;
end

--*****************************************************************************
-- This function will do its utmost to retrieve some sort of valid position
-- for the specified unit, including changing the current map zoom (if needed).
-- However, if a monitored WorldMapFrame (See AstrolabeMapMonitor.lua) is
-- visible, then will simply return nil if the current zoom does not provide
-- a valid position for the player unit. Map Zoom is returned to its previous
-- setting before this function returns, if it was changed.
--*****************************************************************************
function Astrolabe:GetCurrentPlayerPosition( noMapChange )
	local x, y = GetPlayerMapPosition("player");
	local vC, vZ = self:GetCurrentVirtualMapContinent(), self:GetCurrentVirtualMapZone();

	if (vC == 1 and vZ == 17 and x > 0 and y > 0 and GetCurrentMapDungeonLevel() == 1 and not noMapChange) then -- Orgrimmar
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition("player");
		vC, vZ = self:GetCurrentVirtualMapContinent(), self:GetCurrentVirtualMapZone();
		if ( self.WorldMapVisible ) then
			if ( lastMapID > 0 ) then
				SetMapByID(lastMapID); -- set map zoom back to what it was before
				SetDungeonMapLevel(lastFloor);
			else
				SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
				if ( lastFloor > 0 ) then
					SetDungeonMapLevel(lastFloor);
				end
			end
		end
	end

	if ( x <= 0 and y <= 0 ) then
		if ( noMapChange ) then
			-- no valid position on the current map, and we aren't allowed
			-- to change map zoom, so return
			return;
		end
		local lastCont, lastZone = GetCurrentMapContinent(), GetCurrentMapZone();
		local lastMapID, lastFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel();
		SetMapToCurrentZone();
		x, y = GetPlayerMapPosition("player");
		if ( x <= 0 and y <= 0 ) then
			SetMapZoom(GetCurrentMapContinent());
			x, y = GetPlayerMapPosition("player");
			if ( x <= 0 and y <= 0 ) then
				-- we are in an instance or otherwise off the continent map
				if ( lastMapID > 0 ) then
					SetMapByID(lastMapID); -- set map zoom back to what it was before
					SetDungeonMapLevel(lastFloor);
				elseif ( IsInInstance() ) then
					SetMapToCurrentZone();
				else
					SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
					if ( lastFloor > 0 ) then
						SetDungeonMapLevel(lastFloor);
					end
				end
				return;
			end
		end
		local vC, vZ = self:GetCurrentVirtualMapContinent(), self:GetCurrentVirtualMapZone();
		if ( lastMapID > 0 ) then
			SetMapByID(lastMapID); -- set map zoom back to what it was before
			SetDungeonMapLevel(lastFloor);
		elseif ( IsInInstance() ) then
			SetMapToCurrentZone();
		else
			SetMapZoom(lastCont, lastZone); -- set map zoom back to what it was before
			if ( lastFloor > 0 ) then
				SetDungeonMapLevel(lastFloor);
			end
		end
		return vC, vZ, x, y;
	end
	--return GetCurrentMapContinent(), GetCurrentMapZone(), x, y;
	return vC, vZ, x, y;
end


--------------------------------------------------------------------------------------------------------------
-- Table Cache System
--------------------------------------------------------------------------------------------------------------

local tableCache = {};
tableCache["__mode"] = "v";
setmetatable(tableCache, tableCache);

local function GetTable( icon )
	if ( tableCache[icon] ) then
		return tableCache[icon];
	else
		local T = {};
		tableCache[icon] = T;
		return T;
	end
end


--------------------------------------------------------------------------------------------------------------
-- Minimap Icon Placement
--------------------------------------------------------------------------------------------------------------

--*****************************************************************************
-- local variables specifically for use in this section
--*****************************************************************************
local minimapRotationEnabled = false;
local minimapShape = false;

local minimapRotationOffset = GetPlayerFacing();


local function placeIconOnMinimap( minimap, minimapZoom, mapWidth, mapHeight, icon, dist, xDist, yDist )
	local mapDiameter;
	if ( Astrolabe.minimapOutside ) then
		mapDiameter = MinimapSize.outdoor[minimapZoom];
	else
		mapDiameter = MinimapSize.indoor[minimapZoom];
	end
	local mapRadius = mapDiameter / 2;
	local xScale = mapDiameter / mapWidth;
	local yScale = mapDiameter / mapHeight;
	local iconDiameter = ((icon:GetWidth() / 2) + 3) * xScale;
	local iconOnEdge = nil;
	local isRound = true;

	if ( minimapRotationEnabled ) then
		local sinTheta = sin(minimapRotationOffset)
		local cosTheta = cos(minimapRotationOffset)
		--[[
		Math Note
		The math that is acutally going on in the next 3 lines is:
			local dx, dy = xDist, -yDist
			xDist = (dx * cosTheta) + (dy * sinTheta)
			yDist = -((-dx * sinTheta) + (dy * cosTheta))

		This is because the origin for map coordinates is the top left corner
		of the map, not the bottom left, and so we have to reverse the vertical
		distance when doing the our rotation, and then reverse the result vertical
		distance because this rotation formula gives us a result with the origin based
		in the bottom left corner (of the (+, +) quadrant).
		The actual code is a simplification of the above.
		]]
		local dx, dy = xDist, yDist
		xDist = (dx * cosTheta) - (dy * sinTheta)
		yDist = (dx * sinTheta) + (dy * cosTheta)
	end

	if ( minimapShape and not (xDist == 0 or yDist == 0) ) then
		isRound = (xDist < 0) and 1 or 3;
		if ( yDist < 0 ) then
			isRound = minimapShape[isRound];
		else
			isRound = minimapShape[isRound + 1];
		end
	end

	-- for non-circular portions of the Minimap edge
	if not ( isRound ) then
		dist = max(abs(xDist), abs(yDist))
	end

	if ( (dist + iconDiameter) > mapRadius ) then
		-- position along the outside of the Minimap
		iconOnEdge = true;
		local factor = (mapRadius - iconDiameter) / dist;
		xDist = xDist * factor;
		yDist = yDist * factor;
	end

	if ( Astrolabe.IconsOnEdge[icon] ~= iconOnEdge ) then
		Astrolabe.IconsOnEdge[icon] = iconOnEdge;
		Astrolabe.IconsOnEdgeChanged = true;
	end

	icon:ClearAllPoints();
	icon:SetPoint("CENTER", minimap, "CENTER", xDist/xScale, -yDist/yScale);
end

function Astrolabe:PlaceIconOnMinimap( icon, continent, zone, xPos, yPos )
	-- check argument types
	argcheck(icon, 2, "table");
	assert(3, icon.SetPoint and icon.ClearAllPoints, "Usage Message");
	argcheck(continent, 3, "number");
	argcheck(zone, 4, "number", "nil");
	argcheck(xPos, 5, "number");
	argcheck(yPos, 6, "number");

	-- if the positining system is currently active, just use the player position used by the last incremental (or full) update
	-- otherwise, make sure we base our calculations off of the most recent player position (if one is available)
	local lC, lZ, lx, ly;
	if ( self.processingFrame:IsShown() ) then
		lC, lZ, lx, ly = unpack(self.LastPlayerPosition);
	else
		lC, lZ, lx, ly = self:GetCurrentPlayerPosition();
		if ( lC and lC >= 0 ) then
			local lastPosition = self.LastPlayerPosition;
			lastPosition[1] = lC;
			lastPosition[2] = lZ;
			lastPosition[3] = lx;
			lastPosition[4] = ly;
		else
			lC, lZ, lx, ly = unpack(self.LastPlayerPosition);
		end
	end

	local dist, xDist, yDist = self:ComputeDistance(lC, lZ, lx, ly, continent, zone, xPos, yPos);
	if not ( dist ) then
		--icon's position has no meaningful position relative to the player's current location
		return -1;
	end

	local iconData = GetTable(icon);
	if ( self.MinimapIcons[icon] ) then
		self.MinimapIcons[icon] = nil;
	else
		self.MinimapIconCount = self.MinimapIconCount + 1
	end

	AddedOrUpdatedIcons[icon] = iconData
	iconData.continent = continent;
	iconData.zone = zone;
	iconData.xPos = xPos;
	iconData.yPos = yPos;
	iconData.dist = dist;
	iconData.xDist = xDist;
	iconData.yDist = yDist;

	minimapRotationEnabled = GetCVar("rotateMinimap") ~= "0"
	if ( minimapRotationEnabled ) then
		minimapRotationOffset = GetPlayerFacing();
	end

	-- check Minimap Shape
	minimapShape = GetMinimapShape and ValidMinimapShapes[GetMinimapShape()];

	-- place the icon on the Minimap and :Show() it
	local map = Minimap
	placeIconOnMinimap(map, map:GetZoom(), map:GetWidth(), map:GetHeight(), icon, dist, xDist, yDist);
	icon:Show()

	-- We know this icon's position is valid, so we need to make sure the icon placement system is active.
	self.processingFrame:Show()

	return 0;
end

function Astrolabe:RemoveIconFromMinimap( icon )
	if not ( self.MinimapIcons[icon] ) then
		return 1;
	end
	AddedOrUpdatedIcons[icon] = nil
	self.MinimapIcons[icon] = nil;
	self.IconsOnEdge[icon] = nil;
	icon:Hide();

	local MinimapIconCount = self.MinimapIconCount - 1
	if ( MinimapIconCount <= 0 ) then
		-- no icons left to manage
		self.processingFrame:Hide()
		MinimapIconCount = 0 -- because I'm paranoid
	end
	self.MinimapIconCount = MinimapIconCount

	return 0;
end

function Astrolabe:RemoveAllMinimapIcons()
	self:DumpNewIconsCache()
	local MinimapIcons = self.MinimapIcons;
	local IconsOnEdge = self.IconsOnEdge;
	for k, v in pairs(MinimapIcons) do
		MinimapIcons[k] = nil;
		IconsOnEdge[k] = nil;
		k:Hide();
	end
	self.MinimapIconCount = 0
	self.processingFrame:Hide()
end

local lastZoom; -- to remember the last seen Minimap zoom level

-- local variables to track the status of the two update coroutines
local fullUpdateInProgress = true
local resetIncrementalUpdate = false
local resetFullUpdate = false

-- Incremental Update Code
do
	-- local variables to track the incremental update coroutine
	local incrementalUpdateCrashed = true
	local incrementalUpdateThread

	local function UpdateMinimapIconPositions( self )
		yield()

		while ( true ) do
			self:DumpNewIconsCache() -- put new/updated icons into the main datacache

			resetIncrementalUpdate = false -- by definition, the incremental update is reset if it is here

			local C, Z, x, y = self:GetCurrentPlayerPosition();
			if ( C and C >= 0 ) then
				local Minimap = Minimap;
				local lastPosition = self.LastPlayerPosition;
				local lC, lZ, lx, ly = unpack(lastPosition);

				minimapRotationEnabled = GetCVar("rotateMinimap") ~= "0"
				if ( minimapRotationEnabled ) then
					minimapRotationOffset = GetPlayerFacing();
				end

				-- check current frame rate
				local numPerCycle = min(50, GetFramerate() * (self.MinimapUpdateMultiplier or 1))

				-- check Minimap Shape
				minimapShape = GetMinimapShape and ValidMinimapShapes[GetMinimapShape()];

				if ( lC == C and lZ == Z and lx == x and ly == y ) then
					-- player has not moved since the last update
					if ( lastZoom ~= Minimap:GetZoom() or self.ForceNextUpdate or minimapRotationEnabled ) then
						local currentZoom = Minimap:GetZoom();
						lastZoom = currentZoom;
						local mapWidth = Minimap:GetWidth();
						local mapHeight = Minimap:GetHeight();
						numPerCycle = numPerCycle * 2
						local count = 0
						for icon, data in pairs(self.MinimapIcons) do
							placeIconOnMinimap(Minimap, currentZoom, mapWidth, mapHeight, icon, data.dist, data.xDist, data.yDist);

							count = count + 1
							if ( count > numPerCycle ) then
								count = 0
								yield()
								-- check if the incremental update cycle needs to be reset
								-- because a full update has been run
								if ( resetIncrementalUpdate ) then
									break;
								end
							end
						end
						self.ForceNextUpdate = false;
					end
				else
					local dist, xDelta, yDelta = self:ComputeDistance(lC, lZ, lx, ly, C, Z, x, y);
					if ( dist ) then
						local currentZoom = Minimap:GetZoom();
						lastZoom = currentZoom;
						local mapWidth = Minimap:GetWidth();
						local mapHeight = Minimap:GetHeight();
						local count = 0
						for icon, data in pairs(self.MinimapIcons) do
							local xDist = data.xDist - xDelta;
							local yDist = data.yDist - yDelta;
							local dist = sqrt(xDist*xDist + yDist*yDist);
							placeIconOnMinimap(Minimap, currentZoom, mapWidth, mapHeight, icon, dist, xDist, yDist);

							data.dist = dist;
							data.xDist = xDist;
							data.yDist = yDist;

							count = count + 1
							if ( count >= numPerCycle ) then
								count = 0
								yield()
								-- check if the incremental update cycle needs to be reset
								-- because a full update has been run
								if ( resetIncrementalUpdate ) then
									break;
								end
							end
						end
						if not ( resetIncrementalUpdate ) then
							lastPosition[1] = C;
							lastPosition[2] = Z;
							lastPosition[3] = x;
							lastPosition[4] = y;
						end
					else
						self:RemoveAllMinimapIcons()
						lastPosition[1] = C;
						lastPosition[2] = Z;
						lastPosition[3] = x;
						lastPosition[4] = y;
					end
				end
			else
				if not ( self.WorldMapVisible ) then
					self.processingFrame:Hide();
				end
			end

			-- if we've been reset, then we want to start the new cycle immediately
			if not ( resetIncrementalUpdate ) then
				yield()
			end
		end
	end

	function Astrolabe:UpdateMinimapIconPositions()
		if ( fullUpdateInProgress ) then
			-- if we're in the middle a a full update, we want to finish that first
			self:CalculateMinimapIconPositions()
		else
			if ( incrementalUpdateCrashed ) then
				incrementalUpdateThread = coroutine.wrap(UpdateMinimapIconPositions)
				incrementalUpdateThread(self) --initialize the thread
			end
			incrementalUpdateCrashed = true
			incrementalUpdateThread()
			incrementalUpdateCrashed = false
		end
	end
end

-- Full Update Code
do
	-- local variables to track the full update coroutine
	local fullUpdateCrashed = true
	local fullUpdateThread

	local function CalculateMinimapIconPositions( self )
		yield()

		while ( true ) do
			self:DumpNewIconsCache() -- put new/updated icons into the main datacache

			resetFullUpdate = false -- by definition, the full update is reset if it is here
			fullUpdateInProgress = true -- set the flag the says a full update is in progress

			local C, Z, x, y = self:GetCurrentPlayerPosition();
			if ( C and C >= 0 ) then
				minimapRotationEnabled = GetCVar("rotateMinimap") ~= "0"
				if ( minimapRotationEnabled ) then
					minimapRotationOffset = GetPlayerFacing();
				end

				-- check current frame rate
				local numPerCycle = GetFramerate() * (self.MinimapUpdateMultiplier or 1) * 2

				-- check Minimap Shape
				minimapShape = GetMinimapShape and ValidMinimapShapes[GetMinimapShape()];

				local currentZoom = Minimap:GetZoom();
				lastZoom = currentZoom;
				local Minimap = Minimap;
				local mapWidth = Minimap:GetWidth();
				local mapHeight = Minimap:GetHeight();
				local count = 0
				for icon, data in pairs(self.MinimapIcons) do
					local dist, xDist, yDist = self:ComputeDistance(C, Z, x, y, data.continent, data.zone, data.xPos, data.yPos);
					if ( dist ) then
						placeIconOnMinimap(Minimap, currentZoom, mapWidth, mapHeight, icon, dist, xDist, yDist);

						data.dist = dist;
						data.xDist = xDist;
						data.yDist = yDist;
					else
						self:RemoveIconFromMinimap(icon)
					end

					count = count + 1
					if ( count >= numPerCycle ) then
						count = 0
						yield()
						-- check if we need to restart due to the full update being reset
						if ( resetFullUpdate ) then
							break;
						end
					end
				end

				if not ( resetFullUpdate ) then
					local lastPosition = self.LastPlayerPosition;
					lastPosition[1] = C;
					lastPosition[2] = Z;
					lastPosition[3] = x;
					lastPosition[4] = y;

					resetIncrementalUpdate = true
				end
			else
				if not ( self.WorldMapVisible ) then
					self.processingFrame:Hide();
				end
			end

			-- if we've been reset, then we want to start the new cycle immediately
			if not ( resetFullUpdate ) then
				fullUpdateInProgress = false
				yield()
			end
		end
	end

	function Astrolabe:CalculateMinimapIconPositions( reset )
		if ( fullUpdateCrashed ) then
			fullUpdateThread = coroutine.wrap(CalculateMinimapIconPositions)
			fullUpdateThread(self) --initialize the thread
		elseif ( reset ) then
			resetFullUpdate = true
		end
		fullUpdateCrashed = true
		fullUpdateThread()
		fullUpdateCrashed = false

		-- return result flag
		if ( fullUpdateInProgress ) then
			return 1 -- full update started, but did not complete on this cycle

		else
			if ( resetIncrementalUpdate ) then
				return 0 -- update completed
			else
				return -1 -- full update did no occur for some reason
			end

		end
	end
end

function Astrolabe:GetDistanceToIcon( icon )
	local data = self.MinimapIcons[icon];
	if ( data ) then
		return data.dist, data.xDist, data.yDist;
	end
end

function Astrolabe:IsIconOnEdge( icon )
	return self.IconsOnEdge[icon];
end

function Astrolabe:GetDirectionToIcon( icon )
	local data = self.MinimapIcons[icon];
	if ( data ) then
		local dir = atan2(data.xDist, -(data.yDist))
		if ( dir > 0 ) then
			return twoPi - dir;
		else
			return -dir;
		end
	end
end

function Astrolabe:Register_OnEdgeChanged_Callback( func, ident )
	-- check argument types
	argcheck(func, 2, "function");

	self.IconsOnEdge_GroupChangeCallbacks[func] = ident;
end

--*****************************************************************************
-- INTERNAL USE ONLY PLEASE!!!
-- Calling this function at the wrong time can cause errors
--*****************************************************************************
function Astrolabe:DumpNewIconsCache()
	local MinimapIcons = self.MinimapIcons
	for icon, data in pairs(AddedOrUpdatedIcons) do
		MinimapIcons[icon] = data
		AddedOrUpdatedIcons[icon] = nil
	end
	-- we now need to restart any updates that were in progress
	resetIncrementalUpdate = true
	resetFullUpdate = true
end


--------------------------------------------------------------------------------------------------------------
-- World Map Icon Placement
--------------------------------------------------------------------------------------------------------------

function Astrolabe:PlaceIconOnWorldMap( worldMapFrame, icon, continent, zone, xPos, yPos )
	-- check argument types
	argcheck(worldMapFrame, 2, "table");
	assert(3, worldMapFrame.GetWidth and worldMapFrame.GetHeight, "Usage Message");
	argcheck(icon, 3, "table");
	assert(3, icon.SetPoint and icon.ClearAllPoints, "Usage Message");
	argcheck(continent, 4, "number");
	argcheck(zone, 5, "number", "nil");
	argcheck(xPos, 6, "number");
	argcheck(yPos, 7, "number");

	--local C, Z = GetCurrentMapContinent(), GetCurrentMapZone();
	local C, Z = self:GetCurrentVirtualMapCZ();
	local nX, nY = self:TranslateWorldMapPosition(continent, zone, xPos, yPos, C, Z);

	-- anchor and :Show() the icon if it is within the boundry of the current map, :Hide() it otherwise
	if ( nX and nY and (0 < nX and nX <= 1) and (0 < nY and nY <= 1) ) then
		icon:ClearAllPoints();
		icon:SetPoint("CENTER", worldMapFrame, "TOPLEFT", nX * worldMapFrame:GetWidth(), -nY * worldMapFrame:GetHeight());
		icon:Show();
	else
		icon:Hide();
	end
	return nX, nY;
end


--------------------------------------------------------------------------------------------------------------
-- Handler Scripts
--------------------------------------------------------------------------------------------------------------

function Astrolabe:OnEvent( frame, event )
	if ( event == "MINIMAP_UPDATE_ZOOM" ) then
		-- update minimap zoom scale
		local Minimap = Minimap;
		local curZoom = Minimap:GetZoom();
		if ( GetCVar("minimapZoom") == GetCVar("minimapInsideZoom") ) then
			if ( curZoom < 2 ) then
				Minimap:SetZoom(curZoom + 1);
			else
				Minimap:SetZoom(curZoom - 1);
			end
		end
		if ( GetCVar("minimapZoom")+0 == Minimap:GetZoom() ) then
			self.minimapOutside = true;
		else
			self.minimapOutside = false;
		end
		Minimap:SetZoom(curZoom);

		-- re-calculate all Minimap Icon positions
		if ( frame:IsVisible() ) then
			self:CalculateMinimapIconPositions(true);
		end

	elseif ( event == "PLAYER_LEAVING_WORLD" ) then
		frame:Hide(); -- yes, I know this is redunant
		self:RemoveAllMinimapIcons(); --dump all minimap icons
		-- TODO: when I uncouple the point buffer from Minimap drawing,
		--       I should consider updating LastPlayerPosition here

	elseif ( event == "PLAYER_ENTERING_WORLD" ) then
		frame:Show();
		if not ( frame:IsVisible() ) then
			-- do the minimap recalculation anyways if the OnShow script didn't execute
			-- this is done to ensure the accuracy of information about icons that were
			-- inserted while the Player was in the process of zoning
			self:CalculateMinimapIconPositions(true);
		end

	elseif ( event == "ZONE_CHANGED_NEW_AREA" ) then
		frame:Hide();
		frame:Show();

	end
end

function Astrolabe:OnUpdate( frame, elapsed )
	-- on-edge group changed call-backs
	if ( self.IconsOnEdgeChanged ) then
		self.IconsOnEdgeChanged = false;
		for func in pairs(self.IconsOnEdge_GroupChangeCallbacks) do
			pcall(func);
		end
	end

	self:UpdateMinimapIconPositions();
end

function Astrolabe:OnShow( frame )
	-- set the world map to a zoom with a valid player position
	if not ( self.WorldMapVisible ) then
		SetMapToCurrentZone();
	end
	local C, Z = Astrolabe:GetCurrentPlayerPosition();
	local mapID, mapFloor = GetCurrentMapAreaID(), GetCurrentMapDungeonLevel();
	if ( C and C >= 0 and C < 99 ) then
		SetMapZoom(C, Z);
		SetDungeonMapLevel(mapFloor);
	else
		frame:Hide();
		return
	end

	-- re-calculate minimap icon positions (if needed)
	if ( next(self.MinimapIcons) ) then
		self:CalculateMinimapIconPositions(true);
	else
		-- needed so that the cycle doesn't overwrite an updated LastPlayerPosition
		resetIncrementalUpdate = true;
	end

	if ( self.MinimapIconCount <= 0 ) then
		-- no icons left to manage
		frame:Hide();
	end
end

function Astrolabe:OnHide( frame )
	-- dump the new icons cache here
	-- a full update will performed the next time processing is re-actived
	self:DumpNewIconsCache()
end

-- called by AstrolabMapMonitor when all world maps are hidden
function Astrolabe:AllWorldMapsHidden()
	if ( IsLoggedIn() ) then
		self.processingFrame:Hide();
		self.processingFrame:Show();
	end
end


--------------------------------------------------------------------------------------------------------------
-- Library Registration
--------------------------------------------------------------------------------------------------------------

local function activate( newInstance, oldInstance )
	if ( oldInstance ) then -- this is an upgrade activate
		if ( oldInstance.DumpNewIconsCache ) then
			oldInstance:DumpNewIconsCache()
		end
		for k, v in pairs(oldInstance) do
			if ( type(v) ~= "function" and (not configConstants[k]) ) then
				newInstance[k] = v;
			end
		end
		-- sync up the current MinimapIconCount value
		local iconCount = 0
		for _ in pairs(newInstance.MinimapIcons) do
			iconCount = iconCount + 1
		end
		newInstance.MinimapIconCount = iconCount

		Astrolabe = oldInstance;
	else
		local frame = CreateFrame("Frame");
		newInstance.processingFrame = frame;

		--newInstance.ContinentList = { GetMapContinents() };
		newInstance.ContinentList = Astrolabe:GetMapVirtualContinents();
		for C in pairs(newInstance.ContinentList) do
			--local zones = { GetMapZones(C) };
			local zones = Astrolabe:GetMapVirtualZones(C);
			newInstance.ContinentList[C] = zones;
			--zones[0] = Astrolabe:GetMapTexture(C, 0);
			for Z in ipairs(zones) do
				--SetMapZoom(C, Z);
				--zones[Z] = GetMapInfo();
				zones[Z] = Astrolabe:GetMapTexture(C, Z);
			end
		end
	end
	configConstants = nil -- we don't need this anymore

	local frame = newInstance.processingFrame;
	frame:Hide();
	frame:SetParent("Minimap");
	frame:UnregisterAllEvents();
	frame:RegisterEvent("MINIMAP_UPDATE_ZOOM");
	frame:RegisterEvent("PLAYER_LEAVING_WORLD");
	frame:RegisterEvent("PLAYER_ENTERING_WORLD");
	frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	frame:SetScript("OnEvent",
		function( frame, event, ... )
			Astrolabe:OnEvent(frame, event, ...);
		end
	);
	frame:SetScript("OnUpdate",
		function( frame, elapsed )
			Astrolabe:OnUpdate(frame, elapsed);
		end
	);
	frame:SetScript("OnShow",
		function( frame )
			Astrolabe:OnShow(frame);
		end
	);
	frame:SetScript("OnHide",
		function( frame )
			Astrolabe:OnHide(frame);
		end
	);

	setmetatable(Astrolabe.MinimapIcons, MinimapIconsMetatable)
end

--DongleStub:Register(Astrolabe, activate)


--------------------------------------------------------------------------------------------------------------
-- Data
--------------------------------------------------------------------------------------------------------------

-- diameter of the Minimap in game yards at
-- the various possible zoom levels
MinimapSize = {
	indoor = {
		[0] = 300, -- scale
		[1] = 240, -- 1.25
		[2] = 180, -- 5/3
		[3] = 120, -- 2.5
		[4] = 80,  -- 3.75
		[5] = 50,  -- 6
	},
	outdoor = {
		[0] = 466 + 2/3, -- scale
		[1] = 400,       -- 7/6
		[2] = 333 + 1/3, -- 1.4
		[3] = 266 + 2/6, -- 1.75
		[4] = 200,       -- 7/3
		[5] = 133 + 1/3, -- 3.5
	},
}

ValidMinimapShapes = {
	-- { upper-left, lower-left, upper-right, lower-right }
	["SQUARE"]                = { false, false, false, false },
	["CORNER-TOPLEFT"]        = { true,  false, false, false },
	["CORNER-TOPRIGHT"]       = { false, false, true,  false },
	["CORNER-BOTTOMLEFT"]     = { false, true,  false, false },
	["CORNER-BOTTOMRIGHT"]    = { false, false, false, true },
	["SIDE-LEFT"]             = { true,  true,  false, false },
	["SIDE-RIGHT"]            = { false, false, true,  true },
	["SIDE-TOP"]              = { true,  false, true,  false },
	["SIDE-BOTTOM"]           = { false, true,  false, true },
	["TRICORNER-TOPLEFT"]     = { true,  true,  true,  false },
	["TRICORNER-TOPRIGHT"]    = { true,  false, true,  true },
	["TRICORNER-BOTTOMLEFT"]  = { true,  true,  false, true },
	["TRICORNER-BOTTOMRIGHT"] = { false, true,  true,  true },
}

-- distances across and offsets of the world maps
-- in game yards
WorldMapSize = {
	-- World Map of Azeroth
	[0] = {
		parentContinent = 0,
		height = 31809.64857610083,
		width = 47714.278579261,
	},
	-- Kalimdor
	{ -- [1]
		parentContinent = 0,
		height = 24533.025279205,
		width = 36800.210572494,
		xOffset = -8590.40725049343,
		yOffset = 5628.692856102324,
		zoneData = {
			Ashenvale = {
				height = 3843.722331667447,
				width = 5766.728885829694,
				xOffset = 15366.76675592628,
				yOffset = 8126.925930315996,
			},
			Aszhara = {
				height = 3677.0833740234,
				width = 5514.5834960938,
				xOffset = 20439.737000518,
				yOffset = 7418.5979507852,
			},
			AzuremystIsle = {
				height = 2714.563862990522,
				width = 4070.87719998905,
				xOffset = 9966.708003150136,
				yOffset = 5460.278492344226,
			},
			Barrens = {
				height = 3831.2498779297,
				width = 5745.8333282471,
				xOffset = 16864.698280546,
				yOffset = 10989.40571763,
			},
			BloodmystIsle = {
				height = 2174.984213312164,
				width = 3262.535628257626,
				xOffset = 9541.702868577344,
				yOffset = 3424.87645454774,
			},
			Darkshore = {
				height = 4310.4162597656,
				width = 6464.5830078125,
				xOffset = 14050.085765229,
				yOffset = 4576.9520219014,
			},
			Darnassus = {
				height = 1027.0830078125,
				width = 1539.5833740234,
				xOffset = 13878.316212362,
				yOffset = 2336.0346670859,
			},
			Desolace = {
				height = 2997.895112390325,
				width = 4495.882561567951,
				xOffset = 12833.40568925697,
				yOffset = 12347.7291386989,
			},
			Durotar = {
				height = 3524.975103516492,
				width = 5287.556393263762,
				xOffset = 19029.30641831177,
				yOffset = 10991.48866520544,
			},
			Dustwallow = {
				height = 3499.975024621409,
				width = 5250.057340719384,
				xOffset = 18041.79555525144,
				yOffset = 14833.12785083746,
			},
			Felwood = {
				height = 4041.6662597656,
				width = 6062.4996337891,
				xOffset = 15268.847988646,
				yOffset = 5562.3615602437,
			},
			Feralas = {
				height = 4633.300220031075,
				width = 6950.07476479001,
				xOffset = 11625.05968863682,
				yOffset = 15166.45859885191,
			},
			Moonglade = {
				height = 1539.572004392277,
				width = 2308.359613919473,
				xOffset = 18448.04941280923,
				yOffset = 4308.203668830446,
			},
			Mulgore = {
				height = 3633.3332519531,
				width = 5449.9997558594,
				xOffset = 14862.594565223,
				yOffset = 12968.55871978,
			},
			Orgrimmar = {
				height = 1159.5834960938,
				width = 1739.375,
				xOffset = 20573.157720553,
				yOffset = 10313.683827378,
				--[[{
					height = 1159.5834960938,
					width = 1739.375,
					xOffset = 0,
					yOffset = 0,
				},
				{
					height = 241.390014648,
					width = 362.08984375,
					xOffset = 657.61624286067,
					yOffset = 554.39673413128,
				},]]--
			},
			Silithus = {
				height = 2706.2495117188,
				width = 4058.3332519531,
				xOffset = 14083.419134063,
				yOffset = 18672.684414654,
			},
			StonetalonMountains = {
				height = 3933.3331298828,
				width = 5899.9998779297,
				xOffset = 13164.659183301,
				yOffset = 9395.6679598463,
			},
			Tanaris = {
				height = 4808.3330078125,
				width = 7212.4995155334,
				xOffset = 17129.285130567,
				yOffset = 18570.603104987,
			},
			Teldrassil = {
				height = 3916.6665039063,
				width = 5874.9997558594,
				xOffset = 12831.322565239,
				yOffset = 951.97764575606,
			},
			TheExodar = {
				height = 704.6827795715492,
				width = 1056.782908333002,
				xOffset = 10533.08067788734,
				yOffset = 6276.205943683332,
			},
			ThousandNeedles = {
				height = 2933.311990587089,
				width = 4400.046926738385,
				xOffset = 17500.12330544135,
				yOffset = 16766.44742072096,
			},
			ThunderBluff = {
				height = 695.8286363220358,
				width = 1043.761162566134,
				xOffset = 16550.11296988024,
				yOffset = 13649.80296445508,
			},
			UngoroCrater = {
				height = 2466.648940830691,
				width = 3700.039952384531,
				xOffset = 16533.4461782123,
				yOffset = 18766.43318304924,
			},
			Winterspring = {
				height = 4100,
				width = 6149.9998779297,
				xOffset = 18058.461209767,
				yOffset = 4006.1220725518,
			},
			SouthernBarrens = {
				height = 4941.6666564941,
				width = 7412.5,
				xOffset = 15710.519764424,
				yOffset = 12595.644363612,
			},
			Uldum = {
				height = 4129.1665039063,
				width = 6193.7497558594,
				xOffset = 14625.091992917,
				yOffset = 20828.919172879,
			},
			AhnQirajTheFallenKingdom = {
				height = 2700,
				width = 4049.9998321533,
				xOffset = 13175.076288313,
				yOffset = 20833.087112167,
			},
			Hyjal = {
				height = 2831.2497558594,
				width = 4245.8333740234,
				xOffset = 17995.960532844,
				yOffset = 6604.0207009745,
			},
		},
	},
	-- Eastern Kingdoms
	{ -- [2]
		parentContinent = 0,
		height = 27149.795290881,
		width = 40741.175327834,
		xOffset = 18542.31220836664,
		yOffset = 3585.574573158966,
		zoneData = {
			--[[Alterac = {
				height = 1866.674220583334,
				width = 2799.999464405289,
				xOffset = 17388.63468066122,
				yOffset = 9676.382149825957,
			},]]--
			Arathi = {
				height = 2318.7498474121,
				width = 3477.0832519531,
				xOffset = 19299.052685209,
				yOffset = 11318.055681133,
			},
			Badlands = {
				height = 2045.8330078125,
				width = 3070.8332519531,
				xOffset = 20074.052604802,
				yOffset = 17030.577386444,
			},
			BlastedLands = {
				height = 2441.666015625,
				width = 3662.5,
				xOffset = 19365.718693917,
				yOffset = 21759.76217955,
			},
			BurningSteppes = {
				height = 2099.9995117188,
				width = 3152.083190918,
				xOffset = 18636.552753943,
				yOffset = 18161.83187815,
			},
			DeadwindPass = {
				height = 1666.673717206878,
				width = 2499.999255461505,
				xOffset = 19005.30099399293,
				yOffset = 21043.09319963172,
			},
			DunMorogh = {
				height = 3264.5830078125,
				width = 4897.9165039063,
				xOffset = 16034.469039537,
				yOffset = 15118.06979273,
			},
			Duskwood = {
				height = 1800.007435102674,
				width = 2699.999451812027,
				xOffset = 17338.63474984946,
				yOffset = 20893.09262994406,
			},
			EasternPlaguelands = {
				height = 2687.510259086504,
				width = 4031.248684963022,
				xOffset = 20459.46800337001,
				yOffset = 7472.207074316265,
			},
			Elwynn = {
				height = 2314.592478810788,
				width = 3470.832795915813,
				xOffset = 16636.55146195304,
				yOffset = 19116.00248086271,
			},
			EversongWoods = {
				height = 3283.346090242183,
				width = 4925.00271131707,
				xOffset = 20259.46550654072,
				yOffset = 2534.687768168357,
			},
			Ghostlands = {
				height = 2200.008615840919,
				width = 3300.001914001321,
				xOffset = 21055.29898547313,
				yOffset = 5309.698628620597,
			},
			HillsbradFoothills = {
				height = 3241.6666259766,
				width = 4862.4998779297,
				xOffset = 16321.969009708,
				yOffset = 9695.1322450413,
			},
			Hinterlands = {
				height = 2566.6767425107,
				width = 3849.999302583992,
				xOffset = 19746.96759079755,
				yOffset = 9709.715638073398,
			},
			Ironforge = {
				height = 527.6066263822604,
				width = 790.625237342102,
				xOffset = 18885.55918004965,
				yOffset = 15745.64757909506,
			},
			LochModan = {
				height = 1839.589436540107,
				width = 2758.333078630792,
				xOffset = 20165.71744013867,
				yOffset = 15663.90644131906,
			},
			Redridge = {
				height = 1712.5,
				width = 2568.7498779297,
				xOffset = 19651.13467993,
				yOffset = 19691.003965435,
			},
			SearingGorge = {
				height = 1487.505327445583,
				width = 2231.249676776115,
				xOffset = 18494.88412729142,
				yOffset = 17276.41249042905,
			},
			SilvermoonCity = {
				height = 806.7736903384404,
				width = 1211.459296502504,
				xOffset = 22172.71642224908,
				yOffset = 3422.648306718702,
			},
			Silverpine = {
				height = 2800.0110500699,
				width = 4199.999060067367,
				xOffset = 14721.96859379216,
				yOffset = 9509.714862642681,
			},
			StormwindCity = {
				height = 1158.337650999629,
				width = 1737.500553362899,
				xOffset = 16449.05109973784,
				yOffset = 19172.25293704512,
			},
			StranglethornVale = {
				height = 4368.75,
				width = 6552.0830078125,
				xOffset = 15194.886118831,
				yOffset = 22141.013693329,
			},
			Sunwell = {
				height = 2218.75784157939,
				width = 3327.080984022923,
				xOffset = 21074.05125342849,
				yOffset = 7.594755912743345,
			},
			SwampOfSorrows = {
				height = 1672.9169921875,
				width = 2508.3330078125,
				xOffset = 20253.218601838,
				yOffset = 20711.842003102,
			},
			Tirisfal = {
				height = 3012.512329627232,
				width = 4518.747902731258,
				xOffset = 15138.6360714653,
				yOffset = 7338.872677268415,
			},
			Undercity = {
				height = 640.1066040851099,
				width = 959.3745478926886,
				xOffset = 17298.77542115219,
				yOffset = 9298.435370484816,
			},
			WesternPlaguelands = {
				height = 2866.677851772014,
				width = 4299.999720893135,
				xOffset = 17755.30124459509,
				yOffset = 7809.708293788776,
			},
			Westfall = {
				height = 2333.342511708478,
				width = 3499.999662793482,
				xOffset = 15155.30169114852,
				yOffset = 20576.42535247717,
			},
			Wetlands = {
				height = 2756.260945423485,
				width = 4135.416085415621,
				xOffset = 18561.55114967782,
				yOffset = 13324.31325114659,
			},
			StranglethornJungle = {
				height = 2733.3330078125,
				width = 4099.9998779297,
				xOffset = 16428.219351321,
				yOffset = 22193.096911925,
			},
			TheCapeOfStranglethorn = {
				height = 2631.25,
				width = 3945.8331298828,
				xOffset = 16063.634314135,
				yOffset = 23693.102902299,
			},
			TwilightHighlands = {
				height = 3514.5830078125,
				width = 5270.8330078125,
				xOffset = 20609.468564877,
				yOffset = 13332.646687987,
			},
			TolBarad = {
				height = 1343.75,
				width = 2014.5832924843,
				xOffset = 13361.553301229,
				yOffset = 13336.812720153,
			},
			TolBaradDailyArea = {
				height = 1224.9999389648,
				width = 1837.5,
				xOffset = 12959.46935857,
				yOffset = 12399.309974311,
			},
			RuinsofGilneas = {
				height = 2097.9166870117,
				width = 3145.8332519531,
				xOffset = 14732.386166816,
				yOffset = 11709.723251895,
			},
			RuinsofGilneasCity = {
				height = 593.74987792969,
				width = 889.58325195313,
				xOffset = 16243.50252909,
				yOffset = 12487.189966743,
			},
			Vashjir = {
				height = 4631.2497558594,
				width = 6945.8327636719,
				xOffset = 9417.8037103957,
				yOffset = 14897.235908085,
			},
			VashjirDepths = {
				height = 2716.6665039063,
				width = 4075,
				xOffset = 9938.63657123,
				yOffset = 16082.657393663,
			},
			VashjirKelpForest = {
				height = 1868.7502441406,
				width = 2802.0830078125,
				xOffset = 13101.135676054,
				yOffset = 15195.153988728,
			},
			VashjirRuins = {
				height = 3233.3330078125,
				width = 4849.9996337891,
				xOffset = 11490.719911088,
				yOffset = 15932.656794897,
			},
		},
	},
	-- Outland
	{ -- [3]
		parentContinent = 3,
		height = 11642.355227091,
		width = 17463.987300595,
		zoneData = {
			BladesEdgeMountains = {
				height = 3616.553525584605,
				width = 5424.971374542539,
				xOffset = 4150.184588602209,
				yOffset = 1412.982196881336,
			},
			Hellfire = {
				height = 3443.64230460125,
				width = 5164.556244744065,
				xOffset = 7456.417231266903,
				yOffset = 4339.973859432732,
			},
			Nagrand = {
				height = 3683.218433421437,
				width = 5524.971116484553,
				xOffset = 2700.192056890117,
				yOffset = 5779.512082963144,
			},
			Netherstorm = {
				height = 3716.550667470386,
				width = 5574.970542741407,
				xOffset = 7512.666973902843,
				yOffset = 365.0979868806522,
			},
			ShadowmoonValley = {
				height = 3666.551832578994,
				width = 5499.971055470069,
				xOffset = 8770.993482940312,
				yOffset = 7769.033432511459,
			},
			ShattrathCity = {
				height = 870.8062268244973,
				width = 1306.243111124071,
				xOffset = 6860.744657085816,
				yOffset = 7295.086006462451,
			},
			TerokkarForest = {
				height = 3599.887549731843,
				width = 5399.971897226099,
				xOffset = 5912.67529110344,
				yOffset = 6821.146327166267,
			},
			Zangarmarsh = {
				height = 3351.978661481413,
				width = 5027.057239215307,
				xOffset = 3521.020775148071,
				yOffset = 3885.821395736634,
			},
		},
	},
	-- Northrend
	{ -- [4]
		parentContinent = 0,
		height = 11834.3119870532,
		width = 17751.3962441049,
		xOffset = 16020.94044398222,
		yOffset = 454.2451915717977,
		zoneData = {
			BoreanTundra = {
				height = 3843.764953143499,
				width = 5764.582303295793,
				xOffset = 646.3192474426043,
				yOffset = 5695.48114050537,
			},
			CrystalsongForest = {
				height = 1814.590295101352,
				width = 2722.916513743646,
				xOffset = 7773.401390128443,
				yOffset = 4091.308181657137,
			},
			Dalaran = {
				height = 553.3418567935553,
				width = 830.0149393375432,
				xOffset = 8164.641313001377,
				yOffset = 4526.723129621716,
				--[[{
					height = 553.3418567935553,
					width = 830.0149393375432,
					xOffset = 0,
					yOffset = 0,
				},
				{
					height = 375.489746094,
					width = 563.223999023,
					xOffset = 136.60434786015,
					yOffset = 91.314884748835,
				},]]--
			},
			Dragonblight = {
				height = 3739.598062842169,
				width = 5608.332396545997,
				xOffset = 5590.068422600026,
				yOffset = 5018.394866268677,
			},
			GrizzlyHills = {
				height = 3500.013689934217,
				width = 5249.998732532693,
				xOffset = 10327.56786162186,
				yOffset = 5076.728707808831,
			},
			HrothgarsLanding = {
				height = 2452.093653509858,
				width = 3677.082560623348,
				xOffset = 6419.234857391856,
				yOffset = -187.8757232657943,
			},
			HowlingFjord = {
				height = 4031.265457002443,
				width = 6045.831836878359,
				xOffset = 10615.0679627145,
				yOffset = 7476.73831512609,
			},
			IcecrownGlacier = {
				height = 4181.266519840844,
				width = 6270.832975322177,
				xOffset = 3773.401695036191,
				yOffset = 1166.296622984233,
			},
			LakeWintergrasp = {
				height = 1983.341134082619,
				width = 2974.99948105957,
				xOffset = 4887.98528918423,
				yOffset = 4876.727878058311,
			},
			SholazarBasin = {
				height = 2904.178067737769,
				width = 4356.249510482578,
				xOffset = 2287.985538503677,
				yOffset = 3305.888591396293,
			},
			TheStormPeaks = {
				height = 4741.684740041381,
				width = 7112.498187401986,
				xOffset = 7375.484940713573,
				yOffset = 395.46058562991,
			},
			ZulDrak = {
				height = 3329.179762967791,
				width = 4993.749118857795,
				xOffset = 9817.151095677416,
				yOffset = 2924.637042390465,
			},
		},
	},
	-- Maelstrom
	{ -- [5]
		parentContinent = 0,
		height = 8662,
		width = 12993,
		xOffset = 18302.592749507,
		yOffset = 10766.404238242,
		zoneData = {
			Deepholm = {
				height = 3399.9998779297,
				width = 5099.9998779297,
				xOffset = 4287.69,
				yOffset = 866.2,
			},
			Kezan = {
				height = 900.00048828125,
				width = 1352.083190918,
				xOffset = 4207.445232,
				yOffset = 5023.96,
			},
			TheLostIsles = {
				height = 3010.4166564941,
				width = 4514.5830078125,
				xOffset = -699.0234,
				yOffset = 2950.2772,
			},
			TheMaelstrom = {
				height = 1033.3332519531,
				width = 1550,
				xOffset = -389.79,
				yOffset = 866.2,
			},
		},
	},
	-- The Nexus
	[101] = { -- [TheNexus]
		parentContinent = 4,
		height = 0,
		width = 0,
		xOffset = 18252.726704011,
		yOffset = 7147.7529981817,
		zoneData = {
			["TheNexus:1"] = {
				height = 734.1875,
				width = 1101.2809753418,
				xOffset = -398.7137847557,
				yOffset = -646.05000250875,
			},
		},
	},
	-- The Oculus
	[102] = { -- [Nexus80]
		parentContinent = 4,
		height = 1733.3332214356,
		width = 2600,
		xOffset = 18253.8714,
		yOffset = 7176.16034,
		zoneData = {
			["Nexus80:1"] = {
				height = 343.13897705078,
				width = 514.70697021484,
				xOffset = -315.53895231889,
				yOffset = -163.25000430062,
			},
			["Nexus80:2"] = {
				height = 443.13897705078,
				width = 664.70697021484,
				xOffset = -389.45094105409,
				yOffset = -315.94895818699,
			},
			["Nexus80:3"] = {
				height = 343.13897705078,
				width = 514.70697021484,
				xOffset = -314.38261597962,
				yOffset = -212.95070618259,
			},
			["Nexus80:4"] = {
				height = 196.46398925781,
				width = 294.70098876953,
				xOffset = -204.220,
				yOffset = -130.340,
			},
		},
	},
	-- Ahn'kahet: The Old Kingdom
	[103] = { -- [Ahnkahet]
		parentContinent = 4,
		height = 647.91661071777,
		width = 972.91667175293,
		xOffset = 23208.819644693,
		yOffset = 7407.2948811612,
		zoneData = {
			["Ahnkahet:1"] = {
				height = 648.2790222168,
				width = 972.41796875,
				xOffset = -875.05796697108,
				yOffset = -512.94802667527,
			},
		},
	},
	-- Azjol-Nerub
	[104] = { -- [AzjolNerub]
		parentContinent = 4,
		height = 714.58329772949,
		width = 1072.9166450501,
		xOffset = 23066.061727501,
		yOffset = 7376.1862035569,
		zoneData = {
			["AzjolNerub:1"] = {
				height = 501.98300170898,
				width = 752.97399902344,
				xOffset = 74.238040763612,
				yOffset = -378.52772551455,
			},
			["AzjolNerub:2"] = {
				height = 195.31597900391,
				width = 292.97399902344,
				xOffset = 101.97,
				yOffset = -234.24,
			},
			["AzjolNerub:3"] = {
				height = 245,
				width = 367.5,
				xOffset = -34.677979554981,
				yOffset = -228.62999916077,
			},
		},
	},
	-- Drak'Tharon Keep
	[105] = { -- [DrakTharonKeep]
		parentContinent = 4,
		height = 418.75,
		width = 627.08331298828,
		xOffset = 27262.671177062,
		yOffset = 6273.152140419,
		zoneData = {
			["DrakTharonKeep:1"] = {
				height = 413.29399108887,
				width = 619.94100952148,
				xOffset = -182.13199662963,
				yOffset = -334.58404208043,
			},
			["DrakTharonKeep:2"] = {
				height = 413.29399108887,
				width = 619.94100952148,
				xOffset = -107.13199857747,
				yOffset = -334.58404208043,
			},
		},
	},
	-- Gundrak
	[106] = { -- [Gundrak]
		parentContinent = 4,
		height = 762.49987792969,
		width = 1143.7499694824,
		xOffset = 29638.575306563,	-- 29899.77624653
		yOffset = 4075.5891101248,	-- 4346.61462744
		zoneData = {
			["Gundrak:1"] = {
				height = 603.35009765625,
				width = 905.03305053711,
				xOffset = -508.11149835287,	-- -337.32189852067
				yOffset = -170.67309128723,	-- -172.84178995357
			},
		},
	},
	-- Utgarde Keep
	[107] = { -- [UtgardeKeep]
		parentContinent = 4,
		height = 0,
		width = 0,
		xOffset = 30098.241668311,
		yOffset = 9812.9131575594,
		zoneData = {
			["UtgardeKeep:1"] = {
				height = 489.72150039673,
				width = 734.58099365234,
				xOffset = -511.85817705072,
				yOffset = -364.86096800601,
			},
			["UtgardeKeep:2"] = {
				height = 320.72029304504,
				width = 481.08100891113,
				xOffset = -330.51139638553,
				yOffset = -153.89352757821,
			},
			["UtgardeKeep:3"] = {
				height = 491.05451202393,
				width = 736.58100891113,
				xOffset = -313.33490525675,
				yOffset = -265.38054647718,
			},
		},
	},
	-- Utgarde Pinnacle
	[108] = { -- [UtgardePinnacle]
		parentContinent = 4,
		height = 4366.6665039063,
		width = 6549.9995117188,
		xOffset = 30098.241668311,
		yOffset = 9812.9131575594,
		zoneData = {
			["UtgardePinnacle:1"] = {
				height = 365.95701599121,
				width = 548.93601989746,
				xOffset = -184.61036997062,
				yOffset = 36.778913047326,
			},
			["UtgardePinnacle:2"] = {
				height = 504.1190032959,
				width = 756.17994308472,
				xOffset = -336.62784792167,
				yOffset = -72.1799906544570,
			},
		},
	},
	-- Halls of Stone
	[109] = { -- [Ulduar77]
		parentContinent = 4,
		height = 2266.6666641235,
		width = 3399.9998168945,
		xOffset = 26206.541817527,
		yOffset = 2125.2421182473,
		zoneData = {
			["Ulduar77:1"] = {
				height = 613.46606445313,
				width = 920.1960144043,
				xOffset = -314.08435509596,
				yOffset = -222.78479061825,
			},
		},
	},
	-- Halls of Lightning
	[110] = { -- [HallsofLightning]
		parentContinent = 4,
		height = 2266.6666641235,
		width = 3399.9999389648,
		xOffset = 26624.292583363,
		yOffset = 1862.7488603731,
		zoneData = {
			["HallsofLightning:1"] = {
				height = 377.48999023438,
				width = 566.23501586914,
				xOffset = -10.36029020663,
				yOffset = -202.87999860556,
			},
			["HallsofLightning:2"] = {
				height = 472.16003417969,
				width = 708.23701477051,
				xOffset = 102.065168869,
				yOffset = -100.36008728027,
			},
		},
	},
	-- The Obsidian Sanctum
	[111] = { -- [TheObsidianSanctum]
		parentContinent = 4,
		height = 775,
		width = 1162.499917984,
		xOffset = 24976.875901726,
		yOffset = 7604.3720817432,
		xOffsetEntrance = -751.23499051315,
		yOffsetEntrance = -388.54180801421,
	},
	-- The Violet Hold
	[112] = { -- [VioletHold]
		parentContinent = 4,
		height = 256.25,
		width = 383.33331298828,
		xOffset = 24755.607721443,
		yOffset = 5370.73292695750,
		zoneData = {
			["VioletHold:1"] = {
				height = 170.82006835938,
				width = 256.22900390625,
				xOffset = -118.2020007577,
				yOffset = -170.7835670742,
			},
		},
	},
	-- The Culling of Stratholme
	[113] = { -- [CoTStratholme]
		parentContinent = 1,
		height = 1216.6665039063,
		width = 1824.9999389648,
		xOffset = 12938.988936299,
		yOffset = 27185.849747891,
		zoneData = {
			["CoTStratholme:1"] = {
				height = 750.19995117188,
				width = 1125.299987793,
				xOffset = -980.6495236531,
				yOffset = -535.9063076975,
			},
			["CoTStratholme:2"] = {
				height = 1216.6665039063,
				width = 1824.9999389648,
				xOffset = -1370,
				yOffset = -1480,
			},
		},
	},
	-- The Eye of Eternity
	[114] = { -- [TheEyeofEternity]
		parentContinent = 4,
		height = 2266.6666641235,
		width = 3399.9998168945,
		xOffset = 18253.875610297,
		yOffset = 7177.2994559144,
		zoneData = {
			["TheEyeofEternity:1"] = {
				height = 286.71301269531,
				width = 430.07006835938,
				xOffset = -165.10241957714,
				yOffset = -143.62469072987,
			},
		},
	},
	-- Naxxramas
	[115] = { -- [Naxxramas]
		parentContinent = 4,
		height = 1237.5,
		width = 1856.2497558594,
		xOffset = 26508.615913457,
		yOffset = 7376.7126872491,
		zoneData = {
			["Naxxramas:1"] = {	-- construct quarter
				height = 729.21997070313,
				width = 1093.830078125,
				xOffset = -793.99040482804,
				yOffset = -610.0476326656,
			},
			["Naxxramas:2"] = {	-- arachnid quarter
				height = 729.21997070313,
				width = 1093.830078125,
				xOffset = -294.14620887355,
				yOffset = -610.08986771775,
			},
			["Naxxramas:3"] = {	-- military quarter
				height = 800,
				width = 1200,
				xOffset = -847.25957693975,
				yOffset = -130.41481740333,
			},
			["Naxxramas:4"] = {	-- plague quarter
				height = 800.21997070313,
				width = 1200.330078125,
				xOffset = -348.2453619332,
				yOffset = -131.0746379848,
			},
			["Naxxramas:5"] = {	-- overview 53.34 49.68
				height = 1379.8798828125,
				width = 2069.8098144531,
				xOffset = -1104.0786225178,
				yOffset = -685.53662924611,
			},
			["Naxxramas:6"] = {	-- lair
				height = 437.2900390625,
				width = 655.93994140625,
				xOffset = -482.96758902606,
				yOffset = -318.25537347762,
			},
		},
	},
	-- Vault of Archavon
	[116] = { -- [VaultofArchavon]
		parentContinent = 4,
		height = 1733.3332519531,
		width = 2599.9998779297,
		xOffset = 22398.08079125,
		yOffset = 5557.59456903460,
		zoneData = {
			["VaultofArchavon:1"] = {
				height = 932.17001342773,
				width = 1398.2550048828,
				xOffset = -688.80303442448,
				yOffset = -812.00758375764,
			},
		},
	},
	-- The Argent Coliseum
	[117] = { -- [TheArgentColiseum]
		parentContinent = 4,
		height = 1733.3333435059,
		width = 2599.9999694824,
		xOffset = 24497.972582364,
		yOffset = 2532.4907318311,
		zoneData = {
			["TheArgentColiseumRaid:1"] = {
				height = 246.65798950195,
				width = 369.9861869812,
				xOffset = -252.50470230154,
				yOffset = -129.4489068985,
			},
			["TheArgentColiseumRaid:2"] = {
				height = 493.33001708984,
				width = 739.99601745606,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	--[[
	CurrentPlayerPosition: 117 1 0.51151859760284 0.52488881349564
	AbsoluteContinentPosition: 4 24445.818782507 2542.9385888315
	Distance: 229.30 189.25481549705 129.4680194489
	]]--
	-- Ulduar
	[118] = { -- [Ulduar]
		parentContinent = 4,
		height = 2191.6666259766,
		width = 3287.4998779297,
		xOffset = 26353.120051859,
		yOffset = 1685.1416748984,
		zoneData = {
			["Ulduar:1"] = {
				height = 2191.6666259766,
				width = 3287.4998779297,
				xOffset = -352.77753439988,
				yOffset = -425.62928323496,
			},
			["Ulduar:2"] = {
				height = 446.30004882813,
				width = 669.45098876953,
				xOffset = 993.57796380721 ,
				yOffset = -571.06979680704,
			},
			["Ulduar:3"] = {
				height = 885.63989257813,
				width = 1328.4609985352,
				xOffset = 564.06105318676,
				yOffset = -1294.6863104789,
			},
			["Ulduar:4"] = {
				height = 607,
				width = 910.5,
				xOffset = 623.1974292763,
				yOffset = -949.00687048118,
			},
			["Ulduar:5"] = {
				height = 1046.3000488281,
				width = 1569.4599609375,
				xOffset = 251.27542,
				yOffset = -1898.6741,
			},
			["Ulduar:6"] = {
				height = 412.97998046875,
				width = 619.46899414063,
				xOffset = 910.8092964638,
				yOffset = -969.73797790306,
			},
		},
	},
	-- The Forge of Souls
	[119] = { -- [TheForgeofSouls]
		parentContinent = 4,
		height = 7599.9997558594,
		width = 11399.999511719,
		xOffset = 23238.497769551,
		yOffset = 5373.53821046460,
		zoneData = {
			["TheForgeofSouls:1"] = {
				height = 965.400390625,
				width = 1448.0998535156,
				xOffset = -962.87741973881 ,
				yOffset = -855.487774733340,
			},
		},
	},
	-- Pit of Saron
	[120] = { -- [PitofSaron]
		parentContinent = 4,
		height = 1022.9166717529,
		width = 1533.3333129883,
		xOffset = 23231.155790278,
		yOffset = 5459.76559746780,
		xOffsetEntrance = -627.1700338753,
		yOffsetEntrance = -830.6435545608,
	},
	-- Halls of Reflection
	[121] = { -- [HallsofReflection]
		parentContinent = 4,
		height = 8666.6665039063,
		width = 12999.999511719,
		xOffset = 23272.848051034,
		yOffset = 5419.6559122592,
		zoneData = {
			["HallsofReflection:1"] = {
				height = 586.01953125,
				width = 879.02001953125,
				xOffset = -424.50000749258,
				yOffset = -481.460455209720,
			},
		},
	},
	-- Icecrown Citadel
	[122] = { -- [IcecrownCitadel]
		parentContinent = 4,
		height = 8133.3330078125,
		width = 12199.999511719,
		xOffset = 23171.618591557,
		yOffset = 5268.9580486912,
		zoneData = {
			["IcecrownCitadel:1"] = {
				height = 903.64703369141,
				width = 1355.4700927734,
				xOffset = -528.34206050172,
				yOffset = -57.599667196246,
			},
			["IcecrownCitadel:2"] = {
				height = 711.33369064331,
				width = 1067,
				xOffset = -485.4778402481231,
				yOffset = 114.3667867819734,
			},
			["IcecrownCitadel:3"] = {
				height = 130.31500244141,
				width = 195.46997070313,
				xOffset = -99.277873590826,
				yOffset = 521.0314690142,
			},
			["IcecrownCitadel:4"] = {
				height = 515.81030273438,
				width = 773.71008300781,
				xOffset = 3.1649684898152,
				yOffset = 128.59836601656,
			},
			["IcecrownCitadel:5"] = {
				height = 765.82006835938,
				width = 1148.7399902344,
				xOffset = -594.35504568133,
				yOffset = -111.41142633053,
			},
			["IcecrownCitadel:6"] = {
				height = 249.1298828125,
				width = 373.7099609375,
				xOffset = -189.84329545114,
				yOffset = -48.081833499295,
			},
			["IcecrownCitadel:7"] = {
				height = 195.50701904297,
				width = 293.26000976563,
				xOffset = -145.55672591196,
				yOffset = 197.47741798375,
			},
			["IcecrownCitadel:8"] = {
				height = 165.28799438477,
				width = 247.92993164063,
				xOffset = -121.79620067646,
				yOffset = 215.4067650174,
			},
		},
	},
	-- The Ruby Sanctum
	[123] = { -- [TheRubySanctum]
		parentContinent = 4,
		height = 502.08325195313,
		width = 752.08331298828,
		xOffset = 25050.564502793,
		yOffset = 7440.0503248611,
		xOffsetEntrance = -368.58630133223,
		yOffsetEntrance = -145.3547314767,
	},
	-- The Argent Coliseum
	[124] = { -- [TheArgentColiseum]
		parentContinent = 4,
		height = 1733.3333435059,
		width = 2599.9999694824,
		xOffset = 24497.972582364,
		yOffset = 2532.4907318311,
		zoneData = {
			["TheArgentColiseum:1"] = {
				height = 246.65798950195,
				width = 369.9861869812,
				xOffset = -252.50470230154,
				yOffset = -129.4489068985,
			},
		},
	},
	-- Onyxia's Lair
	[125] = { -- [OnyxiasLair]
		parentContinent = 1,
		height = 322.0787887573,
		width = 483.1179885864,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Scarlet Enclave
	[99] = { -- [ScarletEnclave]
		parentContinent = 2,
		height = 2108.3333740234,
		width = 3162.5,
		xOffset = 0,
		yOffset = 0,
	},
	-- Halls of Origination
	[201] = { -- [HallsofOrigination]
		parentContinent = 1,
		height = 2800,
		width = 4199.99975586,
		xOffset = 10314.176617442,
		yOffset = 28645.750205162,
		zoneData = {
			["HallsofOrigination:1"] = {
				height = 1021.16730547,
				width = 1531.75097656,
				xOffset = -769.74822931902,
				yOffset = -957.1238302207,
			},
			["HallsofOrigination:2"] = {
				height = 848.503425598,
				width = 1272.7550354,
				xOffset = 0,
				yOffset = 0,
			},
			["HallsofOrigination:3"] = {
				height = 752.512664795,
				width = 1128.76898193,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Vortex Pinnacle
	[202] = { -- [Skywall]
		parentContinent = 1,
		height = 2800,
		width = 4199.99975586,
		xOffset = 10792.145790942,
		yOffset = 29948.953211042,
		zoneData = {
			["Skywall:1"] = {
				height = 1345.81802368,
				width = 2018.72503662,
				xOffset = -1083.5383334177,
				yOffset = -217.26647498656,
			},
		},
	},
	-- The Stonecore
	[203] = { -- [TheStonecore]
		parentContinent = 5,
		height = 2266.66666412,
		width = 3399.99981689,
		xOffset = 25023.907095926,
		yOffset = 13398.895532272,
		zoneData = {
			["TheStonecore:1"] = {
				height = 878.086975098,
				width = 1317.1289978,
				xOffset = -714.6905873759,
				yOffset = -851.4960468624,
			},
		},
	},
	-- Throne of the Tides
	[204] = { -- [ThroneofTides]
		parentContinent = 2,
		height = 1733.33325195,
		width = 2599.99996948,
		xOffset = 31312.889640643,
		yOffset = 20346.975642234,
		zoneData = {
			["ThroneofTides:1"] = {
				height = 665.447998047,
				width = 998.171936035,
				xOffset = -499.4660264448,
				yOffset = -634.1955481236,
			},
			["ThroneofTides:2"] = {
				height = 665.447998047,
				width = 998.171936035,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Grim Batol
	[205] = { -- [GrimBatol]
		parentContinent = 2,
		height = 2266.66662598,
		width = 3399.99987793,
		xOffset = 40160.403276569,
		yOffset = 18809.671660333,
		zoneData = {
			["GrimBatol:1"] = {
				height = 579.364990234,
				width = 869.047431946,
				xOffset = -91.614663744952,
				yOffset = -326.26721210604,
			},
		},
	},
	-- Blackrock Caverns
	[206] = { -- [BlackrockCaverns]
		parentContinent = 2,
		height = 1733.33325195,
		width = 2599.99996948,
		xOffset = 38043.649279693,
		yOffset = 22332.358622046,
		zoneData = {
			["BlackrockCaverns:1"] = {
				height = 679.672319412,
				width = 1019.50793457,
				xOffset = -319.26278725462,
				yOffset = -471.22695291844,
			},
			["BlackrockCaverns:2"] = {
				height = 679.672319412,
				width = 1019.50793457,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Lost City of the Tol'vir
	[207] = { -- [LostCityofTolvir]
		parentContinent = 1,
		height = 647.916992188,
		width = 970.833251953,
		xOffset = 9785.0743177007,
		yOffset = 29114.273758087,
		xOffsetEntrance = -305.60988258547,
		yOffsetEntrance = -97.67478756457,
	},
	-- Throne of the Four Winds
	[208] = { -- [ThroneoftheFourWinds]
		parentContinent = 1,
		height = 1733.33325195,
		width = 2599.99996948,
		xOffset = 8406.4391275987,
		yOffset = 29788.2414352,
		zoneData = {
			["ThroneoftheFourWinds:1"] = {
				height = 1000,
				width = 1500,
				xOffset = -708.50712060924,
				yOffset = -770.07097005844,
			},
		},
	},
	-- The Bastion of Twilight
	[209] = { -- [TheBastionofTwilight]
		parentContinent = 2,
		height = 2266.66662598,
		width = 3399.99987793,
		xOffset = 40940.001499276,
		yOffset = 19662.040602142,
		zoneData = {
			["TheBastionofTwilight:1"] = {
				height = 718.889984131,
				width = 1078.33402252,
				xOffset = -424.50070878585,
				yOffset = -394.12934824443,
			},
			["TheBastionofTwilight:2"] = {
				height = 518.894958496,
				width = 778.343017578,
				xOffset = 0,
				yOffset = 0,
			},
			["TheBastionofTwilight:3"] = {
				height = 694.894958496,
				width = 1042.34202576,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Deadmines
	[210] = { -- [TheDeadmines]
		parentContinent = 2,
		height = 3333.33325195,
		width = 4999.99987793,
		xOffset = 35035.001272858,
		yOffset = 25969.958408249,
		zoneData = {
			["TheDeadmines:1"] = {
				height = 372.842502594,
				width = 559.264007568,
				xOffset = -150.27034069657,
				yOffset = -49.979319579234,
			},
			["TheDeadmines:2"] = {
				height = 332.842300415,
				width = 499.263000488,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Shadowfang Keep
	[211] = { -- [ShadowfangKeep]
		parentContinent = 2,
		height = 2799.99987793,
		width = 4200,
		xOffset = 35143.230345709,
		yOffset = 14993.219203314,
		zoneData = {
			["ShadowfangKeep:1"] = {
				height = 234.953392029,
				width = 352.430053711,
				xOffset = -246.94603183455,
				yOffset = -144.25739674209,
			},
			["ShadowfangKeep:2"] = {
				height = 141.617996216,
				width = 212.426757812,
				xOffset = 0,
				yOffset = 0,
			},
			["ShadowfangKeep:3"] = {
				height = 101.6199646,
				width = 152.429931641,
				xOffset = 0,
				yOffset = 0,
			},
			["ShadowfangKeep:4"] = {
				height = 101.624694824,
				width = 152.429931641,
				xOffset = 0,
				yOffset = 0,
			},
			["ShadowfangKeep:5"] = {
				height = 101.624694824,
				width = 152.429931641,
				xOffset = 0,
				yOffset = 0,
			},
			["ShadowfangKeep:6"] = {
				height = 132.286605835,
				width = 198.429931641,
				xOffset = 0,
				yOffset = 0,
			},
			["ShadowfangKeep:7"] = {
				height = 181.619903565,
				width = 272.429931641,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Blackwing Descent
	[212] = { -- [BlackwingDescent]
		parentContinent = 2,
		height = 1733.33325195,
		width = 2599.99987793,
		xOffset = 37903.838332099,
		yOffset = 22303.285324892,
		zoneData = {
			["BlackwingDescent:1"] = {
				height = 566.46234107,
				width = 849.694015503,
				xOffset = -398.6660625067,
				yOffset = -358.85779832246,
			},
			["BlackwingDescent:2"] = {
				height = 666.462005615,
				width = 999.692977905,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Baradin Hold
	[213] = { -- [BaradinHold]
		parentContinent = 2,
		height = 1733.33325195,
		width = 2599.99991989,
		xOffset = 32864.377805975,
		yOffset = 17633.747066145,
		zoneData = {
			["BaradinHold:1"] = {
				height = 390,
				width = 585,
				xOffset = -280.35791069269,
				yOffset = -362.43395447731,
			},
		},
	},
	-- Firelands
	[214] = { -- [Firelands]
		parentContinent = 1,
		height = 1058.33328247,
		width = 1587.49993896,
		xOffset = 11401.094968142,
		yOffset = 14441.098366647,
		zoneData = {
			["Firelands:1"] = {
				height = 1058.33328247,
				width = 1587.49993896,
				xOffset = 0,
				yOffset = 0,
			},
			["Firelands:2"] = {
				height = 250,
				width = 375,
				xOffset = 0,
				yOffset = 0,
			},
			["Firelands:3"] = {
				height = 960,
				width = 1440,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Molten Front
	[215] = { -- [MoltenFront]
		parentContinent = 1,
		height = 793.749938964844,
		width = 1189.58331298828,
		xOffset = 10573.273518747,
		yOffset = 13829.44495846,
		xOffsetEntrance = -630.32142981719,
		yOffsetEntrance = -664.92064101696,
	},
	-- Darkmoon Island
	[216] = { -- [DarkmoonFaireIsland]
		parentContinent = 2,
		height = 1239.58349609,
		width = 1858.3330078,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- The Nexus Legendary
	[217] = { -- [TheNexusLegendary]
		parentContinent = 4,
		height = 0,
		width = 0,
		xOffset = 18252.726704011,
		yOffset = 7147.7529981817,
		zoneData = {
			["TheNexusLegendary:1"] = {
				height = 734.189697265625,
				width = 1101.2841796875,
				xOffset = -398.7137847557,
				yOffset = -646.05000250875,
			},
		},
	},
	-- Dragon Soul
	[218] = { -- [DragonSoul]
		parentContinent = 4,
		height = 2063.06518555,
		width = 3106.70849609,
		xOffset = 12989.00544057,
		yOffset = 26691.356640456,
		zoneData = {
			["DragonSoul:1"] = {
				height = 2063.06518555,
				width = 3106.70849609,
				xOffset = -1553.3542480447,
				yOffset = -1749.4792773467,
			},
			["DragonSoul:2"] = {
				height = 265,
				width = 397.5,
				xOffset = 670.69703757808,
				yOffset = 689.5720671062,
			},
			["DragonSoul:3"] = {
				height = 285,
				width = 427.5,
				xOffset = 447.5530645932,
				yOffset = -689.57196959638,
			},
			["DragonSoul:4"] = {
				height = 123.466796875,
				width = 185.19921875,
				xOffset = -75.267567358124,
				yOffset = -516.05199082137,
			},
			["DragonSoul:5"] = {
				height = 1,
				width = 1.5,
				xOffset = 0,
				yOffset = -587.97357788205,
			},
			["DragonSoul:6"] = {
				height = 1,
				width = 1.5,
				xOffset = 0,
				yOffset = -587.97357788205,
			},
			["DragonSoul:7"] = {
				height = 738.900390625,
				width = 1108.3515625,
				xOffset = 11051.839277749,
				yOffset = -10701.004830795,
			},
		},
	},
	-- The Temple of Atal'Hakkar
	[221] = { -- [TheTempleOfAtalHakkar]
		parentContinent = 2,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheTempleOfAtalHakkar:1"] = {
				height = 463.352981567,
				width = 695.028991699,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Dire Maul
	[222] = { -- [DireMaul]
		parentContinent = 1,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["DireMaul:1"] = {
				height = 850,
				width = 1275,
				xOffset = 0,
				yOffset = 0,
			},
			["DireMaul:2"] = {
				height = 350,
				width = 525,
				xOffset = 0,
				yOffset = 0,
			},
			["DireMaul:3"] = {
				height = 325,
				width = 487.5,
				xOffset = 0,
				yOffset = 0,
			},
			["DireMaul:4"] = {
				height = 500,
				width = 750,
				xOffset = 0,
				yOffset = 0,
			},
			["DireMaul:5"] = {
				height = 533.333999634,
				width = 800.000801086,
				xOffset = 0,
				yOffset = 0,
			},
			["DireMaul:6"] = {
				height = 650,
				width = 975,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Blackfathom Deeps
	[223] = { -- [BlackfathomDeeps]
		parentContinent = 1,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["BlackfathomDeeps:1"] = {
				height = 589.47997284,
				width = 884.220001221,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackfathomDeeps:2"] = {
				height = 589.480010986,
				width = 884.220031738,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackfathomDeeps:3"] = {
				height = 189.482666016,
				width = 284.224004269,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Blackrock Depths
	[224] = { -- [BlackrockDepths]
		parentContinent = 2,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["BlackrockDepths:1"] = {
				height = 938.040756226,
				width = 1407.06097412,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackrockDepths:2"] = {
				height = 1004.70742798,
				width = 1507.06097412,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Molten Core
	[225] = { -- [MoltenCore]
		parentContinent = 2,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["MoltenCore:1"] = {
				height = 843.199066162,
				width = 1264.80006409,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Blackrock Spire
	[226] = { -- [BlackrockSpire]
		parentContinent = 2,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["BlackrockSpire:1"] = {
				height = 591.226013184,
				width = 886.839014053,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackrockSpire:2"] = {
				height = 591.226013184,
				width = 886.839014053,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackrockSpire:3"] = {
				height = 591.226013184,
				width = 886.839014053,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackrockSpire:4"] = {
				height = 591.226013184,
				width = 886.839014053,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackrockSpire:5"] = {
				height = 591.226013184,
				width = 886.839014053,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackrockSpire:6"] = {
				height = 591.226013184,
				width = 886.839014053,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackrockSpire:7"] = {
				height = 591.226013184,
				width = 886.839014053,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Blackwing Lair
	[227] = { -- [BlackwingLair]
		parentContinent = 2,
		height = 2266.66699219,
		width = 3399.99981689,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["BlackwingLair:1"] = {
				height = 332.949707031,
				width = 499.428039551,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackwingLair:2"] = {
				height = 432.949707031,
				width = 649.427062988,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackwingLair:3"] = {
				height = 432.949707031,
				width = 649.427062988,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackwingLair:4"] = {
				height = 432.949707031,
				width = 649.427062988,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Uldaman
	[228] = { -- [Uldaman]
		parentContinent = 2,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["Uldaman:1"] = {
				height = 595.778991699,
				width = 893.668014526,
				xOffset = 0,
				yOffset = 0,
			},
			["Uldaman:2"] = {
				height = 328.380493164,
				width = 492.570419312,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Stratholme
	[229] = { -- [Stratholme]
		parentContinent = 2,
		height = 2266.66650391,
		width = 3399.99987793,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["Stratholme:1"] = {
				height = 470.479980469,
				width = 705.719970703,
				xOffset = 0,
				yOffset = 0,
			},
			["Stratholme:2"] = {
				height = 670.480224609,
				width = 1005.72045898,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Scholomance
	[230] = { -- [Scholomance]
		parentContinent = 2,
		height = 1200,
		width = 1799.99993896,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["Scholomance:1"] = {
				height = 213.364997864,
				width = 320.048904419,
				xOffset = 0,
				yOffset = 0,
			},
			["Scholomance:2"] = {
				height = 293.366405487,
				width = 440.049011231,
				xOffset = 0,
				yOffset = 0,
			},
			["Scholomance:3"] = {
				height = 273.385799408,
				width = 410.0779953,
				xOffset = 0,
				yOffset = 0,
			},
			["Scholomance:4"] = {
				height = 354.028198242,
				width = 531.042007446,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Razorfen Kraul
	[231] = { -- [RazorfenKraul]
		parentContinent = 1,
		height = 1200,
		width = 1799.99987793,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["RazorfenKraul:1"] = {
				height = 490.959838867,
				width = 736.449951172,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Razorfen Downs
	[232] = { -- [RazorfenDowns]
		parentContinent = 1,
		height = 2266.66656494,
		width = 3399.99981689,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["RazorfenDowns:1"] = {
				height = 472.699951172,
				width = 709.048950195,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Maraudon
	[233] = { -- [Maraudon]
		parentContinent = 1,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["Maraudon:1"] = {
				height = 650,
				width = 975,
				xOffset = 0,
				yOffset = 0,
			},
			["Maraudon:2"] = {
				height = 1091.66600037,
				width = 1637.5,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Zul'Farrak
	[234] = { -- [ZulFarrak]
		parentContinent = 1,
		height = 922.916625976,
		width = 1383.33322144,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Wailing Caverns
	[235] = { -- [WailingCaverns]
		parentContinent = 1,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["WailingCaverns:1"] = {
				height = 624.315994263,
				width = 936.475006103,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Ragefire Chasm
	[236] = { -- [Ragefire]
		parentContinent = 1,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["Ragefire:1"] = {
				height = 492.576202393,
				width = 738.864013672,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Ahn'Qiraj
	[237] = { -- [AhnQiraj]
		parentContinent = 1,
		height = 1200,
		width = 1800,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["AhnQiraj:1"] = {
				height = 1851.69042969,
				width = 2777.54411316,
				xOffset = 0,
				yOffset = 0,
			},
			["AhnQiraj:2"] = {
				height = 651.700195312,
				width = 977.559936523,
				xOffset = 0,
				yOffset = 0,
			},
			["AhnQiraj:3"] = {
				height = 385.040039062,
				width = 577.559936523,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Ruins of Ahn'Qiraj
	[238] = { -- [RuinsofAhnQiraj]
		parentContinent = 1,
		height = 1675,
		width = 2512.49987793,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Gnomeregan
	[241] = { -- [Gnomeregan]
		parentContinent = 2,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["Gnomeregan:1"] = {
				height = 513.111999512,
				width = 769.667999268,
				xOffset = 0,
				yOffset = 0,
			},
			["Gnomeregan:2"] = {
				height = 513.111999512,
				width = 769.667999268,
				xOffset = 0,
				yOffset = 0,
			},
			["Gnomeregan:3"] = {
				height = 579.778015137,
				width = 869.667999268,
				xOffset = 0,
				yOffset = 0,
			},
			["Gnomeregan:4"] = {
				height = 579.779998779,
				width = 869.669708252,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Scarlet Monastery
	[242] = { -- [ScarletMonastery]
		parentContinent = 2,
		height = 3333.33325195,
		width = 4999.99987793,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["ScarletMonastery:1"] = {
				height = 413.322753906,
				width = 619.983947754,
				xOffset = 0,
				yOffset = 0,
			},
			["ScarletMonastery:2"] = {
				height = 213.460494995,
				width = 320.190994263,
				xOffset = 0,
				yOffset = 0,
			},
			["ScarletMonastery:3"] = {
				height = 408.459960938,
				width = 612.696609497,
				xOffset = 0,
				yOffset = 0,
			},
			["ScarletMonastery:4"] = {
				height = 468.866699219,
				width = 703.300048828,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Stockade
	[243] = { -- [TheStockade]
		parentContinent = 2,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheStockade:1"] = {
				height = 252.102495194,
				width = 378.152999878,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Zul'Gurub
	[244] = { -- [ZulGurub]
		parentContinent = 2,
		height = 1414.58300781,
		width = 2120.83325195,
		xOffset = 37935.094626597,
		yOffset = 26678.097206426,
		xOffsetEntrance = -609.57350577266,
		yOffsetEntrance = -691.10642736947,
	},
	-- Zul'Aman
	[245] = { -- [ZulAman]
		parentContinent = 2,
		height = 845.833312988,
		width = 1268.74993896,
		xOffset = 42314.888699312,
		yOffset = 10310.639314698,
		xOffsetEntrance = -71.8025,
		yOffsetEntrance = -448.0047,
	},
	-- End Time
	[246] = { -- [EndTime]
		parentContinent = 1,
		height = 2197.91650391,
		width = 3295.83312988,
		xOffset = 12933.06146277,
		yOffset = 26715.716980634,
		zoneData = {
			["EndTime:1"] = {
				height = 2197.91650391,
				width = 3295.83312988,
				xOffset = -2684.041103436,
				yOffset = -967.1279295376,
			},
			["EndTime:2"] = {
				height = 375,
				width = 562.5,
				xOffset = 0,
				yOffset = 0,
			},
			["EndTime:3"] = {
				height = 577.080322266,
				width = 865.620544434,
				xOffset = 0,
				yOffset = 0,
			},
			["EndTime:4"] = {
				height = 316.666503906,
				width = 475,
				xOffset = 0,
				yOffset = 0,
			},
			["EndTime:5"] = {
				height = 464.58984375,
				width = 696.884765625,
				xOffset = 0,
				yOffset = 0,
			},
			["EndTime:6"] = {
				height = 302.08984375,
				width = 453.135009766,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Well of Eternity
	[247] = { -- [WellofEternity]
		parentContinent = 1,
		height = 833.33325195,
		width = 1252.0830078,
		xOffset = 12472.459212466,
		yOffset = 27023.76101687,
		xOffsetEntrance = -355.66356645649,
		yOffsetEntrance = -519.67675379063,
		--[[zoneData = {
			["WellofEternity:1"] = {
				height = 0,
				width = 0,
				xOffset = -355.66356645649,
				yOffset = -519.67675379063,
			},
		},]]--
	},
	-- Hour of Twilight
	[248] = { -- [HourofTwilight]
		parentContinent = 1,
		height = 2029.16650391,
		width = 3043.74987793,
		xOffset = 13066.291727991,
		yOffset = 26719.883843217,
		zoneData = {
			["HourofTwilight:1"] = {
				height = 2029.16650391,
				width = 3043.74987793,
				xOffset = -1460.3026147141,
				yOffset = -398.21042378595,
			},
			["HourofTwilight:2"] = {
				height = 250,
				width = 375,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Shattered Halls
	[301] = { -- [TheShatteredHalls]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheShatteredHalls:1"] = {
				height = 709.164993286,
				width = 1063.74746704,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Auchenai Crypts
	[302] = { -- [AuchenaiCrypts]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["AuchenaiCrypts:1"] = {
				height = 495.026992798,
				width = 742.540435791,
				xOffset = 0,
				yOffset = 0,
			},
			["AuchenaiCrypts:2"] = {
				height = 545.026992798,
				width = 817.540466309,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Sethekk Halls
	[303] = { -- [SethekkHalls]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["SethekkHalls:1"] = {
				height = 468.996994019,
				width = 703.495483398,
				xOffset = 0,
				yOffset = 0,
			},
			["SethekkHalls:2"] = {
				height = 468.996994019,
				width = 703.495483398,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Shadow Labyrinth
	[304] = { -- [ShadowLabyrinth]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["ShadowLabyrinth:1"] = {
				height = 561.014888763,
				width = 841.522354126,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Mana-Tombs
	[305] = { -- [ManaTombs]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["ManaTombs:1"] = {
				height = 548.856811523,
				width = 823.28515625,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Hellfire Ramparts
	[306] = { -- [HellfireRamparts]
		parentContinent = 3,
		height = 437.5,
		width = 658.333374024,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["HellfireRamparts:1"] = {
				height = 463.040039063,
				width = 694.560058594,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Blood Furnace
	[307] = { -- [TheBloodFurnace]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheBloodFurnace:1"] = {
				height = 669.012687683,
				width = 1003.51901245,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Magtheridon's Lair
	[308] = { -- [MagtheridonsLair]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["MagtheridonsLair:1"] = {
				height = 370.666694641,
				width = 556,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Underbog
	[309] = { -- [TheUnderbog]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheUnderbog:1"] = {
				height = 596.613357544,
				width = 894.919998169,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Steamvault
	[310] = { -- [TheSteamvault]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheSteamvault:1"] = {
				height = 584.509414673,
				width = 876.764007568,
				xOffset = 0,
				yOffset = 0,
			},
			["TheSteamvault:2"] = {
				height = 584.509414673,
				width = 876.764007568,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Slave Pens
	[311] = { -- [TheSlavePens]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheSlavePens:1"] = {
				height = 593.372070312,
				width = 890.058124542,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Serpentshrine Cavern
	[312] = { -- [CoilfangReservoir]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["CoilfangReservoir:1"] = {
				height = 1050.00201416,
				width = 1575.00297546,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Botanica
	[313] = { -- [TheBotanica]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheBotanica:1"] = {
				height = 504.934997559,
				width = 757.402481079,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Mechanar
	[314] = { -- [TheMechanar]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheMechanar:1"] = {
				height = 450.825401306,
				width = 676.238006592,
				xOffset = 0,
				yOffset = 0,
			},
			["TheMechanar:2"] = {
				height = 450.825366974,
				width = 676.238006592,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Arcatraz
	[315] = { -- [TheArcatraz]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TheArcatraz:1"] = {
				height = 459.789352417,
				width = 689.684020996,
				xOffset = 0,
				yOffset = 0,
			},
			["TheArcatraz:2"] = {
				height = 364.03201294,
				width = 546.048049927,
				xOffset = 0,
				yOffset = 0,
			},
			["TheArcatraz:3"] = {
				height = 424.45602417,
				width = 636.684005737,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Gruul's Lair
	[316] = { -- [GruulsLair]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["GruulsLair:1"] = {
				height = 350,
				width = 525,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Black Temple
	[317] = { -- [BlackTemple]
		parentContinent = 3,
		height = 522.916625976,
		width = 783.333343506,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["BlackTemple:1"] = {
				height = 834.833007812,
				width = 1252.24957848,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackTemple:2"] = {
				height = 650,
				width = 975,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackTemple:3"] = {
				height = 670,
				width = 1005,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackTemple:4"] = {
				height = 293.333984375,
				width = 440.000976562,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackTemple:5"] = {
				height = 446.666687012,
				width = 670,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackTemple:6"] = {
				height = 470,
				width = 705,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackTemple:7"] = {
				height = 236.666625977,
				width = 355,
				xOffset = 0,
				yOffset = 0,
			},
			["BlackTemple:8"] = {
				height = 522.916625976,
				width = 783.333343506,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Tempest Keep: The Eye
	[318] = { -- [TempestKeep]
		parentContinent = 3,
		height = 0,
		width = 0,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["TempestKeep:1"] = {
				height = 1050,
				width = 1575,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Karazhan
	[319] = { -- [Karazhan]
		parentContinent = 2,
		height = 1172.91699219,
		width = 1758.33337402,
		xOffset = 0,
		yOffset = 0,
		zoneData = {
			["Karazhan:1"] = {
				height = 366.69921875,
				width = 550.048828125,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:2"] = {
				height = 171.90625,
				width = 257.859863281,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:3"] = {
				height = 230.099609375,
				width = 345.149414062,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:4"] = {
				height = 346.69921875,
				width = 520.048828125,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:5"] = {
				height = 156.099609375,
				width = 234.149902344,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:6"] = {
				height = 387.69921875,
				width = 581.548828125,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:7"] = {
				height = 127.69921875,
				width = 191.548828125,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:8"] = {
				height = 92.900390625,
				width = 139.350585938,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:9"] = {
				height = 506.69921875,
				width = 760.048828125,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:10"] = {
				height = 300.166015625,
				width = 450.25,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:11"] = {
				height = 271.050048828,
				width = 180.69921875,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:12"] = {
				height = 396.69921875,
				width = 595.048828125,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:13"] = {
				height = 352.69921875,
				width = 529.048828125,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:14"] = {
				height = 163.5,
				width = 245.25,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:15"] = {
				height = 140.765625,
				width = 211.149902344,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:16"] = {
				height = 67.5,
				width = 101.25,
				xOffset = 0,
				yOffset = 0,
			},
			["Karazhan:17"] = {
				height = 227.5,
				width = 341.25,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- The Black Morass
	[320] = { -- [CoTTheBlackMorass]
		parentContinent = 1,
		height = 725,
		width = 1087.5,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Old Hillsbrad Foothills
	[321] = { -- [CoTHillsbradFoothills]
		parentContinent = 1,
		height = 1554.16662598,
		width = 2331.24993896,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Hyjal Summit
	[322] = { -- [CoTMountHyjal]
		parentContinent = 1,
		height = 1666.66650391,
		width = 2499.99975586,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Magisters' Terrace
	[341] = { -- [MagistersTerrace]
		parentContinent = 2,
		height = 364.583332062,
		width = 547.916641235,
		xOffset = 41655.453523467,
		yOffset = 4279.3106633567,
		zoneData = {
			["MagistersTerrace:1"] = {
				height = 353.555969238,
				width = 530.334014893,
				xOffset = -225.80362450752,
				yOffset = -330.02750940604,
			},
			["MagistersTerrace:2"] = {
				height = 353.555992127,
				width = 530.334014893,
				xOffset = -225.80362450752,
				yOffset = -330.02750940604,
			},
		},
	},
	-- Sunwell Plateau
	[342] = { -- [SunwellPlateau]
		parentContinent = 2,
		height = 604.166625977,
		width = 906.25,
		xOffset = 41088.997763127,
		yOffset = 4608.8022100767,
		zoneData = {
			["SunwellPlateau:1"] = {
				height = 604.166625977,
				width = 906.25,
				xOffset = -280.37745691836,
				yOffset = -217.52232089942,
			},
			["SunwellPlateau:2"] = {
				height = 310,
				width = 465,
				xOffset = 0,
				yOffset = 0,
			},
		},
	},
	-- Strand of the Ancients
	[401] = { -- [StrandoftheAncients]
		parentContinent = 4,
		height = 1162.4999389648,
		width = 1743.7499389648,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Arathi Basin
	[402] = { -- [ArathiBasin]
		parentContinent = 2,
		height = 1170.8332519531,
		width = 1756.249923706,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Warsong Gulch
	[403] = { -- [WarsongGulch]
		parentContinent = 1,
		height = 764.58331298828,
		width = 1145.8333129883,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Alterac Valley
	[404] = { -- [AlteracValley]
		parentContinent = 2,
		height = 2824.9998779297,
		width = 4237.4998779297,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Isle of Conquest
	[405] = { -- [IsleofConquest]
		parentContinent = 4,
		height = 1766.6665840149,
		width = 2650,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Eye of the Storm
	[406] = { -- [NetherstormArena]
		parentContinent = 3,
		height = 1514.5833740234,
		width = 2270.833190918,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- The Battle for Gilneas
	[407] = { -- [GilneasBattleground2]
		parentContinent = 2,
		height = 868.75,
		width = 1302.08328247,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Twin Peaks
	[408] = { -- [TwinPeaks]
		parentContinent = 2,
		height = 810.416503906,
		width = 1214.58325195,
		xOffset = 0,
		yOffset = 0,
		xOffsetEntrance = 0,
		yOffsetEntrance = 0,
	},
	-- Orgrimmar
	[1017] = { -- [Orgrimmar]
		parentContinent = 1,
		height = 1159.5834960938,
		width = 1739.375,
		xOffset = 11982.75047005957,
		yOffset = 15942.376683480324,
		zoneData = {
			["Orgrimmar:1"] = {
				height = 1159.5834960938,
				width = 1739.375,
				xOffset = 0,
				yOffset = 0,
			},
			["Orgrimmar:2"] = {
				height = 241.390014648,
				width = 362.08984375,
				xOffset = 657.61624286067,
				yOffset = 554.39673413128,
			},
		},
	},
	-- Dalaran
	[4003] = { -- [Dalaran]
		parentContinent = 4,
		height = 553.3418567935553,
		width = 830.0149393375432,
		xOffset = 24185.581756983597,
		yOffset = 4980.9683211935137,
		zoneData = {
			["Dalaran:1"] = {
				height = 553.3418567935553,
				width = 830.0149393375432,
				xOffset = 0,
				yOffset = 0,
			},
			["Dalaran:2"] = {
				height = 375.489746094,
				width = 563.223999023,
				xOffset = 136.60434786015,
				yOffset = 91.314884748835,
			},
		},
	},
}

-- Virtual Continents
VirtualContinentIndexes = { -- [Azeroth]
	--[0] = "Azeroth",
	--[1] = "Kalimdor",
	--[2] = "Eastern Kingdoms",
	--[3] = "Outland",
	--[4] = "Northrend",
	[99] = {
		continent = "Plaguelands: The Scarlet Enclave",
		mapTexture = "ScarletEnclave",
		mapFloors = 0,
		mapID = 502,
	},
	[101] = {
		continent = "The Nexus",
		mapTexture = "TheNexus",
		mapFloors = 1,
		mapID = 520,
	},
	[102] = {
		continent = "The Oculus",
		mapTexture = "Nexus80",
		mapFloors = 4,
		mapID = 528,
	},
	[103] = {
		continent = "Ahn'kahet: The Old Kingdom",
		mapTexture = "Ahnkahet",
		mapFloors = 1,
		mapID = 522,
	},
	[104] = {
		continent = "Azjol-Nerub",
		mapTexture = "AzjolNerub",
		mapFloors = 3,
		mapID = 533,
	},
	[105] = {
		continent = "Drak'Tharon Keep",
		mapTexture = "DrakTharonKeep",
		mapFloors = 2,
		mapID = 534,
	},
	[106] = {
		continent = "Gundrak",
		mapTexture = "Gundrak",
		mapFloors = 1,
		mapID = 530,
	},
	[107] = {
		continent = "Utgarde Keep",
		mapTexture = "UtgardeKeep",
		mapFloors = 3,
		mapID = 523,
	},
	[108] = {
		continent = "Utgarde Pinnacle",
		mapTexture = "UtgardePinnacle",
		mapFloors = 2,
		mapID = 524,
	},
	[109] = {
		continent = "Halls of Stone",
		mapTexture = "Ulduar77",
		mapFloors = 1,
		mapID = 526,
	},
	[110] = {
		continent = "Halls of Lightning",
		mapTexture = "HallsofLightning",
		mapFloors = 2,
		mapID = 525,
	},
	[111] = {
		continent = "The Obsidian Sanctum",
		mapTexture = "TheObsidianSanctum",
		mapFloors = 0,
		mapID = 531,
	},
	[112] = {
		continent = "The Violet Hold",
		mapTexture = "VioletHold",
		mapFloors = 1,
		mapID = 536,
	},
	[113] = {
		continent = "The Culling of Stratholme",
		mapTexture = "CoTStratholme",
		mapFloors = 2,
		mapID = 521,
	},
	[114] = {
		continent = "The Eye of Eternity",
		mapTexture = "TheEyeofEternity",
		mapFloors = 1,
		mapID = 527,
	},
	[115] = {
		continent = "Naxxramas",
		mapTexture = "Naxxramas",
		mapFloors = 6,
		mapID = 535,
	},
	[116] = {
		continent = "Vault of Archavon",
		mapTexture = "VaultofArchavon",
		mapFloors = 1,
		mapID = 532,
	},
	[117] = {
		--continent = "The Argent Coliseum",
		continent = "Trial of the Crusader",
		mapTexture = "TheArgentColiseumRaid",
		mapFloors = 2,
		mapID = 542,
	},
	[118] = {
		continent = "Ulduar",
		mapTexture = "Ulduar",
		mapFloors = 6,
		mapID = 529,
	},
	[119] = {
		continent = "The Forge of Souls",
		mapTexture = "TheForgeofSouls",
		mapFloors = 1,
		mapID = 601,
	},
	[120] = {
		continent = "Pit of Saron",
		mapTexture = "PitofSaron",
		mapFloors = 0,
		mapID = 602,
	},
	[121] = {
		continent = "Halls of Reflection",
		mapTexture = "HallsofReflection",
		mapFloors = 1,
		mapID = 603,
	},
	[122] = {
		continent = "Icecrown Citadel",
		mapTexture = "IcecrownCitadel",
		mapFloors = 8,
		mapID = 604,
	},
	[123] = {
		continent = "The Ruby Sanctum",
		mapTexture = "TheRubySanctum",
		mapFloors = 0,
		mapID = 609,
	},
	[124] = {
		--continent = "The Argent Coliseum",
		continent = "Trial of the Champion",
		mapTexture = "TheArgentColiseum",
		mapFloors = 1,
		mapID = 543,
	},
	[125] = {
		continent = "Onyxia's Lair",
		mapTexture = "OnyxiasLair",
		mapFloors = 0,
		mapID = 718,
	},
	[201] = {
		continent = "Halls of Origination",
		mapTexture = "HallsofOrigination",
		mapFloors = 3,
		mapID = 759,
	},
	[202] = {
		continent = "The Vortex Pinnacle",
		mapTexture = "Skywall",
		mapFloors = 1,
		mapID = 769,
	},
	[203] = {
		continent = "The Stonecore",
		mapTexture = "TheStonecore",
		mapFloors = 1,
		mapID = 768,
	},
	[204] = {
		continent = "Throne of the Tides",
		mapTexture = "ThroneofTides",
		mapFloors = 2,
		mapID = 767,
	},
	[205] = {
		continent = "Grim Batol",
		mapTexture = "GrimBatol",
		mapFloors = 1,
		mapID = 757,
	},
	[206] = {
		continent = "Blackrock Caverns",
		mapTexture = "BlackrockCaverns",
		mapFloors = 2,
		mapID = 753,
	},
	[207] = {
		continent = "Lost City of the Tol'vir",
		mapTexture = "LostCityofTolvir",
		mapFloors = 0,
		mapID = 747,
	},
	[208] = {
		continent = "Throne of the Four Winds",
		mapTexture = "ThroneoftheFourWinds",
		mapFloors = 1,
		mapID = 773,
	},
	[209] = {
		continent = "The Bastion of Twilight",
		mapTexture = "TheBastionofTwilight",
		mapFloors = 3,
		mapID = 758,
	},
	[210] = {
		continent = "The Deadmines",
		mapTexture = "TheDeadmines",
		mapFloors = 2,
		mapID = 756,
	},
	[211] = {
		continent = "Shadowfang Keep",
		mapTexture = "ShadowfangKeep",
		mapFloors = 7,
		mapID = 764,
	},
	[212] = {
		continent = "Blackwing Descent",
		mapTexture = "BlackwingDescent",
		mapFloors = 2,
		mapID = 754,
	},
	[213] = {
		continent = "Baradin Hold",
		mapTexture = "BaradinHold",
		mapFloors = 1,
		mapID = 752,
	},
	[214] = {
		continent = "Firelands",
		mapTexture = "Firelands",
		mapFloors = 3,
		mapID = 800,
	},
	[215] = {
		continent = "Molten Front",
		mapTexture = "MoltenFront",
		mapFloors = 0,
		mapID = 795,
	},
	[216] = {
		continent = "Darkmoon Island",
		mapTexture = "DarkmoonFaireIsland",
		mapFloors = 0,
		mapID = 823,
	},
	[217] = {
		continent = "The Nexus Legendary",
		mapTexture = "TheNexusLegendary",
		mapFloors = 1,
		mapID = 803,
	},
	[218] = {
		continent = "Dragon Soul",
		mapTexture = "DragonSoul",
		mapFloors = 7,
		mapID = 824,
	},
	[221] = {
		continent = "The Temple of Atal'Hakkar",
		mapTexture = "TheTempleOfAtalHakkar",
		mapFloors = 1,
		mapID = 687,
	},
	[222] = {
		continent = "Dire Maul",
		mapTexture = "DireMaul",
		mapFloors = 6,
		mapID = 699,
	},
	[223] = {
		continent = "Blackfathom Deeps",
		mapTexture = "BlackfathomDeeps",
		mapFloors = 3,
		mapID = 688,
	},
	[224] = {
		continent = "Blackrock Depths",
		mapTexture = "BlackrockDepths",
		mapFloors = 2,
		mapID = 704,
	},
	[225] = {
		continent = "Molten Core",
		mapTexture = "MoltenCore",
		mapFloors = 1,
		mapID = 696,
	},
	[226] = {
		continent = "Blackrock Spire",
		mapTexture = "BlackrockSpire",
		mapFloors = 7,
		mapID = 721,
	},
	[227] = {
		continent = "Blackwing Lair",
		mapTexture = "BlackwingLair",
		mapFloors = 4,
		mapID = 755,
	},
	[228] = {
		continent = "Uldaman",
		mapTexture = "Uldaman",
		mapFloors = 2,
		mapID = 692,
	},
	[229] = {
		continent = "Stratholme",
		mapTexture = "Stratholme",
		mapFloors = 2,
		mapID = 765,
	},
	[230] = {
		continent = "Scholomance",
		mapTexture = "Scholomance",
		mapFloors = 4,
		mapID = 763,
	},
	[231] = {
		continent = "Razorfen Kraul",
		mapTexture = "RazorfenKraul",
		mapFloors = 1,
		mapID = 761,
	},
	[232] = {
		continent = "Razorfen Downs",
		mapTexture = "RazorfenDowns",
		mapFloors = 1,
		mapID = 760,
	},
	[233] = {
		continent = "Maraudon",
		mapTexture = "Maraudon",
		mapFloors = 2,
		mapID = 750,
	},
	[234] = {
		continent = "Zul'Farrak",
		mapTexture = "ZulFarrak",
		mapFloors = 0,
		mapID = 686,
	},
	[235] = {
		continent = "Wailing Caverns",
		mapTexture = "WailingCaverns",
		mapFloors = 1,
		mapID = 749,
	},
	[236] = {
		continent = "Ragefire Chasm",
		mapTexture = "Ragefire",
		mapFloors = 1,
		mapID = 680,
	},
	[237] = {
		continent = "Ahn'Qiraj",
		mapTexture = "AhnQiraj",
		mapFloors = 3,
		mapID = 766,
	},
	[238] = {
		continent = "Ruins of Ahn'Qiraj",
		mapTexture = "RuinsofAhnQiraj",
		mapFloors = 0,
		mapID = 717,
	},
	[241] = {
		continent = "Gnomeregan",
		mapTexture = "Gnomeregan",
		mapFloors = 4,
		mapID = 691,
	},
	[242] = {
		continent = "Scarlet Monastery",
		mapTexture = "ScarletMonastery",
		mapFloors = 4,
		mapID = 762,
	},
	[243] = {
		continent = "The Stockade",
		mapTexture = "TheStockade",
		mapFloors = 1,
		mapID = 690,
	},
	[244] = {
		continent = "Zul'Gurub",
		mapTexture = "ZulGurub",
		mapFloors = 0,
		mapID = 793,
	},
	[245] = {
		continent = "Zul'Aman",
		mapTexture = "ZulAman",
		mapFloors = 0,
		mapID = 781,
	},
	[246] = {
		continent = "End Time",
		mapTexture = "EndTime",
		mapFloors = 6,
		mapID = 820,
	},
	[247] = {
		continent = "Well of Eternity",
		mapTexture = "WellofEternity",
		mapFloors = 0,
		mapID = 816,
	},
	[248] = {
		continent = "Hour of Twilight",
		mapTexture = "HourofTwilight",
		mapFloors = 2,
		mapID = 819,
	},
	[301] = {
		continent = "The Shattered Halls",
		mapTexture = "TheShatteredHalls",
		mapFloors = 1,
		mapID = 710,
	},
	[302] = {
		continent = "Auchenai Crypts",
		mapTexture = "AuchenaiCrypts",
		mapFloors = 2,
		mapID = 722,
	},
	[303] = {
		continent = "Sethekk Halls",
		mapTexture = "SethekkHalls",
		mapFloors = 2,
		mapID = 723,
	},
	[304] = {
		continent = "Shadow Labyrinth",
		mapTexture = "ShadowLabyrinth",
		mapFloors = 1,
		mapID = 724,
	},
	[305] = {
		continent = "Mana-Tombs",
		mapTexture = "ManaTombs",
		mapFloors = 1,
		mapID = 732,
	},
	[306] = {
		continent = "Hellfire Ramparts",
		mapTexture = "HellfireRamparts",
		mapFloors = 1,
		mapID = 797,
	},
	[307] = {
		continent = "The Blood Furnace",
		mapTexture = "TheBloodFurnace",
		mapFloors = 1,
		mapID = 725,
	},
	[308] = {
		continent = "Magtheridon's Lair",
		mapTexture = "MagtheridonsLair",
		mapFloors = 1,
		mapID = 779,
	},
	[309] = {
		continent = "The Underbog",
		mapTexture = "TheUnderbog",
		mapFloors = 1,
		mapID = 726,
	},
	[310] = {
		continent = "The Steamvault",
		mapTexture = "TheSteamvault",
		mapFloors = 2,
		mapID = 727,
	},
	[311] = {
		continent = "The Slave Pens",
		mapTexture = "TheSlavePens",
		mapFloors = 1,
		mapID = 728,
	},
	[312] = {
		continent = "Serpentshrine Cavern",
		mapTexture = "CoilfangReservoir",
		mapFloors = 1,
		mapID = 780,
	},
	[313] = {
		continent = "The Botanica",
		mapTexture = "TheBotanica",
		mapFloors = 1,
		mapID = 729,
	},
	[314] = {
		continent = "The Mechanar",
		mapTexture = "TheMechanar",
		mapFloors = 1,
		mapID = 730,
	},
	[315] = {
		continent = "The Arcatraz",
		mapTexture = "TheArcatraz",
		mapFloors = 1,
		mapID = 731,
	},
	[316] = {
		continent = "Gruul's Lair",
		mapTexture = "GruulsLair",
		mapFloors = 1,
		mapID = 776,
	},
	[317] = {
		continent = "Black Temple",
		mapTexture = "BlackTemple",
		mapFloors = 8,
		mapID = 796,
	},
	[318] = {
		continent = "Tempest Keep: The Eye",
		mapTexture = "TempestKeep",
		mapFloors = 1,
		mapID = 782,
	},
	[319] = {
		continent = "Karazhan",
		mapTexture = "Karazhan",
		mapFloors = 17,
		mapID = 799,
	},
	[320] = {
		continent = "The Black Morass",
		mapTexture = "CoTTheBlackMorass",
		mapFloors = 0,
		mapID = 733,
	},
	[321] = {
		continent = "Old Hillsbrad Foothills",
		mapTexture = "CoTHillsbradFoothills",
		mapFloors = 0,
		mapID = 734,
	},
	[322] = {
		continent = "Hyjal Summit",
		mapTexture = "CoTMountHyjal",
		mapFloors = 0,
		mapID = 775,
	},
	[341] = {
		continent = "Magisters' Terrace",
		mapTexture = "MagistersTerrace",
		mapFloors = 2,
		mapID = 798,
	},
	[342] = {
		continent = "Sunwell Plateau",
		mapTexture = "SunwellPlateau",
		mapFloors = 2,
		mapID = 789,
	},
	[401] = {
		continent = "Strand of the Ancients",
		mapTexture = "StrandoftheAncients",
		mapFloors = 0,
		mapID = 512,
	},
	[402] = {
		continent = "Arathi Basin",
		mapTexture = "ArathiBasin",
		mapFloors = 0,
		mapID = 461,
	},
	[403] = {
		continent = "Warsong Gulch",
		mapTexture = "WarsongGulch",
		mapFloors = 0,
		mapID = 443,
	},
	[404] = {
		continent = "Alterac Valley",
		mapTexture = "AlteracValley",
		mapFloors = 0,
		mapID = 401,
	},
	[405] = {
		continent = "Isle of Conquest",
		mapTexture = "IsleofConquest",
		mapFloors = 0,
		mapID = 540,
	},
	[406] = {
		continent = "Eye of the Storm",
		mapTexture = "NetherstormArena",
		mapFloors = 0,
		mapID = 482,
	},
	[407] = {
		continent = "The Battle for Gilneas",
		mapTexture = "GilneasBattleground2",
		mapFloors = 0,
		mapID = 736,
	},
	[408] = {
		continent = "Twin Peaks",
		mapTexture = "TwinPeaks",
		mapFloors = 0,
		mapID = 626,
	},
	[1017] = {
		continent = "Orgrimmar",
		mapTexture = "Orgrimmar",
		mapFloors = 2,
		mapID = 321,
	},
	[4003] = {
		continent = "Dalaran",
		mapTexture = "Dalaran",
		mapFloors = 2,
		mapID = 504,
	},
}
-- Virtual Continents
VirtualMapTextures = {};
do
	for continentIndex, continents in pairs(VirtualContinentIndexes) do
		VirtualMapTextures[continents.mapTexture] = continentIndex;
	end

	ContinentMapTextures = {
		[WORLDMAP_COSMIC_ID] = "Cosmic", -- That constant is -1
		[0] = "World",
		[1] = "Kalimdor",
		[2] = "Azeroth",
		[3] = "Expansion01",
		[4] = "Northrend",
		[5] = "TheMaelstromContinent",
	}
end

DongleStub:Register(Astrolabe, activate)

-- Map Textures
MapTexturesC = {};
MapTexturesZ = {};
do
	for c = 1, 5 do
		for z = 1, #Astrolabe.ContinentList[c] do
			local mapTexture = Astrolabe.ContinentList[c][z];
			MapTexturesC[mapTexture] = c;
			MapTexturesZ[mapTexture] = z;
		end
	end
	for c = -1, 5 do
		local mapTexture = ContinentMapTextures[c];
		MapTexturesC[mapTexture] = c;
		MapTexturesZ[mapTexture] = 0;
	end

	MapZones = {};
	local continents = Astrolabe:GetMapVirtualContinents();
	for C, continentName in pairs(continents) do
		MapZones[Astrolabe:GetMapTexture(C, 0)] = continentName;
		if C < 99 then
			local zones = Astrolabe:GetMapVirtualZones(C);
			for Z, zoneName in pairs(zones) do
				MapZones[Astrolabe:GetMapTexture(C, Z)] = zoneName;
			end
		end
	end
	MapZones[ContinentMapTextures[WORLDMAP_COSMIC_ID]] = "Cosmic map";
	MapZones[ContinentMapTextures[0]] = "Azeroth";

	MapZoneTextures = {};
	for mapTexture, mapName in pairs(MapZones) do
		MapZoneTextures[mapName] = mapTexture;
	end
end

local zeroData;
zeroData = { xOffset = 0, height = 0, yOffset = 0, width = 0, __index = function() return zeroData end };
setmetatable(zeroData, zeroData);
setmetatable(WorldMapSize, zeroData);

for continent, zones in pairs(Astrolabe.ContinentList) do
	local mapData = WorldMapSize[continent];
	for index, mapName in pairs(zones) do
		if not ( mapData.zoneData[mapName] ) then
			--WE HAVE A PROBLEM!!!
			ChatFrame1:AddMessage("Astrolabe is missing data for "..select(index, unpack(Astrolabe:GetMapVirtualZones(continent)))..".");
			mapData.zoneData[mapName] = zeroData;
		end
		mapData[index] = mapData.zoneData[mapName];
		mapData.zoneData[mapName] = nil;
	end
end

-- register this library with AstrolabeMapMonitor, this will cause a full update if PLAYER_LOGIN has already fired
local AstrolabeMapMonitor = DongleStub("AstrolabeMapMonitor");
AstrolabeMapMonitor:RegisterAstrolabeLibrary(Astrolabe, LIBRARY_VERSION_MAJOR);

--[[MapAreaID = {
	[WORLDMAP_COSMIC_ID] = "Cosmic", -- That constant is -1
	[0] = "World",
	[4] = "Durotar",
	[9] = "Mulgore",
	[11] = "Barrens",
	[13] = "Kalimdor",
	[14] = "Azeroth",
	[16] = "Arathi",
	[17] = "Badlands",
	[19] = "BlastedLands",
	[20] = "Tirisfal",
	[21] = "Silverpine",
	[22] = "WesternPlaguelands",
	[23] = "EasternPlaguelands",
	[24] = "HillsbradFoothills",
	[26] = "Hinterlands",
	[27] = "DunMorogh",
	[28] = "SearingGorge",
	[29] = "BurningSteppes",
	[30] = "Elwynn",
	[32] = "DeadwindPass",
	[34] = "Duskwood",
	[35] = "LochModan",
	[36] = "Redridge",
	[37] = "StranglethornJungle",
	[38] = "SwampOfSorrows",
	[39] = "Westfall",
	[40] = "Wetlands",
	[41] = "Teldrassil",
	[42] = "Darkshore",
	[43] = "Ashenvale",
	[61] = "ThousandNeedles",
	[81] = "StonetalonMountains",
	[101] = "Desolace",
	[121] = "Feralas",
	[141] = "Dustwallow",
	[161] = "Tanaris",
	[181] = "Aszhara",
	[182] = "Felwood",
	[201] = "UngoroCrater",
	[241] = "Moonglade",
	[261] = "Silithus",
	[281] = "Winterspring",
	[301] = "StormwindCity",
	[321] = "Orgrimmar",
	[341] = "Ironforge",
	[362] = "ThunderBluff",
	[381] = "Darnassus",
	[382] = "Undercity",
	[401] = "AlteracValley",
	[443] = "WarsongGulch",
	[461] = "ArathiBasin",
	[462] = "EversongWoods",
	[463] = "Ghostlands",
	[464] = "AzuremystIsle",
	[465] = "Hellfire",
	[466] = "Expansion01",
	[467] = "Zangarmarsh",
	[471] = "TheExodar",
	[473] = "ShadowmoonValley",
	[475] = "BladesEdgeMountains",
	[476] = "BloodmystIsle",
	[477] = "Nagrand",
	[478] = "TerokkarForest",
	[479] = "Netherstorm",
	[480] = "SilvermoonCity",
	[481] = "ShattrathCity",
	[482] = "NetherstormArena",
	[485] = "Northrend",
	[486] = "BoreanTundra",
	[488] = "Dragonblight",
	[490] = "GrizzlyHills",
	[491] = "HowlingFjord",
	[492] = "IcecrownGlacier",
	[493] = "SholazarBasin",
	[495] = "TheStormPeaks",
	[496] = "ZulDrak",
	[499] = "Sunwell",
	[501] = "LakeWintergrasp",
	[502] = "ScarletEnclave",
	[504] = "Dalaran",
	[510] = "CrystalsongForest",
	[512] = "StrandoftheAncients",
	[520] = "TheNexus",
	[521] = "CoTStratholme",
	[522] = "Ahnkahet",
	[523] = "UtgardeKeep",
	[524] = "UtgardePinnacle",
	[525] = "HallsofLightning",
	[526] = "Ulduar77",
	[527] = "TheEyeofEternity",
	[528] = "Nexus80",
	[529] = "Ulduar",
	[530] = "Gundrak",
	[531] = "TheObsidianSanctum",
	[532] = "VaultofArchavon",
	[533] = "AzjolNerub",
	[534] = "DrakTharonKeep",
	[535] = "Naxxramas",
	[536] = "VioletHold",
	[539] = "GilneasX",
	[540] = "IsleofConquest",
	[541] = "HrothgarsLanding",
	[542] = "TheArgentColiseum",
	[543] = "TheArgentColiseum",
	[544] = "TheLostIsles",
	[545] = "Gilneas",
	[601] = "TheForgeofSouls",
	[602] = "PitofSaron",
	[603] = "HallsofReflection",
	[604] = "IcecrownCitadel",
	[605] = "Kezan",
	[606] = "Hyjal",
	[607] = "SouthernBarrens",
	[609] = "TheRubySanctum",
	[610] = "VashjirKelpForest",
	[611] = "GilneasCity",
	[613] = "Vashjir",
	[614] = "VashjirDepths",
	[615] = "VashjirRuins",
	[626] = "TwinPeaks",
	[640] = "Deepholm",
	[673] = "TheCapeOfStranglethorn",
	[677] = "BattleforGilneas",
	[678] = "Gilneas_terrain1",
	[679] = "Gilneas_terrain2",
	[680] = "Ragefire",
	[681] = "TheLostIsles_terrain1",
	[682] = "TheLostIsles_terrain2",
	[683] = "Hyjal_terrain1",
	[684] = "RuinsofGilneas",
	[685] = "RuinsofGilneasCity",
	[686] = "ZulFarrak",
	[687] = "TheTempleOfAtalHakkar",
	[688] = "BlackfathomDeeps",
	[689] = "StranglethornVale",
	[690] = "TheStockade",
	[691] = "Gnomeregan",
	[692] = "Uldaman",
	[696] = "MoltenCore",
	[697] = "ZulGurub",
	[699] = "DireMaul",
	[700] = "TwilightHighlands",
	[704] = "BlackrockDepths",
	[708] = "TolBarad",
	[709] = "TolBaradDailyArea",
	[710] = "TheShatteredHalls",
	[717] = "RuinsofAhnQiraj",
	[718] = "OnyxiasLair",
	[720] = "Uldum",
	[721] = "BlackrockSpire",
	[722] = "AuchenaiCrypts",
	[723] = "SethekkHalls",
	[724] = "ShadowLabyrinth",
	[725] = "TheBloodFurnace",
	[726] = "TheUnderbog",
	[727] = "TheSteamvault",
	[728] = "TheSlavePens",
	[729] = "TheBotanica",
	[730] = "TheMechanar",
	[731] = "TheArcatraz",
	[732] = "ManaTombs",
	[733] = "CoTTheBlackMorass",
	[734] = "CoTHillsbradFoothills",
	[736] = "GilneasBattleground2",
	[737] = "TheMaelstrom",
	[747] = "LostCityofTolvir",
	[748] = "Uldum_terrain1",
	[749] = "WailingCaverns",
	[750] = "Maraudon",
	[751] = "TheMaelstromContinent",
	[752] = "BaradinHold",
	[753] = "BlackrockCaverns",
	[754] = "BlackwingDescent",
	[755] = "BlackwingLair",
	[756] = "TheDeadmines",
	[757] = "GrimBatol",
	[758] = "TheBastionofTwilight",
	[759] = "HallsofOrigination",
	[760] = "RazorfenDowns",
	[761] = "RazorfenKraul",
	[762] = "ScarletMonastery",
	[763] = "Scholomance",
	[764] = "ShadowfangKeep",
	[765] = "Stratholme",
	[766] = "AhnQiraj",
	[767] = "ThroneofTides",
	[768] = "TheStonecore",
	[769] = "Skywall",
	[770] = "TwilightHighlands_terrain1",
	[772] = "AhnQirajTheFallenKingdom",
	[773] = "ThroneoftheFourWinds",
	[775] = "CoTMountHyjal",
	[776] = "GruulsLair",
	[779] = "MagtheridonsLair",
	[780] = "CoilfangReservoir",
	[781] = "ZulAman",
	[782] = "TempestKeep",
	[789] = "SunwellPlateau",
	[793] = "ZulGurub",
	[795] = "MoltenFront",
	[796] = "BlackTemple",
	[797] = "HellfireRamparts",
	[798] = "MagistersTerrace",
	[799] = "Karazhan",
	[800] = "Firelands",
	[803] = "TheNexusLegendary",
	[816] = "WellofEternity",
	[819] = "HourofTwilight",
	[820] = "EndTime",
	[823] = "DarkmoonFaireIsland",
	[824] = "DragonSoul",
}]]--
