--[[
Name: LibFriendPositions-1.0
Revision: $Rev: 42 $
Author: Cameron Kenneth Knight (ckknight@gmail.com)
Website: http://www.wowace.com/
Description: Communicates with friends for their positions on the world map
License: MIT
]]

local MAJOR_VERSION = "LibFriendPositions-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 42 $"):match("%d+")) or 0


local lib, oldMinor = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then
	return
end

local COMM_PREFIX = "LFP"

local _G = _G
local pairs = pairs
local next = next
local ShowFriends = ShowFriends
local GetFriendInfo = GetFriendInfo
local GetNumFriends = GetNumFriends
local GetTime = GetTime
local IsInInstance = IsInInstance
local GetPlayerMapPosition = GetPlayerMapPosition
local GetMapInfo = GetMapInfo
local string_char = string.char
local math_floor = math.floor
local math_max = math.max

local frame = lib.frame
if not frame then
	frame = CreateFrame("Frame")
	lib.frame = frame
end
frame:UnregisterAllEvents()
frame:SetScript("OnUpdate", nil)
frame:SetScript("OnEvent", nil)
frame:Show()

local callbacks = lib.callbacks
if not callbacks then
	callbacks = LibStub("CallbackHandler-1.0"):New(lib)
	lib.callbacks = callbacks
end

local friends = {}
--local friendSend = {}
local friendX = {}
local friendY = {}
local friendZone = {}
local friendExpireTime = {}
local sharePos

local function iter(_, friendName)
	local friendName, x = next(friendX, friendName)
	if not friendName then
		return nil
	end
	local y = friendY[friendName]
	local zone = friendZone[friendName]
	return friendName, x, y, zone
end

--[[
Notes:
	x and y are in the range [0, 1]
	zone is the map texture that the friend is in, not the localized name.
Returns:
	iterator that returns name, x, y, and zone
Example:
	for name, x, y, zone in LibStub("LibFriendPositions-1.0"):IterateFriends() do
		-- do stuff here
	end
]]
function lib:IterateFriends()
	return iter
end

--[[
Notes:
x and y are in the range [0, 1]
	zone is the map texture that the friend is in, not the localized name.
Returns:
	x, y, zone - the horizontal position, vertical position, and map texture of the friend. All nil if unknown or not on the map or offline.
Example:
	for name, x, y, zone in LibStub("LibFriendPositions-1.0"):GetPositionOfFriend() do
		-- do stuff here
	end
]]
function lib:GetPositionOfFriend(name)
	local x = friendX[name]
	if x then
		return x, friendY[name], friendZone[name]
	else
		return nil, nil, nil
	end
end

function lib:ShareSelf(value)
	sharePos = not not value
end

local CLEAR_BYTE = ("C"):byte()
local POSITION_BYTE = ("P"):byte()
--local LOGIN_BYTE = ("L"):byte()
--local REPLY_BYTE = ("R"):byte()

local numFriends = 0
local nextFriendlist = 0
--local firstScanDone

local function SendMessage(data, dest)
	if data and dest and friends[dest] and friends[dest] == 1 then
		SendAddonMessage(COMM_PREFIX, data, "WHISPER", dest)
	end
end

local function SendMessageToAll(data)
	if #friends > 0 and data then
		for k, v in pairs(friends) do
			if v == 1 then SendAddonMessage(COMM_PREFIX, data, "WHISPER", k) end
		end
	end
end

local events = {}

function events.ADDON_LOADED()
	if ChatThrottleLib then
		local ChatThrottleLib = ChatThrottleLib
		function SendMessage(data, dest)
			if data and dest and friends[dest] and friends[dest] == 1 then
				ChatThrottleLib:SendAddonMessage("NORMAL", COMM_PREFIX, data, "WHISPER", dest, MAJOR_VERSION)
			end
		end
		function SendMessageToAll(data)
			if #friends > 0 and data then
				for k, v in pairs(friends) do
					if v == 1 then ChatThrottleLib:SendAddonMessage("NORMAL", COMM_PREFIX, data, "WHISPER", k, MAJOR_VERSION) end
				end
			end
		end
		frame:UnregisterEvent("ADDON_LOADED")
		events.ADDON_LOADED = nil
	end
end

local groupMembers = {}
--groupMembers = groupMembers

function events.RAID_ROSTER_UPDATE(event)
	local prefix, amount = "raid", GetNumRaidMembers()
	if amount == 0 then
		prefix, amount = "party", GetNumPartyMembers()
	end
	for k in pairs(groupMembers) do
		groupMembers[k] = nil
	end

	for i = 1, amount do
		local name, realm = UnitName(prefix .. i)
		if not realm or realm == "" then
			groupMembers[name] = true
			if friendX[name] then
				friendX[name] = nil
				friendY[name] = nil
				friendZone[name] = nil
				friendExpireTime[name] = nil

				callbacks:Fire("Clear", name)
			end
		end
	end
end
events.PARTY_MEMBERS_CHANGED = events.RAID_ROSTER_UPDATE
events.PLAYER_LOGIN = events.RAID_ROSTER_UPDATE

local function UpdatePosition()
	local currentTime = GetTime()
	if nextSpamTime and currentTime > nextSpamTime then
		nextSpamTime = currentTime + math_max(10, numFriends/4)

		if numFriends >= 1 then -- yourself and someone else at least
			if IsInInstance() then
				if not clearedLast then
					clearedLast = true
					if sharePos then
						SendMessageToAll(string_char(CLEAR_BYTE))
					end
				end
			else
				local x, y = GetPlayerMapPosition("player")
				local zone = GetMapInfo()
				if (x ~= last_x or y ~= last_y or zone ~= last_zone or last_sendTime < currentTime-90) and x ~= 0 and y ~= 0 and zone then
					last_x = x
					last_y = y
					last_zone = zone
					last_sendTime = currentTime
					x = math_floor(x * 255^2 + 0.5)
					y = math_floor(y * 255^2 + 0.5)
					local x_1 = math_floor(x / 255) + 1
					local x_2 = (x % 255) + 1
					local y_1 = math_floor(y / 255) + 1
					local y_2 = (y % 255) + 1
					clearedLast = false
					if sharePos then
						SendMessageToAll(string_char(POSITION_BYTE, x_1, x_2, y_1, y_2, #zone+1) .. zone)
						--[[for k, v in pairs(friendSend) do
							if v == 1 then SendMessage(string_char(POSITION_BYTE, x_1, x_2, y_1, y_2, #zone+1) .. zone, k) end
						end]]--
					end
				end
			end
		end
	end
end

local playerName = UnitName("player")
function events.CHAT_MSG_ADDON(event, prefix, message, distribution, sender)
	if prefix ~= COMM_PREFIX or distribution ~= "WHISPER" or playerName == sender or groupMembers[sender] then
		return
	end

	local b = message:byte()
	if not b then
		return
	end

	if b == clear_byte then
		friendX[sender] = nil
		friendY[sender] = nil
		friendZone[sender] = nil
		friendExpireTime[sender] = nil

		callbacks:Fire("Clear", sender)
	elseif b == position_byte then
		local x_1, x_2, y_1, y_2, zone_len = message:byte(2, 6)
		if not zone_len then
			return
		end
		local x = ((x_1-1)*255 + (x_2-1)) / (255^2)
		local y = ((y_1-1)*255 + (y_2-1)) / (255^2)
		zone_len = zone_len-1
		local zone = message:sub(7, 6+zone_len)
		friendX[sender] = x
		friendY[sender] = y
		friendZone[sender] = zone
		friendExpireTime[sender] = GetTime() + 120

		callbacks:Fire("Position", sender, x, y, zone)
	--[[elseif b == ("L"):byte() then
		local name = message:sub(2)
		if name == sender then
			friendSend[sender] = 1
			if sharePos then
				SendMessage(string_char(REPLY_BYTE) .. playerName, sender)
			end
		end
	elseif b == ("R"):byte() then
		local name = message:sub(2)
		if name == sender then
			friendSend[sender] = 1
		end]]--
	end
end

local nextSpamTime
local safe = {}
function events.FRIENDLIST_UPDATE()
	local numFriendsTotal = GetNumFriends()
	local last_friends = friends

	if numFriendsTotal and numFriendsTotal > 0 then
		numFriends = 0
		for i = 1, #last_friends do
			last_friends[i] = nil
		end

		nextFriendlist = GetTime() + 15
		for i = 1, numFriendsTotal do
			local name, level, class, zone, online, status = GetFriendInfo(i)
			if name then
				if online then
					safe[name] = true
					last_friends[name] = 1
					numFriends = numFriends + 1
				else
					last_friends[name] = 0
					--friendSend[name] = nil
				end
			end
		end
	else
		numFriends = 0
	end
	friends = last_friends
	UpdatePosition()

	for name in pairs(friendX) do
		if not safe[name] then
			friendX[name] = nil
			friendY[name] = nil
			friendZone[name] = nil
			friendExpireTime[name] = nil

			callbacks:Fire("Clear", name)
		end
	end
	for name in pairs(safe) do
		safe[name] = nil
	end
	if not nextSpamTime then
		nextSpamTime = 0
	end
	--[[if not firstScanDone then
		if sharePos then
			SendMessageToAll(string_char(LOGIN_BYTE) .. playerName)
		end
		firstScanDone = true
	end]]--
end

for event in pairs(events) do
	frame:RegisterEvent(event)
end
frame:SetScript("OnEvent", function(self, event, ...)
	events[event](event, ...)
end)

local clearedLast = false
local last_x, last_y, last_zone
local last_sendTime = 0
local nextOnUpdate = 0

frame:SetScript("OnUpdate", function(self, elapsed)
	local currentTime = GetTime()
	if nextOnUpdate > currentTime then
		return
	end
	nextOnUpdate = currentTime + 1

	if currentTime > nextFriendlist then
		if GetNumFriends() > 0 then
			ShowFriends()
		end
	end

	for name, time in pairs(friendExpireTime) do
		if currentTime > time then
			friendX[name] = nil
			friendY[name] = nil
			friendZone[name] = nil
			friendExpireTime[name] = nil

			callbacks:Fire("Clear", name)
		end
	end
end)
