--[[
MapsterEnhanced - Waypoints (Sindragosa)

Based on Cartographer
Credits: ckknight
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "Waypoints"
local Waypoints = Mapster:NewModule(MODNAME, "AceEvent-3.0", "AceHook-3.0", "AceComm-3.0", "AceConsole-3.0")

MapsterWaypoints = Waypoints


-- Libs --
local Dewdrop = AceLibrary("Dewdrop-2.0")
local LibGuild = LibStub("LibGuild-1.0")
local LSM = LibStub("LibSharedMedia-3.0")
local LDB = LibStub("LibDataBroker-1.1")

-- local --
local _G = getfenv(0)
local GetCursorPosition = GetCursorPosition
local GetPlayerMapPosition = GetPlayerMapPosition
local GetTime = GetTime
local GetCVar = GetCVar
local GetScreenWidth, GetScreenHeight = GetScreenWidth, GetScreenHeight
local GetMapContinents, GetMapZones = GetMapContinents, GetMapZones
local GetCurrentMapContinent, GetCurrentMapZone = GetCurrentMapContinent, GetCurrentMapZone
local GetCurrentMapAreaID, GetCurrentMapDungeonLevel = GetCurrentMapAreaID, GetCurrentMapDungeonLevel
local SetMapToCurrentZone, SetMapZoom = SetMapToCurrentZone, SetMapZoom
local SetMapByID, SetDungeonMapLevel = SetMapByID, SetDungeonMapLevel
local GetPlayerFacing = GetPlayerFacing
local GetRealZoneText = GetRealZoneText
local UnitIsConnected = UnitIsConnected
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local IsInInstance = IsInInstance
local IsActiveBattlefieldArena = IsActiveBattlefieldArena
local WATCHFRAME_FILTER_COMPLETED_QUESTS = WATCHFRAME_FILTER_COMPLETED_QUESTS
local WATCHFRAME_FILTER_REMOTE_ZONES = WATCHFRAME_FILTER_REMOTE_ZONES
local QUEST_POI_SWAP_BUTTONS = QUEST_POI_SWAP_BUTTONS
local MAX_NUM_QUESTS = MAX_NUM_QUESTS
local LOCAL_MAP_QUESTS = LOCAL_MAP_QUESTS
local GetQuestLogTitle = GetQuestLogTitle
local GetNumQuestLeaderBoards = GetNumQuestLeaderBoards
local GetNumQuestWatches = GetNumQuestWatches
local GetQuestIndexForWatch = GetQuestIndexForWatch
local GetQuestWorldMapAreaID, QuestPOIGetIconInfo = GetQuestWorldMapAreaID, QuestPOIGetIconInfo
local WorldMapDetailFrame = WorldMapDetailFrame
local IsControlKeyDown = IsControlKeyDown

local bit_band = bit.band
local table = table
local table_insert = table.insert
local table_remove = table.remove
local string = string
local string_format = string.format
local string_find = string.find
local string_match = string.match
local string_gsub = string.gsub
local string_sub = string.sub
local math = math
local math_floor = math.floor
local math_pow = math.pow
local math_atan2 = math.atan2
local math_pi = math.pi
local math_cos = math.cos
local math_deg = math.deg
local math_random = math.random
local format = format
local floor = floor
local abs = abs
local time = time
local wipe = table.wipe

local type = type
local ipairs = ipairs
local pairs = pairs
local select = select
local setmetatable = setmetatable
local tonumber = tonumber

local Notes

local delimiters = { [" "] = true, [","] = true, ["-"] = true }
local rotateMinimap = GetCVar("rotateMinimap") == "1"
local waypointFrame
local poiHooks = {}
local yards = L["%.0f yd"]

local questPOIs = {}
local closestQuest = {}
local priorityQuest

local function getTexCoords(column, row, up)
	if up == "UP" then
		local xstart = (column * 53) / 512
		local ystart = (row * 70) / 512
		local xend = ((column + 1) * 53) / 512
		local yend = ((row + 1) * 70) / 512
		return xstart, xend, ystart, yend
	else
		local xstart = (column * 56) / 512
		local ystart = (row * 42) / 512
		local xend = ((column + 1) * 56) / 512
		local yend = ((row + 1) * 42) / 512
		return xstart, xend, ystart, yend
	end
end

local texCoords = setmetatable({}, {__index = function(t, k)
	local col, row, up = k:match("(%d+):(%d+):(.*)")
	col, row = tonumber(col), tonumber(row)
	local obj = {getTexCoords(col, row, up)}
	rawset(t, k, obj)
	return obj
end})
rawset(texCoords, "0", {0, 1, 0, 1})

local db
local defaults = {
	profile = {
		scale = 1,
		alpha = 1,
		minimap = true,
		metric = true,
		bearing = true,
		release = true,
		font = "Friz Quadrata TT",
		fontPath = "Fonts\\FRIZQT__.TTF",
		fontSize = 12,
	}
}

local optGetter, optSetter
do
	local mod = Waypoints
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["Waypoints"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The Waypoints module adds an arrow to direct you to a specified note or location."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					width = "full",
					name = L["Enable Waypoints"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
				},
				scale = {
					order = 3,
					type = "range",
					name = L["Scale"],
					desc = L["Set the size of the waypoint arrow."],
					min = 0.1, max = 2, bigStep = 0.01,
					isPercent = true,
				},
				alpha = {
					order = 4,
					type = "range",
					name = L["Alpha"],
					desc = L["The transparency of the waypoint arrow."],
					min = 0, max = 1, bigStep = 0.01,
					isPercent = true,
				},
				desc = {
					order = 4.5,
					type = "description",
					name = "",
				},
				metric = {
					order = 5,
					type = "toggle",
					name = L["Metric"],
					desc = L["Show the distance in meters."],
				},
				minimap = {
					order = 6,
					type = "toggle",
					name = L["Waypoint icons"],
					desc = L["Show the next waypoint on the minimap and all waypoints on the main map."],
				},
				bearing = {
					order = 7,
					type = "toggle",
					name = L["Heading to waypoints"],
					desc = L["Show the relative heading to waypoints."],
				},
				release = {
					order = 8,
					type = "toggle",
					name = L["Release point"],
					desc = L["Show waypoint to release location on death."],
				},
				defaultmessages = {
					order = 9,
					type = "toggle",
					name = L["Default message"],
					desc = L["Show not so entertaining messages on death."],
				},
				clickthrough = {
					order = 10,
					type = "toggle",
					name = L["Click-through arrow"],
					desc = L["Make the waypoint arrow click-through."],
					set = function(info, value)
						db.clickthrough = value
						waypointFrame:EnableMouse(not value)
					end,
				},
				desc2 = {
					order = 10.5,
					type = "description",
					name = "",
				},
				font = {
					order = 11,
					type = "select",
					name = L["Font"],
					desc = L["Some of the fonts may depend on other addons."],
					dialogControl = "LSM30_Font",
					values = AceGUIWidgetLSMlists.font,
					get = function()
						return db.font
					end,
					set = function(info, value)
						db.fontPath = LSM:Fetch("font", value)
						db.font = value
						waypointFrame.dist:SetFont(db.fontPath, db.fontSize)
						waypointFrame.title:SetFont(db.fontPath, db.fontSize)
						waypointFrame.tta:SetFont(db.fontPath, db.fontSize)
					end,
				},
				fontSize = {
					order = 12,
					type = "range",
					name = L["Font size"],
					min = 8, max = 20, step = 1,
					get = function(info)
						return db.fontSize
					end,
					set = function(info, value)
						db.fontSize = value
						waypointFrame.dist:SetFont(db.fontPath, db.fontSize)
						waypointFrame.title:SetFont(db.fontPath, db.fontSize)
						waypointFrame.tta:SetFont(db.fontPath, db.fontSize)
					end,
				},
				datafeed = {
					order = 14,
					type = "toggle",
					name = L["LDB Data Feed"],
					desc = L["Show the LDB Data Feed. Disabling this requires a reload."],
					get = function(info)
						return db.datafeed
					end,
					set = function(info, value)
						db.datafeed = value
						if value and LDB and not Waypoints.WaypointLDB then
							Waypoints:InitializeLDB()
						elseif not value and Waypoints.WaypointLDB then
							Waypoints:Print(L["Disabling the LDB Data Feed requires a reload."])
							Waypoints.WaypointLDB.waypointID = nil
							Waypoints.WaypointLDB.text = ""
							Waypoints.WaypointLDB.facing = nil
							Waypoints.WaypointLDB.dist = nil
							Waypoints.WaypointLDB.icon = "Interface\\WorldMap\\WorldMap-Icon"
							Waypoints.WaypointLDB.iconR = 1
							Waypoints.WaypointLDB.iconG = 1
							Waypoints.WaypointLDB.iconB = 1
							Waypoints.WaypointLDB.iconCoords = texCoords["0"]
							if anchor then LDB_OnLeave(anchor) end
						end
					end,
				},
				questPOI = {
					order = 17,
					type = "toggle",
					name = L["Quest POI tracking"],
					desc = L["Setting waypoints to quest objectives when control-clicking on quest POIs."],
				},
				closestPOI = {
					order = 18,
					type = "toggle",
					name = L["Closest Quest POI"],
					desc = L["The closest quest POI will automatically be tracked."],
					disabled = function(info)
						return not db.questPOI
					end,
				},
			},
		}
	end

	return options
end

local Queue = {}
Waypoints.Queue = Queue

-- round --
local function round(num, digits)
	if not digits then digits = 0 end
	local mantissa = 10^digits
	local norm = num*mantissa
	norm = norm + 0.5
	local norm_f = math_floor(norm)
	return norm_f/mantissa
end

-- getID --
local function getID(x, y)
	if not x or not y then return nil end
	return round(x * 10000, 0) + round(y * 10000, 0) * 10000
end
Waypoints.getID = getID

-- getXY --
local function getXY(id)
	if not id then return nil, nil end
	return (id % 10000) / 10000, math_floor(id / 10000) / 10000
end
Waypoints.getXY = getXY

local function explode(...)
	if select("#", ...) == 2 and delimiters[select(1, ...)] then
		return explode(string.split(select(1, ...), select(2, ...)))
	end

	local arr, quote = {}, nil
	for i = 1,select("#", ...) do -- for each divider found
		local str = select(i, ...)
		local quotefound = string_find(str, "[\"\']")
		if (not quotefound and not quote) or (quotefound and quote) then
			if quotefound and quote then
				local clean = string_gsub(quote.." "..str, "[\"\']", "")
				table_insert(arr, clean)
			else
				table_insert(arr, str)
			end
			quote = nil
		else
			if quote then
				quote = quote.." "..str
			else
				quote = str
			end
		end
	end
	return arr
end

-- checkParameter --
local function checkParameter(method, parameter, position, datatype, ...)
	local datatype2, datatype3, datatype4, datatype5 = ...
	if datatype2 then
		if datatype2 == "nil" then
			if type(parameter) ~= datatype and parameter ~= nil then error(("Bad argument #%d to 'Waypoints:%s'. Expected %q or nil, got %q."):format(position, method, datatype, type(parameter)), 2) end
		else
			if type(parameter) ~= datatype and type(parameter) ~= datatype2 then error(("Bad argument #%d to 'Waypoints:%s'. Expected %q or %q, got %q."):format(position, method, datatype, datatype2, type(parameter)), 2) end
		end
	else
		if type(parameter) ~= datatype then error(("Bad argument #%d to 'Waypoints:%s'. Expected %q, got %q."):format(position, method, datatype, type(parameter)), 2) end
	end
end


-- FormatDurationShort --
local DAYS_ABBR, HOURS_ABBR, MINUTES_ABBR, SECONDS_ABBR = DAYS_ABBR, HOURS_ABBR, MINUTES_ABBR, SECONDS_ABBR
local DAYS_ABBR_S1, HOURS_ABBR_S1, MINUTES_ABBR_S1, SECONDS_ABBR_S1 = "days", "hours", "min", "sec"

local function FormatDurationShort(duration, colorize, hideSeconds)
	local neg = ""
	if duration ~= duration then
		duration = 0
	end
	if duration < 0 then
		neg = "-"
		duration = -duration
	end
	if not colorize then
		if not duration or duration >= 86400*36500 then -- 100 years
			return "***"
		elseif duration >= 172800 then
			return format("%s%.1f %s", neg, duration/86400, DAYS_ABBR_S1)
		elseif duration >= 7200 then
			return format("%s%.1f %s", neg, duration/3600, HOURS_ABBR_S1)
		elseif duration >= 120 or not hideSeconds then
			return format("%s%.1f %s", neg, duration/60, MINUTES_ABBR_S1)
		else
			return format("%s%.0f %s", neg, duration, SECONDS_ABBR_S1)
		end
	else
		if not duration or duration >= 86400*36500 then -- 100 years
			return "|cffffffff***|r"
		elseif duration >= 172800 then
			return format("|cffffffff%s%.1f|r %s", neg, duration/86400, DAYS_ABBR_S1)
		elseif duration >= 7200 then
			return format("|cffffffff%s%.1f|r %s", neg, duration/3600, HOURS_ABBR_S1)
		elseif duration >= 120 or not hideSeconds then
			return format("|cffffffff%s%.1f|r %s", neg, duration/60, MINUTES_ABBR_S1)
		else
			return format("|cffffffff%s%.0f|r %s", neg, duration, SECONDS_ABBR_S1)
		end
	end
end


local function PushQueue(waypoint, append)
	if append then
		table_insert(Queue, waypoint)
	else
		table_insert(Queue, 1, waypoint)
	end
end

local function PopQueue(index)
	local corn = table_remove(Queue, index or 1)

	if corn.onMap and corn.poiID and Waypoint.mapDB[corn.zone] and Waypoint.mapDB[corn.zone][corn.poiID] then
		Waypoint.mapDB[corn.zone][corn.poiID].focus = nil

		if #Queue > 0 then
			local waypoint = Queue[index or 1]
			if waypoint.poiID and Waypoint.mapDB[waypoint.zone] and Waypoint.mapDB[waypoint.zone][waypoint.poiID] then
				Waypoint.mapDB[waypoint.zone][waypoint.poiID].focus = true
			end
			if waypoint and waypoint.questPOI then
				Waypoints:UpdateClosestPOI()
			end
		end
	end

	return corn
end

local waypointLocalZone
local path = "Interface\\AddOns\\MapsterEnhanced\\Textures\\Arrow"

local function createWaypointFrame()
	waypointFrame = CreateFrame("Frame", "Mapster_Waypoints", UIParent)
	waypointFrame:SetWidth(56)
	waypointFrame:SetHeight(64)
	waypointFrame:SetScale(Waypoints.db.profile.scale)
	waypointFrame:SetAlpha(Waypoints.db.profile.alpha)
	waypointFrame:Show()
	local tex = waypointFrame:CreateTexture(nil, "OVERLAY")
	waypointFrame.tex = tex
	tex:SetWidth(56)
	tex:SetHeight(42)
	tex:SetTexture(path)
	waypointFrame.up = false
	tex:SetPoint("TOP")
	local dist = waypointFrame:CreateFontString("Mapster_Waypoints_Distance", "OVERLAY", "GameFontNormal")
	waypointFrame.dist = dist
	dist:SetFont(Waypoints.db.profile.fontPath, Waypoints.db.profile.fontSize)
	dist:SetPoint("BOTTOM")
	local title = waypointFrame:CreateFontString("Mapster_Waypoints_Name", "OVERLAY", "GameFontNormal")
	waypointFrame.title = title
	title:SetFont(Waypoints.db.profile.fontPath, Waypoints.db.profile.fontSize)
	title:SetPoint("TOP", dist, "BOTTOM")
	local tta = waypointFrame:CreateFontString("Mapster_Waypoints_TTA", "OVERLAY", "GameFontNormal")
	waypointFrame.tta = tta
	tta:SetFont(Waypoints.db.profile.fontPath, Waypoints.db.profile.fontSize)
	tta:SetPoint("TOP", title, "BOTTOM")
	waypointFrame:EnableMouse(not Waypoints.db.profile.clickthrough)
	waypointFrame:SetMovable(not Waypoints.db.profile.locked)
	waypointFrame:RegisterForDrag(not Waypoints.db.profile.locked and "LeftButton" or nil)
	waypointFrame:SetScript("OnDragStart", function(this)
		this:StartMoving()
	end)
    local closeFunc = function()
        Dewdrop:Close()
    end
	waypointFrame:SetScript("OnDragStop", function(this)
	this:StopMovingOrSizing()
		local x, y = this:GetCenter()
		x = x - GetScreenWidth() / 2
		y = y - GetScreenHeight() / 2
		Waypoints.db.profile.waypointX = x
		Waypoints.db.profile.waypointY = y
		this:ClearAllPoints()
		this:SetPoint("CENTER", UIParent, "CENTER", x, y)
	end)
	waypointFrame:SetClampedToScreen(true)
	waypointFrame:SetPoint("CENTER", UIParent, "CENTER", Waypoints.db.profile.waypointX, Waypoints.db.profile.waypointY)
	Dewdrop:Register(waypointFrame,
		'children', function()
			Dewdrop:AddLine(
				'text', L["Clear waypoint"],
				'arg1', Waypoints,
				'arg2', Queue,
				'func', function(self, Queue) Queue[1]:Cancel() end,
				'closeWhenClicked', true
			)
			if #Queue > 1 then
				Dewdrop:AddLine(
					'text', L["Move to end of queue"],
					'arg1', Waypoints,
					'arg2', Queue,
					'func', function(self, Queue) PushQueue(PopQueue(), 1) end,
					'closeWhenClicked', true
				)
			end
			Dewdrop:AddLine()
			Dewdrop:AddLine(
				'text', L["Lock waypoint arrow"],
				'desc', L["Lock the waypoint arrow in place"],
				'checked', Waypoints.db.profile.locked,
				'func', function()
					local locked = Waypoints.db.profile.locked
					if locked then
						waypointFrame:RegisterForDrag("LeftButton")
						waypointFrame:EnableMouse(true)
						waypointFrame:SetMovable(true)
						Waypoints.db.profile.locked = false
					else
						waypointFrame:RegisterForDrag(nil)
						waypointFrame:SetMovable(false)
						Waypoints.db.profile.locked = true
					end
				end,
				'closeWhenClicked', true
			)
			Dewdrop:AddLine(
				'text', CLOSE,
                'func', closeFunc,
				'closeWhenClicked', true
			)
		end
	)
end

local noop = function() end -- empty function
local initMeOnce
initMeOnce = function()
	local function waypointCommand(text, temp)
		local waypoint

		if not text or string.len(text) < 1 then
			Waypoints:Print(L["For usage, see '/waypoint help'"])
			return
		end
		if text == "help" then
			Waypoints:Print(L["Commands are:"])
			Waypoints:Print(L["General:  /waypoint [add] <notesearch> OR <0-100>,<0-100> [title] [z-zone] OR g-<guildnamesearch> OR p-<partynamesearch>"])
			Waypoints:Print(L["Clear:  /noway"])
			Waypoints:Print(L["View:  /wayq"])
			return
		end

		SetMapToCurrentZone()
		local zone = Notes:GetCurrentZoneName()

		-- This function will take the explode by delimiter and then further explode
		-- each entry by "-" and then insert them into a table
		-- For an argument string like "Cabbage p-Womble z-Zangarmarsh Frogs" you'll end
		-- up with a table looking like:
		-- {
		--   arg1 = "Cabbage",
		--   p = "Womble",
		--   z = "Zangarmarsh",
		--   arg2 = "Frogs"
		-- }
		--
		local function parseOptions(text, delimiter)
			if not delimiter then delimiter = " " end
			local args, options, count = explode(delimiter, text), {}, 1
			for _, arg in ipairs(args) do
				local split = explode("-", arg)
				if #split == 2 then
					options[split[1]] = split[2]
				else
					options["arg"..count] = arg
					count = count + 1
				end
			end
			return options
		end
		-- The options we have in place are:
		--		g-<GuildMemberNameOrPartThereof>
		--		p-<PartyMemberNameOrPartThereof>
		--		z-<SpecificZoneName>
		--
		local options = parseOptions(text)
		local x, y, title, imprecise
		if options.z then -- Lets qualify the zone name
			options.z, imprecise = Notes:FindZone(options.z)
			if not options.z then
				options.z = L["No matching zone"]
			end
		end
		-- Okay, horribly convoluted.
		-- Basically we want to handle the following possiblities for note based waypoints
		--		/way 23,34
		--		/way add 23,34
		--		/way 23,34 MyNote
		--		/way add 23,34 MyNote
		--		/way CurrentNoteSearchName
		--		/way add CurrentNoteSearchName
		if options.arg1 then -- check for x,y
			if tonumber(options.arg1) and tonumber(options.arg2) then -- /way 23 34
				options.arg1 = options.arg1..","..options.arg2
				options.arg2 = options.arg3 or nil
				options.arg3 = options.arg4 or nil
			end

			local xy = explode(",", options.arg1)
			if #xy == 2 then
				x = xy[1]
				y = xy[2]
				if options.arg2 and type(options.arg2) == "string" then
					title = options.arg2
				else
					title = L["Waypoint"]
				end
			else
				if options.arg1 == "add" then
					if options.arg2 then
						xy = explode(",", options.arg2)
						if #xy == 2 then
							x = xy[1]
							y = xy[2]
						else -- We're a text search for a note, not a x,y note
							if options.z then
								x, y = select(2, Notes:FindNearestNote(options.arg2, options.z, nil, nil, nil, true))
							else
								zone, x, y = Notes:FindNearestNote(options.arg2, "all", nil, nil, nil, true)
							end
							if not x then
								if imprecise then
									Waypoints:Print(format(L["Cannot find note in closest matching zone: %s"], options.z or zone or "all"))
								else
									Waypoints:Print(format(L["No matching note in zone: %s"], options.z or zone or "all"))
								end

								return
							end
						end
						if options.arg3 and type(options.arg3) == "string" then
							title = options.arg3
						else
							title = L["Waypoint"]
						end
					end
				else
					if options.z then
						x, y = select(2, Notes:FindNearestNote(options.arg1, options.z, nil, nil, nil, true))
					else
						zone, x, y = Notes:FindNearestNote(options.arg1, "all", nil, nil, nil, true)
					end
					if not x then
						if imprecise then
							Waypoints:Print(format(L["Cannot find note in closest matching zone: %s"], options.z or zone or "all"))
						else
							Waypoints:Print(format(L["No matching note in zone: %s"], options.z or zone or "all"))
						end

						return
					end
				end
			end
			x, y = tonumber(x), tonumber(y)
			if x > 1 and y > 1 then
				x, y = tonumber(x)/100, tonumber(y)/100
			end
			if NotePoint then
				zone = Notes:GetMapTexture(options.z or zone)
				waypoint = NotePoint:new(zone, x, y, title)
				if not waypoint:IsValid() then
					Waypoints:Print(L["Invalid note entry."])
					return
				end
			end
		end

		if options.g then -- We want to add a guild member
			if GuildPoint then
				waypoint = GuildPoint:new(options.g)
				if not waypoint:IsValid() then
					Waypoints:Print(format(L["Guild member %s not found."], options.g))
					return
				end
			end
		end

		if options.p then -- We want to add a party member
			if PartyPoint then
				waypoint = PartyPoint:new(options.p)
				if not waypoint:IsValid() then
					Waypoints:Print(format(L["No party member %s found."], options.p))
					return
				end
			end
		end

		if waypoint then
			if options.arg1 and options.arg1 == "add" then
				PushQueue(waypoint, 1)
			else
				PushQueue(waypoint)
			end
			local showOnMap = waypoint.showOnMap
			if type(showOnMap) == "function" then
				showOnMap = showOnMap(waypoint)
			end
			if showOnMap and db.minimap then
				waypoint:AddToMap()
			end
			Waypoints.waypointUpdateFrame:SetScript("OnUpdate", function(self, elapsed) Waypoints:UpdateWaypoint(elapsed) end)
			Waypoints.waypointUpdateFrame:Show()
			if not waypointFrame then
				createWaypointFrame()
			end
		end
	end

	local function getQueue()
		if #Queue == 0 then
			Waypoints:Print(L["Waypoint queue empty."])
			return
		end
		Waypoints:Print(L["Waypoint queue:"])
		for i,v in ipairs(Queue) do
			Waypoints:Print("    "..v:ToString())
		end
	end

	Waypoints:RegisterChatCommand("way", function(command) waypointCommand(command) end)
	Waypoints:RegisterChatCommand("waypoint", function(command) waypointCommand(command) end)
	Waypoints:RegisterChatCommand("tway", function(command) waypointCommand(command) end)
	Waypoints:RegisterChatCommand("tloc", function(command) waypointCommand(command) end)
	Waypoints:RegisterChatCommand("wayq", function(command) getQueue() end)
	Waypoints:RegisterChatCommand("noway", function(command)
		Waypoints:Print(L["Waypoints cleared."])
		for i,v in ipairs(Queue) do
			v:Cancel()
		end
		Queue = {}
	end)

	initMeOnce = noop -- nil itself
end


local function LDB_OnClick(level, value)
	if db.datafeed and Waypoints.WaypointLDB then
		Dewdrop:AddLine(
			'text', L["Clear waypoint"],
			'arg1', Waypoints,
			'arg2', Queue,
			'func', function(self, Queue) Queue[1]:Cancel() end,
			'closeWhenClicked', true
		)
		if #Queue > 1 then
			Dewdrop:AddLine(
				'text', L["Move to end of queue"],
				'arg1', Waypoints,
				'arg2', Queue,
				'func', function(self, Queue) PushQueue(PopQueue(), 1) end,
				'closeWhenClicked', true
			)
		end
		if db.questPOI and priorityQuest then
			Dewdrop:AddLine(
				'text', L["Clear quest priority"],
				'func', function()
							priorityQuest = nil
							Waypoints:UpdateQuestPOIs()
						end,
				'closeWhenClicked', true
			)
		end
		Dewdrop:AddLine(
			'text', CLOSE,
			'func', closeFunc,
			'closeWhenClicked', true
		)
	end
end

local anchor
local function LDB_OnTooltipShow(owner)
	if db.datafeed and Waypoints.WaypointLDB then
		anchor = owner
		local waypoint = #Queue and Queue[1] or nil
		if not waypoint or not waypoint:IsValid() then
			return
		end
		GameTooltip:SetOwner(owner, "ANCHOR_BOTTOM")
		GameTooltip:AddLine(waypoint:ToString(), 1, 1, 1)
		GameTooltip:AddDoubleLine(L["Distance:"], format("%.2f meters", Waypoints.WaypointLDB.dist), 1, 0.82, 0, 1, 1, 1)
		if Waypoints.WaypointLDB.facing then
			GameTooltip:AddDoubleLine(L["Heading:"], format("%d\194\176", Waypoints.WaypointLDB.facing), 1, 0.82, 0, 1, 1, 1)
		end
		if Waypoints.WaypointLDB.tta then
			GameTooltip:AddDoubleLine(L["ETA:"], format("%s", Waypoints.WaypointLDB.tta), 1, 0.82, 0, 1, 1, 1)
		end
		GameTooltip:Show()
	end
end

local function LDB_OnLeave(owner)
	anchor = nil
	GameTooltip:Hide()
end

function Waypoints:InitializeLDB()
	if LDB and not self.WaypointLDB then
		self.WaypointLDB = LDB:NewDataObject("MapsterWaypoints", {
			type = "data source",
			icon = "Interface\\WorldMap\\WorldMap-Icon",
			text = "",
			iconR = 1, iconG = 1, iconB = 1, iconCoords = {0, 1, 0, 1},
			OnEnter = LDB_OnTooltipShow,
			OnLeave = LDB_OnLeave,
			OnClick	= function(owner, button)
				if Dewdrop:IsOpen(owner) then
					Dewdrop:Close()
				elseif Waypoints.WaypointLDB.waypointID then
					Dewdrop:Open(owner, "children", LDB_OnClick, "point", "TOP", "relativePoint", "BOTTOM")
				end
			end,
		})
	end
end

local function SetQuestWaypoint(questId)
	local zone = questPOIs[questId].zone
	local x, y = questPOIs[questId].x, questPOIs[questId].y

	local title = questPOIs[questId].title
	if not title then
		title = format(L["Quest %s POI"], questId)
	end
	local waypoint = #Queue and Queue[1] or nil
	if waypoint then PopQueue():Cancel() end

	local questWaypoint = NotePoint:new(zone, x, y, title)
	questPOIs[questId].waypointID = questWaypoint.waypointID
	questWaypoint.questPOI = true
	questWaypoint.questId = questId
	Waypoints:AddWaypoint(questWaypoint)
end

local function poi_OnClick(self, button)
	if not db.questPOI or not Notes:IsEnabled() or not Waypoints:IsEnabled() then return end
	if button == "RightButton" and IsControlKeyDown() then
		if self.parentName == "WatchFrameLines" then
			if not questPOIs[self.questId] or not questPOIs[self.questId].dist then
				return
			else
				priorityQuest = self.questId
				SetQuestWaypoint(self.questId)
			end
		end
	end
end

function Waypoints:UpdateClosestPOI()
	if not db.questPOI or not db.closestPOI or not Notes:IsEnabled() or not Waypoints:IsEnabled() then return end

	local waypoint = #Queue and Queue[1] or nil
	if waypoint and not waypoint.questPOI then return end
	wipe(closestQuest)

	for id, data in pairs(questPOIs) do
		if (not closestQuest.dist or questPOIs[id].dist < closestQuest.dist) then
			closestQuest.id = id
			closestQuest.title = questPOIs[id].title
			closestQuest.map = questPOIs[id].map
			closestQuest.floor = questPOIs[id].floor
			closestQuest.x = questPOIs[id].x
			closestQuest.y = questPOIs[id].y
			closestQuest.zone = questPOIs[id].zone
			closestQuest.dist = questPOIs[id].dist
		end
	end

	if closestQuest.dist and (not priorityQuest or closestQuest.id == priorityQuest) then
		-- Set a waypoint for this POI
		SetQuestWaypoint(closestQuest.id)
	end
	if not closestQuest.dist then
		local waypoint = #Queue and Queue[1] or nil
		if waypoint and waypoint.questPOI and (not priorityQuest or waypoint.questId ~= priorityQuest) then PopQueue():Cancel() end
	end
end

function Waypoints:UpdateQuestPOIs()
	if not db.questPOI or not Notes:IsEnabled() or not Waypoints:IsEnabled() then return end

	if WorldMapFrame:IsShown() then	-- we know there is a visible world map
		Waypoints.watchFrame:Show()
		return
	end

	wipe(questPOIs)
	local mapChange
	local numQuestWatches = GetNumQuestWatches()
	for i = 1, numQuestWatches do
		local questIndex = GetQuestIndexForWatch(i)

		if questIndex then
			local title, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID, startEvent = GetQuestLogTitle(questIndex)
			local numObjectives = GetNumQuestLeaderBoards(questIndex)
			if isComplete and isComplete < 0 then
				isComplete = nil
			end

			-- check filters
			local filterOK = true
			--[[if (isComplete and bit_band(WATCHFRAME_FILTER_TYPE, WATCHFRAME_FILTER_COMPLETED_QUESTS) ~= WATCHFRAME_FILTER_COMPLETED_QUESTS) then
				filterOK = nil
			elseif (bit_band(WATCHFRAME_FILTER_TYPE, WATCHFRAME_FILTER_REMOTE_ZONES) ~= WATCHFRAME_FILTER_REMOTE_ZONES and not LOCAL_MAP_QUESTS[questID]) then
				filterOK = nil
			end]]--

			if filterOK then
				-- Set a waypoint for this POI
				local map, floor = GetQuestWorldMapAreaID(questID)
				if map ~= GetCurrentMapAreaID() or floor ~= GetCurrentMapDungeonLevel() then
					mapChange = true
					SetMapByID(map)
					if floor ~= GetCurrentMapDungeonLevel() then SetDungeonMapLevel(floor) end
					if map == 683 and not QuestPOIGetIconInfo(questID) then
						SetMapByID(606)
						if floor ~= GetCurrentMapDungeonLevel() then SetDungeonMapLevel(floor) end
					elseif map == 748 and not QuestPOIGetIconInfo(questID) then
						SetMapByID(720)
						if floor ~= GetCurrentMapDungeonLevel() then SetDungeonMapLevel(floor) end
					elseif map == 770 and not QuestPOIGetIconInfo(questID) then
						SetMapByID(700)
						if floor ~= GetCurrentMapDungeonLevel() then SetDungeonMapLevel(floor) end
					end
				end

				local _, questx, questy, questObjective = QuestPOIGetIconInfo(questID)
				if map and floor and questx and questy then
					local zone = Notes:GetCurrentMapTexture()
					local dist = Notes:GetDistanceToPoint(questx, questy, zone)
					if zone and dist and dist > 0 then
						questPOIs[questID] = {}
						questPOIs[questID].title = title
						questPOIs[questID].map = map
						questPOIs[questID].floor = floor
						questPOIs[questID].x = questx
						questPOIs[questID].y = questy
						questPOIs[questID].zone = zone
						questPOIs[questID].dist = dist
					end
				end
			end
		end
	end
	if mapChange then SetMapToCurrentZone() end
	if priorityQuest and not questPOIs[priorityQuest] then
		local waypoint = #Queue and Queue[1] or nil
		if waypoint and waypoint.questPOI and waypoint.questId == priorityQuest then PopQueue():Cancel() end
		priorityQuest = nil
	end
	if priorityQuest and not db.closestPOI then
		local waypoint = #Queue and Queue[1] or nil
		if waypoint and waypoint.questPOI then SetQuestWaypoint(priorityQuest) end
	end
	self:UpdateClosestPOI()
end

function Waypoints:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["Waypoints"])
	Mapster:GetPrint(MODNAME)
end

function Waypoints:OnEnable()
	Notes = Mapster:GetModule("Notes")
	if Notes and Notes:IsEnabled() then
		if not Notes.externalDBs["Waypoints"] then
			Notes:RegisterNotesDatabase("Waypoints", Waypoint.mapDB, Waypoint)
		end
	else
		Mapster:SetModuleEnabled(MODNAME, false)
		return
	end

	LibGuild.RegisterCallback(self, "Disconnected", "LibGuild_Disconnected")
	self:RegisterComm("MpW", "OnCommReceived")
	self:RegisterComm("CGP", "OnCommReceived")

	self:RegisterEvent("PLAYER_ALIVE")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", "PLAYER_ALIVE")
	self:RegisterEvent("PLAYER_UNGHOST")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("MINIMAP_ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("CVAR_UPDATE")
	db.fontPath = LSM:Fetch("font", db.fontName)

	if not self.waypointUpdateFrame then
		self.waypointUpdateFrame = CreateFrame("Frame", "MapsterWaypoints_UpdateWaypoint")
		self.waypointUpdateFrame:Hide()
		self.lastWaypointUpdate = 0
	end
	if not self.velocityUpdateFrame then
		self.velocityUpdateFrame = CreateFrame("Frame", "MapsterWaypoints_UpdateVelocity")
		self.velocityUpdateFrame:Hide()
		self.lastVelocityUpdate = 0
	end
	initMeOnce()
	self:ZONE_CHANGED_NEW_AREA()

	if db.datafeed and LDB and not self.WaypointLDB then
		self:InitializeLDB()
	end
	if not self.watchFrame then
		self.watchFrame = CreateFrame("Frame", "MapsterWaypoints_WatchFrame_Update")
		self.watchFrame.lastUpdate = 0
		self.watchFrame:SetScript("OnUpdate", function(self, elapsed)
			self.lastUpdate = self.lastUpdate + elapsed
			if self.lastUpdate >= 1 then
				self:Hide()
				self.lastUpdate = 0
				Waypoints:UpdateQuestPOIs()
			end
		end)
	end
	self:SecureHook("QuestPOI_DisplayButton", function(parentName, buttonType, buttonIndex, questId)
		if db.questPOI then
			local buttonName = "poi"..parentName..buttonType.."_"..buttonIndex
			local poiButton = _G[buttonName]

			if not poiHooks[buttonName] then
				poiButton:HookScript("OnClick", poi_OnClick)
				poiButton:RegisterForClicks("AnyUp")
				poiHooks[buttonName] = true
			end
			if QUEST_POI_SWAP_BUTTONS[parentName] and not poiHooks["poi"..parentName.."_Swap"] then
				poiButton = QUEST_POI_SWAP_BUTTONS[parentName]
				poiButton:HookScript("OnClick", poi_OnClick)
				poiButton:RegisterForClicks("AnyUp")
				poiHooks["poi"..parentName.."_Swap"] = true
			end
		end
	end)
	--[[self:SecureHook("WorldMapFrame_UpdateQuests", function()
		if db.questPOI and db.closestPOI then
			self.watchFrame:Show()
		end
	end)]]--
	self:SecureHook("AddQuestWatch", function()
		if db.questPOI then Waypoints:UpdateQuestPOIs() end
	end)
	self:SecureHook("RemoveQuestWatch", function()
		if db.questPOI then Waypoints:UpdateQuestPOIs() end
	end)
	if db.questPOI then
		self:RegisterEvent("QUEST_LOG_UPDATE")
		self:RegisterEvent("QUEST_POI_UPDATE", "QUEST_LOG_UPDATE")
	end
	self:Refresh()
end

function Waypoints:OnDisable()
	if self.waypointUpdateFrame then
		self.waypointUpdateFrame:SetScript("OnUpdate", nil)
		self.waypointUpdateFrame:Hide()
	end
	if self.velocityUpdateFrame then
		self.velocityUpdateFrame:SetScript("OnUpdate", nil)
		self.velocityUpdateFrame:Hide()
	end
	self:Unhook("QuestPOI_DisplayButton")
	--self:Unhook("WorldMapFrame_UpdateQuests")
	self:Unhook("AddQuestWatch")
	self:Unhook("RemoveQuestWatch")

	LibGuild.UnregisterAll(self)

	if Notes and Notes.externalDBs["Waypoints"] then
		Notes:UnregisterNotesDatabase("Waypoints")
	end
	-- All hooks, timers and events are automatically disabled
	if waypointFrame then
		waypointFrame:Hide()
	end
	if self.watchFrame then
		self.watchFrame:Hide()
	end
	self:UnregisterEvent("QUEST_LOG_UPDATE")
	self:UnregisterEvent("QUEST_POI_UPDATE")
end

function Waypoints:Refresh()
	db = self.db.profile
	if not self:IsEnabled() then return end

	yards = (db.metric and L["%.0f m"] or L["%.0f yd"])
	if waypointFrame then
		waypointFrame:SetPoint("CENTER", UIParent, "CENTER", self.db.profile.waypointX or 0, self.db.profile.waypointY or 0)
	end
	if db.questPOI then
		self:RegisterEvent("QUEST_LOG_UPDATE")
		self:RegisterEvent("QUEST_POI_UPDATE", "QUEST_LOG_UPDATE")
		if db.closestPOI then
			self:UpdateQuestPOIs()
		end
	else
		self:UnregisterEvent("QUEST_LOG_UPDATE")
		self:UnregisterEvent("QUEST_POI_UPDATE")
	end
end

function Waypoints:QUEST_LOG_UPDATE()
	if db.questPOI then
		--self.watchFrame:Show()
		self:UpdateQuestPOIs()
	end
end

function Waypoints:PLAYER_ALIVE() -- We're locked, loaded, dead and not in a battleground
	if ReleasePoint and db.release and UnitIsDeadOrGhost("player") and select(2, IsInInstance()) ~= "pvp" and not IsActiveBattlefieldArena() then
		self:PLAYER_UNGHOST()
		local waypoint = ReleasePoint:new()
		PushQueue(waypoint)
		if not waypointFrame then
			createWaypointFrame()
		end
		self.waypointUpdateFrame:SetScript("OnUpdate", function(self, elapsed) Waypoints:UpdateWaypoint(elapsed) end)
		self.waypointUpdateFrame:Show()
	end
end

function Waypoints:PLAYER_UNGHOST()
	if #Queue > 0 then
		for i,v in ipairs(Queue) do
			if v.release then
				v:Cancel()
			end
		end
	end
end

function Waypoints:ZONE_CHANGED_NEW_AREA()
	if #Queue > 0 then
		self.waypointUpdateFrame:SetScript("OnUpdate", function(self, elapsed) Waypoints:UpdateWaypoint(elapsed) end)
		self.waypointUpdateFrame:Show()
	end
	self:MINIMAP_ZONE_CHANGED()
end

function Waypoints:PLAYER_REGEN_DISABLED() -- We've entered combat
	if waypointFrame and db.locked then -- If we've locked the waypointframe
		waypointFrame:EnableMouse(false)
	end
end


function Waypoints:PLAYER_REGEN_ENABLED() -- We've left combat
	if waypointFrame and db.locked then -- If we've locked the waypointframe
		waypointFrame:EnableMouse(true)
	end
end

function Waypoints:GetWaypoint()
	if #Queue > 0 and Queue[1] then
		local waypointNote = Queue[1]
		return waypointNote.zone, waypointNote.x, waypointNote.y
	else
		return nil
	end
end

function Waypoints:IsNoteWaypoint(id)
	checkParameter("IsNoteWaypoint", id, 1, "string")
	if #Queue > 0 then
		for i,v in ipairs(Queue) do
			if v.MatchNote and v:MatchNote(id) then
				return true
			end
		end
	end
	return false
end

function Waypoints:SetNoteAsWaypoint(zone, id)
	if not self:IsEnabled() and self.db then
		return
	end
	PushQueue(NotePoint:new(zone, id))
	if not waypointFrame then
		createWaypointFrame()
	end
    if(#Queue == 1) then
        self:Mapster_MapClosed()
    end
end

function Waypoints:SetPointAsWaypoint(x, y)
	if not self:IsEnabled() and self.db then
		return
	end
	local zone = Notes:GetCurrentMapTexture()
	self:AddWaypoint(NotePoint:new(zone, x, y, "Waypoint"))
end

function Waypoints:AddLHWaypoint(c, z, x, y, desc)
	if not self:IsEnabled() and self.db then
		return
	end
    local zone = select(z or GetCurrentMapZone(), GetMapZones(c or GetCurrentMapContinent()))
    zone = Notes:GetMapTexture(zone)
    self:AddWaypoint(NotePoint:new(zone, x/100, y/100, desc))
end

function Waypoints:AddRoutesWaypoint(z, id, desc)
	if not self:IsEnabled() and self.db then
		return
	end
	local x, y = getXY(id)
	self:AddWaypoint(NotePoint:new(z, x, y, desc))
end

function Waypoints:AddQuestWaypoint(zone, x, y, desc)
	if not self:IsEnabled() and self.db then
		return
	end
	self:AddWaypoint(NotePoint:new(zone, x, y, desc))
end

function Waypoints:AddWaypoint(waypoint)
	if not self:IsEnabled() and self.db then
		return
	end
	PushQueue(waypoint)

	local showOnMap = waypoint.showOnMap
	if type(showOnMap) == "function" then
		showOnMap = showOnMap(waypoint)
	end
	if showOnMap and db.minimapShow then
		waypoint:AddToMap()
	end
	if not waypointFrame then
		createWaypointFrame()
	end
	if(#Queue == 1) then
		self:Mapster_MapClosed()
	end
end


if not _G.SetWaypoint and _G.GetWaypoint and _G.ClearWaypoint then
	function _G.SetWaypoint(...)
		if not Waypoints:IsEnabled() and Waypoints.db then
			return
		end

		local first = (...)
		local zone, zoneName, x, y, text, metadata
		if type(first) == "number" then
			local continentID, zoneID
			continentID, zoneID, x, y, text, metadata = ...
			zoneName = select(zoneID, GetMapZones(continentID))
			zone = Notes:GetMapTexture(zoneName)
		else
			zone, x, y, text, metadata = ...
		end

		local wp = Waypoint:new(text or '')
		wp.x = x
		wp.y = y
		wp.zone = zone
		wp.metadata = metadata
		Waypoints:AddWaypoint(wp)
	end

	function _G.GetWaypoint()
		if not Waypoints:IsEnabled() and Waypoints.db then
			return
		end
		if #Queue >= 1 then
			return Queue[1].metadata
		end
	end

	function _G.ClearWaypoint()
		if not Waypoints:IsEnabled() and Waypoints.db then
			return
		end

		for i = #Queue, 1, -1 do
			Queue[i]:Cancel()
		end
	end
end

function Waypoints:CancelWaypoint(id)
	if not self:IsEnabled() and self.db then
		return
	end
	if #Queue > 0 then
		for i = 1, #Queue do
			if Queue[i].waypointID == id then
				Queue[i]:Cancel()
			end
		end
	end
	Notes:RefreshMap()
end

function Waypoints:Mapster_MapOpened()
	if Queue[1] then
		self.waypointUpdateFrame:SetScript("OnUpdate", nil)
		self.waypointUpdateFrame:Hide()
		waypointFrame.dist:SetText("-")
	end
end
function Waypoints:Mapster_MapClosed()
	if Queue[1] then
		self.waypointUpdateFrame:SetScript("OnUpdate", function(self, elapsed) Waypoints:UpdateWaypoint(elapsed) end)
		self.waypointUpdateFrame:Show()
	end
end

local waypointHitDistance = 5
function Waypoints:GetWaypointHitDistance()
	return waypointHitDistance
end

function Waypoints:SetWaypointHitDistance(v)
	if v >= 5 then -- limit to a minimum of 5 yards
		waypointHitDistance = v
	end
end
local inf = math.huge

local function GetThresholdPercentage(quality, ...)
	local n = select('#', ...)
	if n <= 1 then
		return GetThresholdPercentage(quality, 0, ... or 1)
	end

	local worst = ...
	local best = select(n, ...)

	if worst == best and quality == worst then
		return 0.5
	end

	if worst <= best then
		if quality <= worst then
			return 0
		elseif quality >= best then
			return 1
		end
		local last = worst
		for i = 2, n-1 do
			local value = select(i, ...)
			if quality <= value then
				return ((i-2) + (quality - last) / (value - last)) / (n-1)
			end
			last = value
		end

		local value = select(n, ...)
		return ((n-2) + (quality - last) / (value - last)) / (n-1)
	else
		if quality >= worst then
			return 0
		elseif quality <= best then
			return 1
		end
		local last = worst
		for i = 2, n-1 do
			local value = select(i, ...)
			if quality >= value then
				return ((i-2) + (quality - last) / (value - last)) / (n-1)
			end
			last = value
		end

		local value = select(n, ...)
		return ((n-2) + (quality - last) / (value - last)) / (n-1)
	end
end

local function GetThresholdColorTrivial(quality, ...)
	if quality ~= quality or quality == inf or quality == -inf then
		return 1, 1, 1
	end

	local percent = GetThresholdPercentage(quality, ...)

	if percent <= 0 then
		return 1, 0, 0
	elseif percent <= 0.5 then
		return 1, percent*2, 0
	elseif percent <= 0.75 then
		return 3 - percent*4, 1, 0
	elseif percent >= 1 then
		return 0.5, 0.5, 0.5
	else
		return percent*2 - 1.5, 2.5 - percent*2, percent*2 - 1.5
	end
end

local waycount = 53
local wayi = 0

local updateInterval = 0.05
function Waypoints:UpdateWaypoint(elapsed)
	self.lastWaypointUpdate = self.lastWaypointUpdate + elapsed
	if (self.lastWaypointUpdate >= updateInterval) then
		self.lastWaypointUpdate = 0

		local realZoneText = GetRealZoneText()
		local realZoneMap = Notes:GetMapTexture(realZoneText)
		local px, py = Notes:GetCurrentPlayerPosition()
		if px == nil then return end	-- Sinus: GetCurrentPlayerPosition() sometimes erroneously returns nil several times upon login, killing valid waypoints set up immediately at login time. It's better to ignore waypoints if player position seems bogus.
		local waypoint = #Queue and Queue[1] or nil

		if not waypoint then
			self.waypointUpdateFrame:SetScript("OnUpdate", nil)
			self.waypointUpdateFrame:Hide()
			waypointFrame:Hide()
			if db.datafeed and Waypoints.WaypointLDB then
				Waypoints.WaypointLDB.waypointID = nil
				Waypoints.WaypointLDB.text = ""
				Waypoints.WaypointLDB.facing = nil
				Waypoints.WaypointLDB.dist = nil
				Waypoints.WaypointLDB.icon = "Interface\\WorldMap\\WorldMap-Icon"
				Waypoints.WaypointLDB.iconR = 1
				Waypoints.WaypointLDB.iconG = 1
				Waypoints.WaypointLDB.iconB = 1
				Waypoints.WaypointLDB.iconCoords = texCoords["0"]
				if anchor then LDB_OnLeave(anchor) end
			end
			return
		end

		if not waypoint:IsValid() then
			--self:Print(L["Clearing up queue."])
			PopQueue():Cancel()
			waypoint = Queue[1]
			return
		end

		if not waypointFrame then
			return
		end

		if not waypointFrame:IsShown() then
			waypointFrame:Show()
		end

		local dist = waypoint:GetDistance()
		if not dist then
			dist = 0
		end
		if dist <= waypointHitDistance and waypoint:IsReady() then
			--self:Print("Waypoint hit")
			PopQueue() -- Don't call :Cancel() here because we are passing the waypoint in the event triggers
			self:SendMessage("MAPSTER_WAYPOINTS_WAYPOINT_HIT", waypoint)
			waypoint:Cancel()
			return
		end

		if dist <= 10 then
			if not waypointFrame.up then
				waypointFrame.tex:SetTexture(nil)
				waypointFrame.tex:SetTexture(path .."-UP")
				waypointFrame.tex:SetWidth(53)
				waypointFrame.tex:SetHeight(70)
				waypointFrame.up = true
			end
		else
			if waypointFrame.up then
				waypointFrame.tex:SetTexture(nil)
				waypointFrame.tex:SetTexture(path)
				waypointFrame.tex:SetWidth(56)
				waypointFrame.tex:SetHeight(42)
				waypointFrame.up = false
			end
		end
		local x, y = Notes:TranslateWorldMapPosition(waypoint.x, waypoint.y, waypoint.zone, realZoneMap)
		yards = L["%.0f m"]

		local rad, rad2
		if waypoint:IsReady() and x and y then
			if not waypointFrame.up then
				local diffX, diffY = x - (px or 0), (py or 0) - y
				rad = -math_atan2(diffX, diffY*2/3)
				local diffX2, diffY2 = x - (px or 0), y - (py or 0)
				rad2 = -math_atan2(diffY2*2/3, diffX2)
				local direction = rad - GetPlayerFacing()
				local val = math_floor(direction / (math_pi*2) * 108 + 0.5) % 108
				local col, row = (val % 9), math_floor(val / 9)
				if direction >= -10000 then
					waypointFrame.tex:SetTexCoord(col*56/512, (col+1)*56/512, row*42/512, (row+1)*42/512)
					if db.datafeed and self.WaypointLDB then
						self.WaypointLDB.icon = path
						self.WaypointLDB.iconCoords = texCoords[col..":"..row..":"]
					end
				end
			else
				waycount = waycount + 1
				if waycount == 54 then
					waycount = 0
				end
				local col, row = math_floor(waycount % 9), math_floor(waycount / 9)
				waypointFrame.tex:SetTexCoord(col*53/512, (col+1)*53/512, row*70/512, (row+1)*70/512)
				if db.datafeed and self.WaypointLDB then
					self.WaypointLDB.icon = path .."-UP"
					self.WaypointLDB.iconCoords = texCoords[col..":"..row..":UP"]
				end
			end
			waypointFrame.tex:SetVertexColor(GetThresholdColorTrivial(dist, 1000, 300, 100, 30, 0))
		else
			waypointFrame.tex:SetTexCoord(0, 0, 0, 0)
			waypointFrame.tex:SetVertexColor("c7c7cf")
		end
		if db.bearing then
			local bearing, facing = self:GetHeading(px, py, x, y)
			if bearing then
				local yardString = yards.." - %3d\194\176"
				waypointFrame.dist:SetText(format(yardString, dist, bearing))
				if db.datafeed and self.WaypointLDB then
					self.WaypointLDB.text = format(yards.." - %d\194\176", dist, facing)
					self.WaypointLDB.facing = facing
					self.WaypointLDB.dist = dist
					if (facing > 180) then facing = facing - 360 end
					local r, g, b = GetThresholdColorTrivial(abs(facing), 90, 60, 30, 0, 0)
					self.WaypointLDB.iconR = r
					self.WaypointLDB.iconG = g
					self.WaypointLDB.iconB = b
				end
			else
				waypointFrame.dist:SetText(format(yards, dist))
				if db.datafeed and self.WaypointLDB then
					self.WaypointLDB.text = format(yards, dist)
					self.WaypointLDB.facing = nil
					self.WaypointLDB.dist = dist
					self.WaypointLDB.icon = "Interface\\WorldMap\\WorldMap-Icon"
					self.WaypointLDB.iconR = 1
					self.WaypointLDB.iconG = 1
					self.WaypointLDB.iconB = 1
					self.WaypointLDB.iconCoords = texCoords["0"]
				end
			end
		else
			waypointFrame.dist:SetText(format(yards, dist))
		end
		waypointFrame.title:SetText(waypoint:ToString())
		if wayi == 0 then
			if rad2 then
				if not self.velocityUpdateFrame:IsShown() then
					self.velocityUpdateFrame:SetScript("OnUpdate", function(self, elapsed) Waypoints:OnUpdate(elapsed) end)
					self.velocityUpdateFrame:Show()
				end
				local speed, angle = self:GetCurrentVelocity()
				local disparity = math_cos(angle - rad2)
				if disparity < 0 then
					disparity = 0
				end
				local time = dist / speed / disparity
				waypointFrame.tta:SetText(FormatDurationShort(time, false, true))
				if db.datafeed and self.WaypointLDB then
					self.WaypointLDB.tta = FormatDurationShort(time, false, true)
				end
			else
				self.velocityUpdateFrame:SetScript("OnUpdate", nil)
				self.velocityUpdateFrame:Hide()
				waypointFrame.tta:SetText("")
				if db.datafeed and self.WaypointLDB then
					self.WaypointLDB.tta = nil
				end
			end
		end
		wayi = (wayi + 1) % 10
		if db.datafeed and self.WaypointLDB then
			self.WaypointLDB.waypointID = waypoint.waypointID
			if anchor then
				LDB_OnTooltipShow(anchor)
			end
		end
	end
end


-- Waypoints:OnCommReceived(prefix, message, distribution, sender) --
function Waypoints:OnCommReceived(prefix, message, distribution, sender)
	if not db.commActive then return end
	if prefix ~= "MpW" then return end
	if distribution ~= "GUILD" and distribution ~= "WHISPER" then return end

	local messagetype, zone, x, y = strmatch(message, "(%s+):(%s+):(%d+):(%d+)")

	-- Verify data received
	if type(messagetype) ~= "string" or messagetype ~= "POSITION" then return end
	if type(zone) ~= "string" then return end
	if type(x) ~= "number" or x < 0 or x > 1 then return end
	if type(y) ~= "number" or y < 0 or y > 1 then return end

	if (GuildPoint.watcher[sender] and #Queue > 0 and Queue[1].waypointID == sender) then
		GuildPoint.watcher[sender]:UpdateLocation(zone, x, y)
	end
end

function Waypoints:LibGuild_Disconnected(event, memberName)
	if GuildPoint.watcher[memberName] then
		GuildPoint.watcher[memberName]:Cancel()
	end
end

function Waypoints:RosterLib_UnitChanged(...)
	if select(6,...)[1] and select(1, ...)[1] then
		local party = PartyPoint.Roster[select(6, ...)]
		PartyPoint.Roster[select(1, ...)] = party
		party.partyID = select(1, ...)
	end

	if not select(1, ...) then
		PartyPoint.Roster[select(6, ...)]:Cancel()
	end
end

local currentYardWidth = 1000
function Waypoints:MINIMAP_ZONE_CHANGED()
	if not WorldMapFrame:IsShown() then
		SetMapToCurrentZone()
	end
	local zone = Notes:GetRealZoneTexture()
	currentYardWidth = Notes:GetZoneYardSize(zone)
	if not currentYardWidth then
		currentYardWidth = 1000
	end
end

local speed, direction = 0, 0
local updateVelocityInterval = 0.1
do
	local last_x, last_y, last_zone, last_time
	function Waypoints:OnUpdate(elapsed)
		self.lastVelocityUpdate = self.lastVelocityUpdate + elapsed
		if (self.lastVelocityUpdate >= updateVelocityInterval) then
			self.lastVelocityUpdate = 0

			local x, y, zone = Notes:GetCurrentPlayerPosition()
			if not x then
				return
			end
			if not last_x then
				last_x, last_y, last_zone = x, y, zone
				last_time = GetTime()
			end
			local time = GetTime()
			if zone ~= last_zone then
				last_x, last_y, last_zone, last_time = x, y, zone, time
				return
			end
			local d_x = x - last_x
			local d_y = (last_y - y) * 2/3
			local d_t = time - last_time
			local dist = (d_x*d_x + d_y*d_y) ^ 0.5 * currentYardWidth
			speed = dist / d_t
			direction = math_atan2(d_y, d_x)
			last_x, last_y, last_zone, last_time = x, y, zone, time
		end
	end

	function Waypoints:GetHeading(px, py, x, y)
		if not x or not y or not px or not py then
			return
		end
		local angle = math_pi - math_atan2(x - px, (y - py) * 2/3)
		local direction = math_deg(angle) % 360
		return direction, (math_deg(angle - -GetPlayerFacing()) % 360)
	end
end

function Waypoints:GetCurrentVelocity()
	return speed, direction
end

function Waypoints:CVAR_UPDATE(caller, event, cvar, value)
	if cvar == "ROTATE_MINIMAP" then
		rotateMinimap = value == "1"
	end
end


------------------------
-- WAYPOINT FUNCTIONS --
------------------------

Waypoint = {}
local mts = setmetatable({}, {
	__index = function(self, key)
		self[key] = {__index = key}
		return self[key]
	end,
	__mode='kv'
	}
)

function Waypoint:new(...)
	local t = setmetatable({}, mts[self])
	t:init(...)
	return t
end

local waypoint_db = Notes and Notes.externalDBs or nil
local Waypoint = Waypoint

Waypoint.mapDB = {}

-- Pool of frames to reuse
Waypoint.framePool = setmetatable({}, ({
	__index = function(self, n)
		if n == 0 then
			return table_remove(self) or {}
		end
	end})
)

local function getFrame()
	return Waypoint.framePool[0]
end

local function returnFrame(frame)
	table_insert(Waypoint.framePool, frame)
end


Waypoint.waypointID = -1	-- Unique ID
Waypoint.x = -1				-- Current X co-ord of waypoint
Waypoint.y = -1				-- Current Y co-ord of waypoint
Waypoint.zone = "Void"		-- Current map zone of waypoint
Waypoint.eZone = "Void"		-- Current english zone of waypoint
Waypoint.poiID = nil		-- Note ID reference
-- Graphics
Waypoint.innerZone = 10
Waypoint.icon = "Star 4"
Waypoint.iconTitle = "Waypoint"
Waypoint.pointType = "Way"
Waypoint.onMap = false
Waypoint.frame = getFrame()	-- Frame to show waypoint
Waypoint.colour = "00ff00"
Waypoint.showOnMap = false

function Waypoint:init(id)
	self.waypointID = id
end

-- This function will return the x and y position relative
-- to the map along with the zone and english zone name.
function Waypoint:Location()
	return self.x, self.y, self.zone, self.eZone
end

-- Use Mapster.Notes to find the distance
function Waypoint:GetDistance()
	return Notes:GetDistanceToPoint(self.x, self.y, self.zone)
end

-- This function returns the colour of the arrow determined by distance
function Waypoint:GetColour(dist)
    return GetThresholdColorTrivial(dist, 1000, 300, 100, 30, 0)
end

-- This function clears out values specific to the class
function Waypoint:Cancel()
	self:RemoveFromMap()
	self.waypointID = nil
	self.x = nil
	self.y = nil
	self.zone = nil
	self.eZone = nil
	self.poiID = nil
	self.icon = nil
	self.iconTitle = nil
	self.pointType = nil
	self.db = nil
	self.onMap = nil
	self.metadata = nil
	if db.questPOI and not self.questPOI then Waypoints:UpdateQuestPOIs() end
	returnFrame(self.frame)
end

-- Default ToString reverts to the ID
function Waypoint:ToString()
	return self.waypointID
end

-- Default check to see if waypoint is ready
function Waypoint:IsReady()
	return true
end

-- Check if waypoint is valid
function Waypoint:IsValid()
	return true
end

function Waypoint:MatchNote()
	return false
end

function Waypoint:AddToMap()
	local db = Waypoint.mapDB

	if not db[self.zone] then
		db[self.zone] = {}
	end
	-- We're adding 1 to the standard poi so as not to cover up existing notes
	self.poiID = round(self.x * 10000, 0) + round(self.y * 10000, 0) * 10000 + 1

	for i,v in pairs(db[self.zone]) do
		v.focus = nil
	end

	-- Work with duplicate waypoints
	if db[self.zone][self.poiID] then
		self.poiID = self.poiID + 1
	end

	db[self.zone][self.poiID] = {
			icon = self.icon,
			title = self:ToString() or self.iconTitle,
			point = self.pointType,
			focus = true;
	}
	self.onMap = true
	if Notes then
		Notes:ShowNote(self.zone, self.poiID, "Waypoints")
		Notes:MINIMAP_UPDATE_ZOOM()
	end
end

function Waypoint:RemoveFromMap()
	if not self.onMap then return end
	local db = Waypoint.mapDB

	if self.onMap and db and db[self.zone] and db[self.zone][self.poiID] then
		db[self.zone][self.poiID] = nil
	end

	if #Waypoints.Queue > 1 and db[self.zone] then
		for i,v in pairs(db[self.zone]) do
			if i == Waypoints.Queue[2].poiID then
				v.focus = true
			end
		end
	end
end

function Waypoint:ShowOnMap()
	return false
end

-- Waypoint Notes

function Waypoint:GetNoteIcon(zone, id, data)
	return data.icon
end

function Waypoint:GetRadius()
	return math.huge
end

function Waypoint:IsTracking()
	return false
end

function Waypoint:IsMiniNoteHidden(zone, id, data)
	return not data.focus
end


--------------------------
-- PARTYPOINT FUNCTIONS --
--------------------------

local Roster = LibStub("Roster-3.0")

PartyPoint = Waypoint:new()
--local PartyPoint = PartyPoint
PartyPoint.Roster = {}

PartyPoint.partyID = "noparty"
PartyPoint.continent = -1
PartyPoint.zoneRef = -1

-- Parse the id for the note and propogate the info
-- Possible inputs (zone, x, y) or (zone, id)
function PartyPoint:init(memberName)
	for unit in Roster:IterateRoster(false) do
		if string_find(string.lower(unit.name), string.lower(memberName)) then
			Waypoint.init(self, unit.name)
			self.partyID = unit.unitid
			self:UpdateZone()
			PartyPoint.Roster[self.partyID] = self
			break
		end
	end
end

-- Is this the same note as the waypoint?
function PartyPoint:MatchNote(memberName)
	return self.waypointID == memberName
end

function PartyPoint:GetDistance()
	self:UpdateZone()
	if self.continent == -1 then
		return -1
	end
	return Waypoint.GetDistance(self)
end


function PartyPoint:UpdateZone()
	if self.waypointID ~= Roster:GetUnitObjectFromUnit(self.partyID).name then -- sanity check
		self.partyID = Roster:GetUnitIDFromName(self.waypointID)
	end

	local x, y = 0, 0

	x, y = GetPlayerMapPosition(self.partyID)
	if x ~= 0 and y ~= 0 then -- Found them! Store the info
		self.continent = GetCurrentMapContinent()
		self.zoneRef = GetCurrentMapZone()
		self.x = x
		self.y = y
		self.zone = GetMapInfo()
		return
	end

	if self.continent ~= -1 then -- If we have a cached continent we are on
		SetMapZoom(self.continent)
		x, y = GetPlayerMapPosition(self.partyID) -- See if we're still on that continent
	end

	if x == 0 and y == 0 then -- We're no longer on this continent or we never knew which we were on
		for i=1,select("#", GetMapContinents()) do
			SetMapZoom(i) -- Cycle through the continents
			x, y = GetPlayerMapPosition(self.partyID)
			if x ~= 0 and y ~= 0 then
				self.continent = i
				break
			end
		end
		if x == 0 and y == 0 then
			self.continent = -1 -- noncontigious location
		end
	end
	if self.continent ~= -1 then -- We've found a continent they're on
		x, y = 0, 0 -- nil out continent position
		if self.zoneRef ~= -1 then -- We have a cached zone from last check
			SetMapZoom(self.continent, self.zoneRef)
			x, y = GetPlayerMapPosition(self.partyID)
		end
			if x == 0 and y == 0 then -- They're no longer in that zone or we never had a zone
			for i=1,select("#", GetMapZones(self.continent)) do
				SetMapZoom(self.continent, i)
				x, y = GetPlayerMapPosition(self.partyID)

				if x ~= 0 and y ~= 0 then -- Found them! Store the info
					self.zoneRef = i
					self.x = x
					self.y = y
					self.zone = GetMapInfo()
					break
				end
			end
		end
	end
end

-- Use the Partymember name as the Title
function PartyPoint:ToString()
	if self:IsReady() then
		return self.waypointID
	else
		return format(L["%s is not locateable."], self.waypointID)
	end
end

-- Is the member connected and in contiguous space?
function PartyPoint:IsValid()
	return self.partyID and self.partyID ~= "noparty" and UnitIsConnected(self.partyID) and self.continent ~= -1
end

function PartyPoint:IsReady()
	SetMapToCurrentZone()
	return self.continent == GetCurrentMapContinent()
end

function PartyPoint:Cancel()
	PartyPoint.Roster[self.partyID] = nil
	self.partyID, self.continent, self.zoneRef = nil, nil, nil
	Waypoint.Cancel(self)
end


--------------------------
-- GUILDPOINT FUNCTIONS --
--------------------------

local nameCache = {}

GuildPoint = Waypoint:new()
--local GuildPoint = GuildPoint

GuildPoint.watcher = {}
GuildPoint.pending = L["Pending update"]

-- Store the guildmember in the static class for later
-- reference. waypointID is the guildmember name.
function GuildPoint:init(memberName)
	checkParameter("GuildPoint:init", memberName, 1, "string")

	for name in LibGuild:GetIterator("NAME") do
		if string_find(string.lower(name), string.lower(memberName)) then
			Waypoint.init(self, name)
			if not GuildPoint.watcher[name] then
				GuildPoint.watcher[name] = self
			end
			break
		end
	end
--	Waypoint.init(self, -1)
end

-- Is this the same note as the waypoint?
function GuildPoint:MatchNote(memberName)
	return self.waypointID == memberName
end

function GuildPoint:UpdateLocation(zone, _x, _y)
	local eZone = LibGuild:GetEnglishZone(self.waypointID)
	local x, y = Notes:TranslateWorldMapPosition(_x, _y, Notes:GetMapTexture(eZone), Notes:GetMapTexture(LibGuild:GetZone(self.waypointID)))
	self.zone, self.x, self.y = zone, x, y
	--[[if not nameCache[zone] then
		nameCache[zone] = LibGuild:GetEnglishZone(self.waypointID)
	end
	self.x, self.y = Notes:TranslateWorldMapPosition(_x, _y, Notes:GetMapTexture(nameCache[zone]), Notes:GetMapTexture(LibGuild:GetZone(self.waypointID)))
	self.zone = zone]]--
end

function GuildPoint:GetDistance()
	return Waypoint.GetDistance(self)
end

-- Use the Guildmember name as the Title
function GuildPoint:ToString()
	if self:IsReady() then
		return self.waypointID
	else
		return self.pending
	end
end

function GuildPoint:IsReady()
	if self.zone ~= "Void" and self.x ~= -1 and self.y ~= -1 then
		return true
	else
		if not self.firstCheck then
			self.firstCheck = time()
		else
			local delay = time() - self.firstCheck - (LibGuild:GetNumOnline() / 5)
			if delay < 5 then
				self.pending = L["Waiting for location update"]
			elseif delay < 9 then
				self.pending = L["Guild Member not locateable"]
			else -- if they haven't updated in the normal update period
				self:Cancel()
			end
		end
		return false
	end
end

function GuildPoint:Cancel()
	GuildPoint.watcher[self.waypointID] = nil
	self.icon, self.db, self.data = nil
	Waypoint.Cancel(self)
end

-- Are we online?
function GuildPoint:IsValid()
	return (self.waypointID ~= -1) and LibGuild:IsMemberOnline(self.waypointID)
end


-------------------------
-- NOTEPOINT FUNCTIONS --
-------------------------

NotePoint = Waypoint:new()
--local NotePoint = NotePoint

NotePoint.db = nil
NotePoint.data = nil
NotePoint.showOnMap = true

-- Parse the id for the note and propogate the info
-- Possible inputs (zone, x, y) or (zone, id)
function NotePoint:init(zone, x, y, title)
	if not Notes:GetZoneFromMapTexture(zone) then
		error((L["Trying to set a waypoint with an unknown zone: %q"]):format(zone), 2)
	end
	local id
	if y then
		id = getID(x, y)
	else
		id = x
		x,y = getXY(id)
	end
	Waypoint.init(self, id..zone)
	if not title then
		local _
		self.zone, self.x, self.y, _, self.db, self.data = Notes:GetNote(zone, x, y, true)
	else
		self.zone, self.x, self.y, self.data = zone, x, y, {title = title}
	end
end

-- Is this the same note as the waypoint?
function NotePoint:MatchNote(id)
	return self.waypointID == id
end

-- Use the data.title as the name of the note, otherwise the zone
function NotePoint:ToString()
	if self.data then
		if not self.data.title then
			if self.data.icon then
				return self.data.icon
			end
		else
			return self.data.title
		end
	else
		return L["Waypoint"]
	end
end

function NotePoint:GetDistance()
	return Notes:GetDistanceToPoint(self.x, self.y, self.zone)
end

function NotePoint:Cancel()
	Waypoint.Cancel(self)
	self.db, self.data = nil, nil
end

function NotePoint:IsValid()
	if self.zone == "Void" then return false end
	if self.data.title or Notes:GetNote(self.zone, self.x, self.y) then
		return true
	else
		return false
	end
end

function NotePoint:ShowOnMap()
	return true
end


----------------------------
-- RELEASEPOINT FUNCTIONS --
----------------------------

ReleasePoint = Waypoint:new()
--local ReleasePoint = ReleasePoint

ReleasePoint.continent = -1
ReleasePoint.zoneRef = -1
ReleasePoint.release = true
ReleasePoint.icon = "Skull"
ReleasePoint.iconTitle = "Release"
ReleasePoint.pointType = "ReleasePoint"
ReleasePoint.showOnMap = true

L["ReleasePoint"] = true
L["ReleasePointNames"] = {
	"ReleasePoint", -- Insert cultural humour for death here.
	"Location of demise",
	"Compost Heap",
	"Ex-Parrot Location",
	"Worm Food Factory",
	"Note to self: Don't hug the bear",
	"This many steps away from the light...",
	"This is nature's way of telling you to slow down",
	"Well you're a silly sausage, aren't you?",
	"Well that was a mistake",
	"No no, Guard, turn, parry, dodge, spin and -then- thrust",
	"Despite all signs to the negative, it was obviously duck season",
	"You've had worse",
	"Well... you'll always have Paris",
	"Blame the Hunter",
	"Blame the Priest",
	"Blame Canada",
	"DARK IN HERE, ISN'T IT?",
	"Well you can't say you weren't asking for it...",
	"Bring a shovel",
	"A gnome would've dodged that.",
	"Stop crying, it doesn't help.",
	"Blame WoW",
	"If only they hadn't nerfed you last patch...",
	"Remember to put on a fresh pair of socks.",
	"Want me to freshen your drink while you're slacking?",
	"HLEP LOL",
	"Less slacking, more healing.",
	"Geez, get more hitpoints.",
	"WTS: [Flask of Titans]",
	"Was that a tauren or an elk?",
	"Hope you learned your lesson *this* time.",
	"D'oh!",
	"Blame the healer",
	"Don't blame Norway, it's such a nice place",
	"It's Rabbit season!",
	"A rogue would've used vanish.",
	"ZOMG GNAKED!",
	"Can't blame your keyboard this time.",
	"Just .. one .. more .. bah.",
	"Nothing a cup of coffee won't fix.",
	"Don't drink and die.",
	"This way to icecream.",
	"Beer. This. Way.",
	"Always Look on the Bright Side of Life",
	"Impeccable footwork",
}

-- Parse the id for the note and propogate the info
-- Possible inputs (zone, x, y) or (zone, id)
function ReleasePoint:init()
	Waypoint.init(self, Waypoints.db.profile.defaultmessages and L["ReleasePoint"] or L["ReleasePointNames"][math_random(#L["ReleasePointNames"])])
	self:UpdateZone()
end

function ReleasePoint:GetDistance()
	if self.continent == -1 then
		self:UpdateZone()
		if self.continent == -1 then
			return -1
		end
	end
	return Waypoint.GetDistance(self)
end


function ReleasePoint:UpdateZone()
	local x, y = 0, 0

	x, y = GetCorpseMapPosition()
	if x > 0 and y > 0 then -- Found them! Store the info
		self.continent = GetCurrentMapContinent()
		self.zoneRef = GetCurrentMapZone()
		self.x = x
		self.y = y
		self.zone = GetMapInfo()
		self:AddToMap()
		return
	end

	if self.continent ~= -1 then -- If we have a cached continent we are on
		SetMapZoom(self.continent)
		x, y = GetCorpseMapPosition() -- See if we're still on that continent
	end

	if x == 0 and y == 0 then -- We're no longer on this continent or we never knew which we were on
		for i=1,select("#",GetMapContinents()) do
			SetMapZoom(i) -- Cycle through the continents
			x, y = GetCorpseMapPosition()
			if x ~= 0 and y ~= 0 then
				self.continent = i
				break
			end
		end
	end
	if self.continent ~= -1 then -- We've found a continent they're on
		x, y = 0, 0 -- nil out continent position
		if self.zoneRef ~= -1 then -- We have a cached zone from last check
			SetMapZoom(self.continent, self.zoneRef)
			x, y = GetCorpseMapPosition()
		end
		if x == 0 and y == 0 then -- They're no longer in that zone or we never had a zone
			for i=1,select("#",GetMapZones(self.continent)) do
				SetMapZoom(self.continent, i)
				x, y = GetCorpseMapPosition()

				if x > 0 and y > 0 then -- Found them! Store the info
					self.zoneRef = i
					self.x = x
					self.y = y
					self.zone = GetMapInfo()
					self:AddToMap()
					break
				end
			end
		end
	end
end

-- Use the member name as the Title
function ReleasePoint:ToString()
	return self.waypointID
end

-- Is the member connected and in contiguous space?
function ReleasePoint:IsValid()
	return UnitIsDeadOrGhost("player") and self.waypointID ~= -1
end

function ReleasePoint:IsReady()
	return self.x > 0 and self.y > 0
end

function ReleasePoint:Cancel()
	self.continent, self.zoneRef = nil, nil
	return Waypoint.Cancel(self)
end
