--[[
MapsterEnhanced - ZoneInfo (Sindragosa)

Based on Cartographer
Credits: ckknight
]]

local Mapster = LibStub("AceAddon-3.0"):GetAddon("Mapster")
local L = LibStub("AceLocale-3.0"):GetLocale("Mapster")

local MODNAME = "ZoneInfo"
local ZoneInfo = Mapster:NewModule(MODNAME, "AceHook-3.0")

local Tourist = LibStub("LibTourist-3.0-Enhanced")


local string_format = string.format
local string_gsub = string.gsub
local table_concat = table.concat
local table_insert = table.insert
local table_wipe = table.wipe
local GetCurrentMapContinent = GetCurrentMapContinent
local GetProfessions = GetProfessions
local GetProfessionInfo = GetProfessionInfo
local GetSpellInfo = GetSpellInfo
local herbalismSpell, miningSpell, fishingSpell
local display

local db
local defaults = {
	profile = {
		showHerbalismLevels = false,
		showMiningLevels = false,
		showFishingLevels = false,
	}
}

local optGetter, optSetter
do
	local mod = ZoneInfo
	function optGetter(info)
		local key = info[#info]
		return db[key]
	end

	function optSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end
end

local options
local function getOptions()
	if not options then
		options = {
			type = "group",
			name = L["ZoneInfo"],
			arg = MODNAME,
			get = optGetter,
			set = optSetter,
			args = {
				intro = {
					order = 1,
					type = "description",
					name = L["The ZoneInfo module enhances the area information when hovering over a zone, and will show the levels of the zone, the instances in the zone, their levels, and the group size the instance is made for (e.g. 5-man, 25-man)."],
				},
				enabled = {
					order = 2,
					type = "toggle",
					width = "full",
					name = L["Enable ZoneInfo"],
					get = function() return Mapster:GetModuleEnabled(MODNAME) end,
					set = function(info, value) Mapster:SetModuleEnabled(MODNAME, value) end,
				},
				showHerbalismLevels = {
					order = 3,
					type = "toggle",
					name = L["Show Herbalism Levels"],
				},
				showMiningLevels = {
					order = 4,
					type = "toggle",
					name = L["Show Mining Levels"],
				},
				showFishingLevels = {
					order = 5,
					type = "toggle",
					name = L["Show Fishing Levels"],
				},
			},
		}
	end

	return options
end

function ZoneInfo:OnInitialize()
	self.db = Mapster.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile

	self:SetEnabledState(Mapster:GetModuleEnabled(MODNAME))
	Mapster:RegisterModuleOptions(MODNAME, getOptions, L["ZoneInfo"])
end

function ZoneInfo:OnEnable()
	if not display then
		display = CreateFrame("Frame", "Mapster_ZoneInfoFrame", WorldMapFrame)

		display.text = WorldMapFrameAreaFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
		local text = display.text
		local font, size = GameFontHighlightLarge:GetFont()
		text:SetFont(font, size, "OUTLINE")
		text:SetPoint("TOP", WorldMapFrameAreaDescription, "BOTTOM", 0, -5)
		text:SetWidth(1024)
	end
	-- Put Zone text in center
	ZoneTextFrame:ClearAllPoints()
	ZoneTextFrame:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 512)
	WorldMapFrameAreaFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 6)
	WorldMapFrameAreaLabel:SetHeight(40)
	herbalismSpell = GetSpellInfo(2366)
	miningSpell = GetSpellInfo(2575)
	fishingSpell = GetSpellInfo(7620)
	display:Show()
	display.text:Show()
	if not self.hookedOnUpdate then
		self:SecureHookScript(WorldMapButton, "OnUpdate", "WorldMapButton_OnUpdate")
		self.hookedOnUpdate = true
	end
end

function ZoneInfo:OnDisable()
	if self.hookedOnUpdate then
		self:Unhook(WorldMapButton, "OnUpdate")
		self.hookedOnUpdate = nil
	end
	display.text:SetText("")
	lastZone = nil
	display.text:Hide()
	display:Hide()
	WorldMapFrameAreaLabel:SetTextColor(1, 1, 1)
	WorldMapFrameAreaFrame:SetFrameLevel(WorldMapButton:GetFrameLevel() + 1)
end

function ZoneInfo:Refresh()
	db = self.db.profile
	if not self:IsEnabled() then return end
end

local lastZone	-- So we don't keep processing zones in every Update
local t = {}	-- Text to display stored here
function ZoneInfo:WorldMapButton_OnUpdate(self, elapsed)
	--if not self:IsEnabled() then return end
	if not display then
		return
	end
	if not WorldMapDetailFrame:IsShown() or not WorldMapFrameAreaLabel:IsShown() then
		display.text:SetText("")
		lastZone = nil
		return
	end

	-- Under Attack is used during events like the Naxxramas opening event.
	local underAttack
	local zone = WorldMapFrameAreaLabel:GetText()
	local description = WorldMapFrameAreaDescription:GetText()
	if zone then
		zone = string_gsub(WorldMapFrameAreaLabel:GetText(), " ?|cff.+$", "")
		if description then
			underAttack = true
			description = string_gsub(WorldMapFrameAreaDescription:GetText(), " ?|cff.+$", "")
		end
	end
	-- Set the text to white and hide the zone info if we're hovering over
	-- Kalimdor or Eastern Kingdoms on the old world continent map (I think)
	if GetCurrentMapContinent() == 0 then
		local c1, c2, c3, c4, c5 = GetMapContinents()
		if zone == c1 or zone == c2 or zone == c5 or zone == c4 then
			WorldMapFrameAreaLabel:SetTextColor(1, 1, 1)
			display.text:SetText("")
			return
		end
	end

	if not zone or not Tourist:IsZoneOrInstance(zone) then
		zone = WorldMapFrame.areaName
	end
	WorldMapFrameAreaLabel:SetTextColor(1, 1, 1)

	if zone and (Tourist:IsZoneOrInstance(zone) or Tourist:DoesZoneHaveInstances(zone)) then
		WorldMapFrameAreaLabel:SetTextColor(Tourist:GetFactionColor(zone))
		if description then
			WorldMapFrameAreaDescription:SetTextColor(Tourist:GetFactionColor(zone))
		else
			WorldMapFrameAreaDescription:SetTextColor(1, 1, 1)
		end
		local low, high = Tourist:GetLevel(zone)

		if db.showHerbalismLevels then
			local minHerb = Tourist:GetHerbalismLevel(zone)
			local herbalismSkillText

			if minHerb then
				local prof1, prof2 = GetProfessions()
				if prof1 or prof2 then
					local herbalism
					local skillName, skillIcon, skillLevel = GetProfessionInfo(prof1)
					if skillName == herbalismSpell then
						herbalism = prof1
					else
						skillName, skillIcon, skillLevel = GetProfessionInfo(prof2)
						if skillName == herbalismSpell then
							herbalism = prof2
						end
					end
					if herbalism then
						local r, g, b = 1, 1, 0
						local r1, g1, b1 = 1, 0, 0
						if minHerb < skillLevel then
							r1, g1, b1 = 0, 1, 0
						end
						herbalismSkillText = string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, skillName, r1 * 255, g1 * 255, b1 * 255, minHerb)
					end
				else
					minHerb = nil
				end
			end
		end
		if db.showMiningLevels then
			local minMining = Tourist:GetMiningLevel(zone)
			local miningSkillText

			if minMining then
				local prof1, prof2 = GetProfessions()
				if prof1 or prof2 then
					local mining
					local skillName, skillIcon, skillLevel = GetProfessionInfo(prof1)
					if skillName == miningSpell then
						mining = prof1
					else
						skillName, skillIcon, skillLevel = GetProfessionInfo(prof2)
						if skillName == miningSpell then
							mining = prof2
						end
					end
					if mining then
						local r, g, b = 1, 1, 0
						local r1, g1, b1 = 1, 0, 0
						if minMining < skillLevel then
							r1, g1, b1 = 0, 1, 0
						end
						miningSkillText = string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, skillName, r1 * 255, g1 * 255, b1 * 255, minMining)
					end
				else
					minMining = nil
				end
			end
		end
		if db.showFishingLevels then
			local minFish = Tourist:GetFishingLevel(zone)
			local fishingSkillText

			if minFish then
				local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
				if fishing then
					local skillName, skillIcon, skillLevel = GetProfessionInfo(fishing)
					if skillName == fishingSpell then
						local r, g, b = 1, 1, 0
						local r1, g1, b1 = 1, 0, 0
						if minFish < skillLevel then
							r1, g1, b1 = 0, 1, 0
						end
						fishingSkillText = string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, skillName, r1 * 255, g1 * 255, b1 * 255, minFish)
					end
				else
					minFish = nil
				end
			end
		end

		-- Set the difficulty colour of the zone, based on our current level.
		if low > 0 and high > 0 then
			local r, g, b = Tourist:GetLevelColor(zone)
			local levelText
			if low == high then
				levelText = string_format(" |cff%02x%02x%02x[%d]|r", r * 255, g * 255, b * 255, high)
			else
				levelText = string_format(" |cff%02x%02x%02x[%d-%d]|r", r * 255, g * 255, b * 255, low, high)
			end
			local groupSize = Tourist:GetInstanceGroupSize(zone)
			local sizeText = ""
			if groupSize > 0 then
				sizeText = " |cffffffff" .. string_format(L["%d-man"], groupSize)
				local altGroupSize = Tourist:GetInstanceAltGroupSize(zone)
				if altGroupSize > 0 then
					sizeText = sizeText .. string_format(L[" and "] .. L["%d-man"], altGroupSize)
				end
			end
			WorldMapFrameAreaLabel:SetText(zone .. levelText .. sizeText .. "|r")
		elseif zone then
			WorldMapFrameAreaLabel:SetText(zone)
		end

		-- List the instances in the zone if it has any.
		if Tourist:DoesZoneHaveInstances(zone) then
			if lastZone ~= zone then
				-- Set lastZone so we don't keep grabbing this info in every Update.
				lastZone = zone
				table_insert(t, string_format("|cffffff00%s:|r", L["Instances"]))
				-- Iterate over the instance list and insert them into t[]
				for instance in Tourist:IterateZoneInstances(zone) do
					local complex = Tourist:GetComplex(instance)
					local low, high = Tourist:GetLevel(instance)
					local r1, g1, b1 = Tourist:GetFactionColor(instance)
					local r2, g2, b2 = Tourist:GetLevelColor(instance)
					local groupSize = Tourist:GetInstanceGroupSize(instance)
					local altGroupSize = Tourist:GetInstanceAltGroupSize(instance)
					local name = instance
					if complex then
						name = complex .. " - " .. instance
					end
					if low == high then
						if (altGroupSize > 0) and (groupSize > 0) then
							table_insert(t, string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r " .. L["%d-man"] .. L[" and "] .. L["%d-man"], r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high, groupSize, altGroupSize))
						elseif groupSize > 0 then
							table_insert(t, string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r " .. L["%d-man"], r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high, groupSize))
						else
							table_insert(t, string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, high))
						end
					else
						if (altGroupSize > 0) and (groupSize > 0) then
							table_insert(t, string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r " .. L["%d-man"] .. L[" and "] .. L["%d-man"], r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high, groupSize, altGroupSize))
						elseif groupSize > 0 then
							table_insert(t, string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r " .. L["%d-man"], r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high, groupSize))
						else
							table_insert(t, string_format("|cff%02x%02x%02x%s|r |cff%02x%02x%02x[%d-%d]|r", r1 * 255, g1 * 255, b1 * 255, name, r2 * 255, g2 * 255, b2 * 255, low, high))
						end
					end
				end

				if db.showHerbalismLevels and minHerb and herbalismSkillText then
					table_insert(t, herbalismSkillText)
				end
				if db.showMiningLevels and minMining and miningSkillText then
					table_insert(t, miningSkillText)
				end
				if db.showFishingLevels and minFish and fishingSkillText then
					table_insert(t, fishingSkillText)
				end
				-- OK, add all of the info from t[] into the zone info!
				display.text:SetText(table_concat(t, "\n"))
				-- Reset t[], ready for the next run.
				table_wipe(t)
			end
		else
			-- If the zone has no instances
			if db.showHerbalismLevels and minHerb and herbalismSkillText then
				table_insert(t, herbalismSkillText)
			end
			if db.showMiningLevels and minMining and miningSkillText then
				table_insert(t, miningSkillText)
			end
			if db.showFishingLevels and minFish and fishingSkillText then
				table_insert(t, fishingSkillText)
			end
			if #t == 0 then
				display.text:SetText("")
			end
			table_wipe(t)
			lastZone = nil
		end
	elseif not zone then
		-- If we couldn't identify a valid zone at all, just blank it out.
		lastZone = nil
		display.text:SetText("")
	end
end
